package org.exoplatform.services.cms.actions;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.PathNotFoundException;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.NodeTypeIterator;
import javax.jcr.nodetype.NodeTypeManager;
import javax.jcr.version.OnParentVersionAction;

import org.exoplatform.container.component.ComponentPlugin;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.cms.CmsService;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.config.RepositoryConfigurationException;
import org.exoplatform.services.jcr.core.ManageableRepository;
import org.exoplatform.services.jcr.core.nodetype.ExtendedNodeTypeManager;
import org.exoplatform.services.jcr.core.nodetype.NodeTypeValue;
import org.exoplatform.services.jcr.core.nodetype.PropertyDefinitionValue;
import org.picocontainer.Startable;

public class ActionServiceContainerImpl implements ActionServiceContainer,
    Startable {

  private static final String ACTIONABLE = "exo:actionable";
  private static final String ACTION = "exo:action";

  private RepositoryService repositoryService_;
  private CmsConfigurationService cmsConfigService_;
  private String workspace_;
  private CmsService cmsService_;
  private Collection<ComponentPlugin> actionPlugins = new ArrayList<ComponentPlugin>();

  public ActionServiceContainerImpl(InitParams params,
      RepositoryService repositoryService, CmsService cmsService,
      CmsConfigurationService cmsConfigService) throws Exception {
    repositoryService_ = repositoryService;
    cmsConfigService_ = cmsConfigService;
    cmsService_ = cmsService;
    workspace_ = params.getValueParam("workspace").getValue();
  }

  public Collection<String> getActionPluginNames() {
    Collection<String> actionPluginNames = new ArrayList<String>();
    for (Iterator iter = actionPlugins.iterator(); iter.hasNext();) {
      ComponentPlugin plugin = (ComponentPlugin) iter.next();
      actionPluginNames.add(plugin.getName());
    }
    return actionPluginNames;
  }

  public ActionPlugin getActionPlugin(String actionsServiceName) {
    for (Iterator iter = actionPlugins.iterator(); iter.hasNext();) {
      ComponentPlugin plugin = (ComponentPlugin) iter.next();
      if (plugin.getName().equals(actionsServiceName))
        return (ActionPlugin) plugin;
    }
    return null;
  }

  public void createActionType(String actionTypeName,
      String parentActionTypeName, String executable, List<String> variableNames, boolean isMoveType)
      throws Exception {
    NodeTypeValue nodeTypeValue = new NodeTypeValue();
    nodeTypeValue.setName(actionTypeName);

    List<String> superTypes = new ArrayList<String>();
    superTypes.add(parentActionTypeName);
    if(isMoveType)
      superTypes.add("exo:move");
    nodeTypeValue.setDeclaredSupertypeNames(superTypes);        

    List propDefs = new ArrayList();
    PropertyDefinitionValue propDef = null;
    for (Iterator iter = variableNames.iterator(); iter.hasNext();) {
      String variableName = (String) iter.next();
      propDef = createPropertyDef(variableName);
      propDefs.add(propDef);
    }

    propDef = createPropertyDef(getActionPluginForActionType(
        parentActionTypeName).getExecutableDefinitionName());
    List defaultValues = new ArrayList();
    defaultValues.add(executable);
    propDef.setDefaultValueStrings(defaultValues);
    propDef.setMandatory(false);
    propDefs.add(propDef);

    nodeTypeValue.setDeclaredPropertyDefinitionValues(propDefs);
    nodeTypeValue.setDeclaredChildNodeDefinitionValues(new ArrayList());
    ExtendedNodeTypeManager ntmanager = repositoryService_.getRepository()
        .getNodeTypeManager();
    ntmanager.registerNodeType(nodeTypeValue,
        ExtendedNodeTypeManager.IGNORE_IF_EXISTS);
  }

  private PropertyDefinitionValue createPropertyDef(String name) {
    PropertyDefinitionValue def = new PropertyDefinitionValue();
    def.setName(name);
    def.setRequiredType(PropertyType.STRING);
    def.setMandatory(false);
    def.setMultiple(false);
    def.setReadOnly(false);
    def.setAutoCreate(false);
    def.setOnVersion(OnParentVersionAction.IGNORE);
    def.setValueConstraints(new ArrayList());
    def.setDefaultValueStrings(new ArrayList());
    return def;
  }

  public Collection<NodeType> getCreatedActionTypes() throws Exception {
    Collection<NodeType> createsActions = new ArrayList<NodeType>();
    Session session = null;
    try {
      session = getSystemSession(workspace_);
    } catch (RepositoryException re) {
      return createsActions;
    }
    NodeTypeManager ntmanager = session.getWorkspace().getNodeTypeManager();
    NodeTypeIterator iter = ntmanager.getAllNodeTypes();
    while (iter.hasNext()) {
      NodeType nt = (NodeType) iter.next();
      String name = nt.getName();
      if (extendsBaseAction(nt) && !isAbstractType(name)) {
        createsActions.add(nt);
      }
    }
    return createsActions;
  }

  private boolean isAbstractType(String name) {
    for (Iterator iter = actionPlugins.iterator(); iter.hasNext();) {
      ComponentPlugin plugin = (ComponentPlugin) iter.next();
      if (plugin.getName().equals(name))
        return true;
    }
    return false;
  }

  public boolean extendsBaseAction(NodeType nt) {
    NodeType[] superTypes = nt.getSupertypes();
    for (int i = 0; i < superTypes.length; i++) {
      NodeType type = superTypes[i];
      if ("exo:action".equals(type.getName()))
        return true;
    }
    return false;
  }

  private Session getSystemSession(String workspace)
      throws RepositoryException, RepositoryConfigurationException {
    ManageableRepository jcrRepository = repositoryService_.getRepository();
    return jcrRepository.getSystemSession(workspace);
  }

  public ActionPlugin getActionPluginForActionType(String actionTypeName) {
    for (Iterator iter = actionPlugins.iterator(); iter.hasNext();) {
      ComponentPlugin plugin = (ComponentPlugin) iter.next();
      String actionServiceName = plugin.getName();
      ActionPlugin actionService = getActionPlugin(actionServiceName);
      if (actionService.isActionTypeSupported(actionTypeName)
          || actionServiceName.equals(actionTypeName))
        return actionService;
    }
    return null;
  }

  public Node getAction(Node node, String actionName) throws Exception {
    try {
      return node.getNode(actionName);
    } catch (PathNotFoundException ex) {
      return null;
    }
  }

  public boolean hasActions(Node node) throws Exception {
    return node.isNodeType(ACTIONABLE);
  }

  public List<Node> getActions(Node node) throws Exception {
    return getActions(node, null);
  }

  public List<Node> getActions(Node node, String lifecyclePhase)
      throws Exception {
    List<Node> actions = new ArrayList<Node>();
    
    NodeIterator iter = node.getNodes();
    while (iter.hasNext()) {
      Node tmpNode = iter.nextNode();
      if (tmpNode.isNodeType(ACTION)
          && (lifecyclePhase == null ||
              lifecyclePhase.equals(tmpNode.getProperty("exo:lifecyclePhase")
                .getString()))) {
        actions.add(tmpNode);
      }
    }

    return actions;
  }

  public void removeAction(Node node, String actionName) throws Exception {
    if (node.isNodeType(ACTIONABLE)) {
      Node node2Remove = node.getNode(actionName);
      String actionTypeName = node2Remove.getPrimaryNodeType().getName();
      node2Remove.remove();
      node.save();
      for (Iterator iter = actionPlugins.iterator(); iter.hasNext();) {
        ComponentPlugin plugin = (ComponentPlugin) iter.next();
        String actionServiceName = plugin.getName();
        ActionPlugin actionService = getActionPlugin(actionServiceName);
        if (actionService.isActionTypeSupported(actionTypeName)) {
          actionService.removeAction(actionName);
        }
      }
    }
  }

  public void addAction(Node node, String type, Map mappings) throws Exception {
    if (!node.isNodeType(ACTIONABLE)) {
      node.addMixin(ACTIONABLE);
    }
    cmsService_.storeNode(type, node, mappings);
    node.save();

    String srcWorkspace = node.getSession().getWorkspace().getName();
    String srcPath = node.getPath();
    for (Iterator iter = actionPlugins.iterator(); iter.hasNext();) {
      ComponentPlugin plugin = (ComponentPlugin) iter.next();
      String actionServiceName = plugin.getName();
      ActionPlugin actionService = getActionPlugin(actionServiceName);
      if (actionService.isActionTypeSupported(type)) {
        actionService.addAction(type, srcWorkspace, srcPath, mappings);
      }
    }
  }

  public void executeAction(String userId, Node node, String actionName,
      Map variables) throws Exception {
    if (node.isNodeType(ACTIONABLE)) {
      Node actionNode = node.getNode(actionName);
      String actionTypeName = actionNode.getPrimaryNodeType().getName();
      for (Iterator iter = actionPlugins.iterator(); iter.hasNext();) {
        ComponentPlugin plugin = (ComponentPlugin) iter.next();
        String actionServiceName = plugin.getName();
        ActionPlugin actionPlugin = getActionPlugin(actionServiceName);
        if (actionPlugin.isActionTypeSupported(actionTypeName)) {
          actionPlugin.executeAction(userId, actionNode, variables);
        }
      }
    }
  }

  public void addPlugin(ComponentPlugin plugin) {
    actionPlugins.add(plugin);
  }

  public ComponentPlugin removePlugin(String pluginName) {
    return null;
  }

  public Collection getPlugins() {
    return actionPlugins;
  }

  public void start() {
    try {
      for (Iterator iter = actionPlugins.iterator(); iter.hasNext();) {
        BaseActionPlugin plugin = (BaseActionPlugin) iter.next();
        plugin.importPredefinedActionsInJcr();
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void stop() {
  }

}
