/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core;


import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ProjectScope;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.preferences.IEclipsePreferences;
import org.eclipse.core.runtime.preferences.IScopeContext;
import org.exoplatform.eclipse.core.runtime.IExoProjectRuntimeProperties;
import org.exoplatform.eclipse.internal.core.PortletProject;

import java.util.ArrayList;
import java.util.List;

/**
 * The plugin runtime class for the eXo core plugin. At this time this class does not take
 * advantage of any life cycle requests issued by the platform. It is here just to provide
 * convenient logging methods.
 * 
 * <p>
 * Like all plug-in runtime classes (subclasses of <code>Plugin</code>), this class is
 * automatically instantiated by the platform when the plug-in gets activated. Clients
 * must not attempt to instantiate plug-in runtime classes directly.
 * </p>
 * 
 * @author Hatim Khan
 */
public class ExoCorePlugin extends Plugin
{
  public static final String PLUGIN_ID = "org.exoplatform.eclipse.core"; //$NON-NLS-1$
  public static final String PORTLET_BUILDER_ID = PLUGIN_ID + ".portletbuilder"; //$NON-NLS-1$
  public static final String PORTLET_NATURE_ID = PLUGIN_ID + ".portletnature"; //$NON-NLS-1$
  public static final String PORTAL_NATURE_ID = PLUGIN_ID + ".portalnature"; //$NON-NLS-1$  
  public static final boolean ALLOW_MULTIPLE_OUTPUT = false;

  private static final String EMPTY_STRING = ""; //$NON-NLS-1$
  private static ExoCorePlugin mPlugin;

  /**
   * The constructor. It needs to be public, so that the platform can instantiate it. No
   * other clients should directly do that.
   */
  public ExoCorePlugin()
  {
    super();
    mPlugin = this;
  }

  /**
   * Returns the shared instance.
   * 
   * @return plugin the eXo core plugin instance
   */
  public static ExoCorePlugin getDefault()
  {
    return mPlugin;
  }

  /**
   * Use this method to gain access to an implemenation for the IPortletProject interface.
   * This method actually caches an instance of the IPorltetProject for each project. It
   * does that by associating this instance as a session property for the project
   * resource.
   * 
   * @param pProject the resource project to wrap
   * @return the portlet project that represents this resource project
   * @throws CoreException if something wrong happens while trying to get an implemenation
   *           of the IPortletProject interface.
   */
  public static IPortletProject createPortletProject(IProject pProject) throws CoreException
  {
    IPortletProject pp = (IPortletProject) pProject.getSessionProperty(IExoProjectRuntimeProperties.PORTLET_PROJECT);

    if (pp == null)
    {
      pp = new PortletProject(pProject);
      pProject.setSessionProperty(IExoProjectRuntimeProperties.PORTLET_PROJECT, pp);
    }

    return pp;
  }

  /**
   * Returns all open projects that have a portlet nature. Notice that if you need to have
   * the IPortletProject then you should call createPortletProject() on each of the
   * returned projects returned from this method.
   * 
   * @param pMonitor to report progress
   * @return an array of opened projects that have a portlet nature
   * @throws CoreException when we have a problem
   * @throws InterruptedException when the monitor cancel the operation
   */
  public static IProject[] getAllOpenPortletProjects(IProgressMonitor pMonitor) throws CoreException,
      InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    // get all projects
    IProject[] allProjects = ResourcesPlugin.getWorkspace().getRoot().getProjects();

    if (allProjects == null)
    {
      return new IProject[0];
    }

    List portletProjects = new ArrayList();

    pMonitor.beginTask("", allProjects.length * 10);
    for (int i = 0, sizei = allProjects.length; i < sizei; i++)
    {
      IProject project = allProjects[i];
      if (!project.exists())
      {
        pMonitor.worked(10);

        continue;
      }
      if (!project.isOpen())
      {
        pMonitor.worked(10);

        continue;
      }
      if (!project.hasNature(ExoCorePlugin.PORTLET_NATURE_ID))
      {
        pMonitor.worked(10);

        continue;
      }
      if (pMonitor.isCanceled())
      {
        throw new InterruptedException();
      }

      portletProjects.add(project);
      pMonitor.worked(10);
    }
    pMonitor.done();

    return (IProject[]) portletProjects.toArray(new IProject[portletProjects.size()]);
  }

  /**
   * A way to get the project-scope preferences node given a project instance.
   * 
   * @param pProject the project to get its project scope preferences node
   * @return the project-scope preferences node
   */
  public static IEclipsePreferences getProjectScopePreferences(IProject pProject)
  {
    IScopeContext projectScope = new ProjectScope(pProject);
    IEclipsePreferences projectNode = projectScope.getNode(ExoCorePlugin.PLUGIN_ID);

    return projectNode;
  }

  /**
   * Writes an error message to the plugin's log
   * 
   * @param pMessage the text to write to the log
   * @param pException the exception to print its stack trace to the log
   */
  public static void logError(String pMessage, Throwable pException)
  {
    IStatus status = createErrorStatus(pMessage, pException);
    getDefault().getLog().log(status);
  }

  /**
   * Writes an IStatus to the plugin's log
   * 
   * @param pStatus the status that contains the information to log
   */
  public static void log(IStatus pStatus)
  {
    getDefault().getLog().log(pStatus);
  }

  /**
   * Writes an error message to the plugin's log. The error message text itself is
   * hardcoded in this method.
   * 
   * @param pException the exception to print its stack trace to the log
   */
  public static void logError(Throwable pException)
  {
    //this message is intentionally not internationalized, as an exception may
    // be due to the resource bundle itself
    String message = EMPTY_STRING;
    if (pException != null)
    {
      message = pException.getMessage();
    }
    getDefault().getLog().log(createErrorStatus("Internal error logged from eXo core plugin : " + message, pException)); //$NON-NLS-1$
  }

  /**
   * Returns a new <code>IStatus</code> for this plugin that encapsulates the given
   * error message and the given exception.
   * 
   * @param pMessage the message to encapsulate in this status
   * @param pException the exception to encapsulate in this status
   * @return return the status object populated with the given message and exception
   */
  public static IStatus createErrorStatus(String pMessage, Throwable pException)
  {
    if (pMessage == null)
    {
      pMessage = EMPTY_STRING;
    }

    return new Status(IStatus.ERROR, PLUGIN_ID, 0, pMessage, pException);
  }
}