/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.core.exoservices;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.exoplatform.eclipse.core.common.Utils;
import org.exoplatform.eclipse.core.xml.AttributeAwareReflectionConverter;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.converters.Converter;
import com.thoughtworks.xstream.io.xml.DomDriver;

/**
 * The main usage of this class is to load Configuration instances from xml, and to do
 * that you simply have to call loadConfiguration() method. In case of more advance usage,
 * this class also provides static methods to acquire a parser that understands how to
 * parse the Configuration object from an xml. If you already have an instance of the
 * XStream parser, then use the method getParser(XStream) to configure the parser to
 * understand Configuration elements. Otherwise, you should obtain a new instance of the
 * parser using getParser().
 * 
 * @author Hatim Khan
 */
public class ConfigurationParser
{

  /**
   * Given a URL (such as, file:c:\configuration.xml, this method returns a list of
   * Configuration instances. Notice that the first instance is the direct result of
   * parsing the given URL, however, since this configuration can have one or more import
   * elements, this method returns all of the configurations declared in the import
   * elements. However, this method does not go more then one level deep, so if one of the
   * imported configuration has by itself another declared import, this method simply
   * ignores it.
   * 
   * @param pURL the file to parse
   * @param pResolver a reslover that can understand the value of the import tag
   * @return please see the method description
   * @throws IOException when things go wrong
   */
  public static List loadConfiguration(URL pURL, IImportConfigurationURLResolver pResolver,
      IProgressMonitor pMonitor) throws IOException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    pMonitor.beginTask("", IProgressMonitor.UNKNOWN);

    List result = new ArrayList();

    pMonitor.subTask("scanning services configuration: " + Utils.shortenString(pURL.toString()));
    InputStream inputStream = pURL.openStream();
    String xml = Utils.readStream(inputStream);
    XStream xstream = ConfigurationParser.getParser();
    Configuration configuration = (Configuration) xstream.fromXML(xml);
    configuration.updateAppearIn(pURL.getFile());
    result.add(configuration);
    pMonitor.worked(1);

    List urls = configuration.getImportConfigurations();
    if (urls != null)
    {
      Iterator iterator = urls.iterator();
      while (iterator.hasNext())
      {
        String unresolvedURL = (String) iterator.next();
        URL resolvedURL = pResolver.resolveImportURL(unresolvedURL, pURL);
        if (resolvedURL == null) continue;
        pMonitor.subTask(": " + Utils.shortenString(resolvedURL.toString()));
        inputStream = resolvedURL.openStream();
        xml = Utils.readStream(inputStream);
        configuration = (Configuration) xstream.fromXML(xml);
        configuration.updateAppearIn(resolvedURL.getFile());
        result.add(configuration);
        pMonitor.worked(1);
      }
    }

    pMonitor.done();
    return result;
  }

  /**
   * Use this method if you want to obtain a new instance of the XStream parser.
   * 
   * @return a parser that understands how to parse the Configuration object from an xml
   */
  public static XStream getParser()
  {
    XStream xstream = new XStream(new DomDriver());
    return getParser(xstream);
  }

  /**
   * Use this method to configure an existing xstream parser to understand the
   * Configuration elements.
   * 
   * @param xstream the parser to configure so that it understands Configuration elements
   * @return a parser that understands how to parse the Configuration object from an xml
   */
  public static XStream getParser(XStream xstream)
  {
    Converter converter = new AttributeAwareReflectionConverter(xstream);
    xstream.registerConverter(converter);
    xstream.alias("configuration", Configuration.class);
    xstream.alias("global-param", GlobalParam.class);
    xstream.alias("init-param", ValuesParam.class);
    xstream.alias("service-configuration", ServiceConfiguration.class);
    xstream.alias("value-param", ValueParam.class);
    xstream.alias("values-param", ValuesParam.class);
    xstream.alias("properties-param", PropertiesParam.class);
    xstream.alias("object-param", PropertiesParam.class);

    xstream.addImplicitCollection(Configuration.class, "globalParam", "global-param", GlobalParam.class);
    xstream.addImplicitCollection(GlobalParam.class, "initParams", "init-param", ValuesParam.class);
    xstream.addImplicitCollection(ValuesParam.class, "values", "value", String.class);
    xstream.addImplicitCollection(Configuration.class, "serviceConfigurations", "service-configuration", ServiceConfiguration.class);
    xstream.addImplicitCollection(ServiceConfiguration.class, "valueParams", "value-param", ValueParam.class);
    xstream.addImplicitCollection(ServiceConfiguration.class, "valuesParams", "values-param", ValuesParam.class);
    xstream.addImplicitCollection(ServiceConfiguration.class, "propertiesParams", "properties-param", PropertiesParam.class);
    xstream.addImplicitCollection(PropertiesParam.class, "properties", "property", Property.class);
    xstream.addImplicitCollection(ServiceConfiguration.class, "objectParams", "object-param", ObjectParam.class);
    xstream.addImplicitCollection(ObjectParam.class, "properties", "property", Property.class);

    xstream.addImplicitCollection(Configuration.class, "importConfigurations", "import", String.class);
    xstream.addImplicitCollection(Configuration.class, "removeServices", "remove-configuration", String.class);

    return xstream;
  }
}