/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.filesprovider;


import org.eclipse.core.runtime.IPath;

import java.io.InputStream;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

/**
 * This class exposes a zip file entries as a structure. It extends the
 * existing functionality of the ZipFileStructureProvider in one aspect. It
 * allows you to rename certain files or directories before exposing the
 * structure. However, this class has a limitation. The limitation is that can
 * not rename two directories if one of them is the ancestor of the other.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class ZipFileWithRenameStructureProvider extends ZipFileStructureProvider
  implements IImportWithRenameStructureProvider
{
  public static final String CLASS_VERSION = "$Id: ZipFileWithRenameStructureProvider.java,v 1.1 2004/04/19 03:45:47 hatimk Exp $"; //$NON-NLS-1$
  private Map mReverseMap = new HashMap();
  private Map mRenameMap = new HashMap();

  /**
   * The constructor method. Creates a
   * <code>ZipFileWithRenameStructureProvider</code>, which will operate on
   * the passed zip file.
   *
   * @param pSourceFile the zip file to walk through
   */
  public ZipFileWithRenameStructureProvider(ZipFile pSourceFile)
  {
    super(pSourceFile);
  }

  /**   
   * @see org.exoplatform.eclipse.core.operation.IImportWithRenameStructureProvider#addRenameEntry(org.eclipse.core.runtime.IPath,
   *      org.eclipse.core.runtime.IPath)
   */
  public void addRenameEntry(IPath pRenameFrom, IPath pRenameTo)
  {
    ZipEntry name = new ZipEntry(pRenameFrom.toString());
    ZipEntry rename = new ZipEntry(pRenameTo.toString());
    mRenameMap.put(name.getName(), rename.getName());
    mReverseMap.put(rename.getName(), name);
  }

  /**   
   * @see org.exoplatform.eclipse.core.filesprovider.ZipFileStructureProvider#getChildren(java.lang.Object)
   */
  public List getChildren(Object pElement)
  {
    ZipEntry e = getOriginalEntry((ZipEntry) pElement);
    List l = super.getChildren(e);

    return replaceList(l);
  }

  /**   
   * @see org.exoplatform.eclipse.core.filesprovider.ZipFileStructureProvider#getContents(java.lang.Object)
   */
  public InputStream getContents(Object pElement)
  {
    ZipEntry e = getOriginalEntry((ZipEntry) pElement);

    return super.getContents(e);
  }

  /**   
   * @see org.exoplatform.eclipse.core.filesprovider.ZipFileStructureProvider#getFullPath(java.lang.Object)
   */
  public String getFullPath(Object pElement)
  {
    return super.getFullPath(pElement);
  }

  /**   
   * @see org.exoplatform.eclipse.core.filesprovider.ZipFileStructureProvider#getLabel(java.lang.Object)
   */
  public String getLabel(Object pElement)
  {
    return super.getLabel(pElement);
  }

  /**   
   * @see org.exoplatform.eclipse.core.filesprovider.ZipFileStructureProvider#isFolder(java.lang.Object)
   */
  public boolean isFolder(Object pElement)
  {
    ZipEntry e = getOriginalEntry((ZipEntry) pElement);

    return super.isFolder(e);
  }

  /**   
   * Looks to see if the given entry is marked as to be renamed, if so the
   * renamed one is returned, while at the same time the original one is
   * remembered.
   *
   * @param pEntry the entry to inspect for possible renaming
   *
   * @return the renamed if needed otherwise the original is returned
   */
  private ZipEntry getReplacedEntry(ZipEntry pEntry)
  {
    String fixedName = replaceName(pEntry);
    ZipEntry fixedEntry = pEntry;

    if (!fixedName.equals(pEntry.getName()))
    {
      fixedEntry = new ZipEntry(fixedName);
      mReverseMap.put(fixedName, pEntry);
    }

    return fixedEntry;
  }

  /**   
   * Looks to see if the given entry is an already renamed entry, if so the
   * original one is returned.
   *
   * @param pEntry the entry to inspect if it has been renamed
   *
   * @return the original entry
   */
  private ZipEntry getOriginalEntry(ZipEntry pEntry)
  {
    ZipEntry original = (ZipEntry) mReverseMap.get(pEntry.getName());
    ZipEntry originalEntry = pEntry;

    if (original != null)
    {
      originalEntry = original;
    }

    return originalEntry;
  }

  /**   
   * Given a list of zip entries, replace all that need renaming.
   *
   * @param pRawList a list that might contain zip entries to be renamed
   *
   * @return the list after renaming is done
   */
  private List replaceList(List pRawList)
  {
    List fixedList = new ArrayList();
    if (pRawList == null)
    {
      return fixedList;
    }

    Iterator i = pRawList.iterator();
    while (i.hasNext())
    {
      ZipEntry ze = (ZipEntry) i.next();
      ZipEntry rename = getReplacedEntry(ze);
      fixedList.add(rename);
    }

    return fixedList;
  }

  /**   
   * Given a raw entry, detect if part of it needs to be renamed, if so then
   * rename that part.
   *
   * @param pRawEntry an entry to inspect for renaming
   *
   * @return the entry with renaming
   */
  private String replaceName(ZipEntry pRawEntry)
  {
    String result = pRawEntry.getName();

    Set keys = mRenameMap.keySet();
    Iterator i = keys.iterator();
    while (i.hasNext())
    {
      String key = (String) i.next();
      if (result.startsWith(key))
      {
        result = result.substring(key.length());
        result = ((String) mRenameMap.get(key)) + result;

        break;
      }
    }

    return result;
  }
}
