/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.launching;


import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.launching.IRuntimeClasspathEntry;
import org.exoplatform.eclipse.core.ExoCorePlugin;

import java.util.ArrayList;
import java.util.List;

/**
 * The central access point for launching support. This class manages
 * ExoTomcatRuntimeConfigurationProviders contributed through the
 * <code>"org.exoplatform.eclipse.core.ExoTomcatRuntimeConfigurationProviders"</code>
 * extension point
 * 
 * <p>
 * This class provides static methods only; it is not intended to be instantiated or
 * subclassed by clients.
 * </p>
 * 
 * @author Hatim Khan
 */
public class ExoLaunching
{
  private static ITomcatRuntimeConfigurationProvider[] mTomcatConfigProviders = null;
  private static IExoTomcatRuntimeConfigurationProvider[] mExoTomcatConfigProviders = null;
  public static final String TOMCAT_EXTENSION_POINT_ID = ExoCorePlugin.PLUGIN_ID
      + ".tomcatRuntimeConfigurationProviders"; //$NON-NLS-1$  
  public static final String EXO_TOMCAT_EXTENSION_POINT_ID = ExoCorePlugin.PLUGIN_ID
      + ".exoTomcatRuntimeConfigurationProviders"; //$NON-NLS-1$  
  public static final String EXO_TOMCAT_CONFIGURATION_TYPE_ID = "org.exoplatform.eclipse.core.launching.ExoTomcatLaunchConfigurationType"; //$NON-NLS-1$

  /**
   * This class contains only static methods, and is not intended to be instantiated.
   */
  private ExoLaunching()
  {
  }

  private static synchronized void initializeExoTomcatRuntimeConfigurationProviders()
  {
    IExtensionPoint extensionPoint = Platform.getExtensionRegistry().getExtensionPoint(EXO_TOMCAT_EXTENSION_POINT_ID);
    IConfigurationElement[] configs = extensionPoint.getConfigurationElements();
    MultiStatus status = new MultiStatus(ExoCorePlugin.PLUGIN_ID, IStatus.OK,
        "An error occured while trying to read all available eXo Tomcat Runtime Configuration Providers", //$NON-NLS-1$
        null);
    mExoTomcatConfigProviders = new IExoTomcatRuntimeConfigurationProvider[configs.length];

    for (int i = 0; i < configs.length; i++)
    {
      try
      {
        IExoTomcatRuntimeConfigurationProvider eXoTomcatConfigProvider = (IExoTomcatRuntimeConfigurationProvider) configs[i].createExecutableExtension("class"); //$NON-NLS-1$
        mExoTomcatConfigProviders[i] = eXoTomcatConfigProvider;
      }
      catch (CoreException e)
      {
        status.add(e.getStatus());
      }
    }
    if (!status.isOK())
    {
      //only happens on a CoreException
      ExoCorePlugin.log(status);

      //cleanup null entries in mTomcatConfigs
      List temp = new ArrayList(mExoTomcatConfigProviders.length);
      for (int i = 0; i < mExoTomcatConfigProviders.length; ++i)
      {
        if (mExoTomcatConfigProviders[i] != null)
        {
          temp.add(mExoTomcatConfigProviders[i]);
        }
        mExoTomcatConfigProviders = new IExoTomcatRuntimeConfigurationProvider[temp.size()];
        mExoTomcatConfigProviders = (IExoTomcatRuntimeConfigurationProvider[]) temp.toArray(mExoTomcatConfigProviders);
      }
    }
  }

  private static synchronized void initializeTomcatRuntimeConfigurationProviders()
  {
    IExtensionPoint extensionPoint = Platform.getExtensionRegistry().getExtensionPoint(TOMCAT_EXTENSION_POINT_ID); //$NON-NLS-1$
    IConfigurationElement[] configs = extensionPoint.getConfigurationElements();
    MultiStatus status = new MultiStatus(ExoCorePlugin.PLUGIN_ID, IStatus.OK,
        "An error occured while trying to read all available Tomcat Runtime Configuration Providers", null); //$NON-NLS-1$
    mTomcatConfigProviders = new ITomcatRuntimeConfigurationProvider[configs.length];

    for (int i = 0; i < configs.length; i++)
    {
      try
      {
        ITomcatRuntimeConfigurationProvider tomcatConfigProvider = (ITomcatRuntimeConfigurationProvider) configs[i].createExecutableExtension("class"); //$NON-NLS-1$
        mTomcatConfigProviders[i] = tomcatConfigProvider;
      }
      catch (CoreException e)
      {
        status.add(e.getStatus());
      }
    }
    if (!status.isOK())
    {
      //only happens on a CoreException
      ExoCorePlugin.log(status);

      //cleanup null entries in mTomcatConfigs
      List temp = new ArrayList(mTomcatConfigProviders.length);
      for (int i = 0; i < mTomcatConfigProviders.length; i++)
      {
        if (mTomcatConfigProviders[i] != null)
        {
          temp.add(mTomcatConfigProviders[i]);
        }
        mTomcatConfigProviders = new ITomcatRuntimeConfigurationProvider[temp.size()];
        mTomcatConfigProviders = (ITomcatRuntimeConfigurationProvider[]) temp.toArray(mTomcatConfigProviders);
      }
    }
  }

  /**
   * Returns the list of registered eXo Tomcat runtime configuration providers. eXo Tomcat
   * runtime configuration providers are registered via
   * <code>"org.exoplatform.eclipse.core.ExoTomcatRuntimeConfigurationProviders"</code>
   * extension point. Returns an empty list if there are no registered eXo Tomcat runtime
   * configuration providers.
   * 
   * @return the list of registered Tomcat Runtime Configuration Providers
   */
  public static IExoTomcatRuntimeConfigurationProvider[] getExoTomcatRuntimeConfigurationProviders()
  {
    if (mExoTomcatConfigProviders == null)
    {
      initializeExoTomcatRuntimeConfigurationProviders();
    }

    return mExoTomcatConfigProviders;
  }

  /**
   * Returns the eXo Tomcat runtime Configuration provider that has the given id. Null if
   * no provider is found.
   * 
   * @return the configuration provider or null if not found
   */
  public static IExoTomcatRuntimeConfigurationProvider getDefaultExoTomcatRuntimeConfigurationProvider()
  {
    IExoTomcatRuntimeConfigurationProvider[] exoTomcatConfigProviders = getExoTomcatRuntimeConfigurationProviders();

    if ((exoTomcatConfigProviders == null) || (exoTomcatConfigProviders.length < 1))
    {
      return null;
    }

    return exoTomcatConfigProviders[0];
  }

  /**
   * Returns the eXo Tomcat runtime Configuration provider that has the given id. Null if
   * no provider is found.
   * 
   * @param pId the id (String) of the configuration provider
   * 
   * @return the configuration provider or null if not found
   */
  public static IExoTomcatRuntimeConfigurationProvider getExoTomcatRuntimeConfigurationProvider(String pId)
  {
    IExoTomcatRuntimeConfigurationProvider[] exoTomcatConfigProviders = getExoTomcatRuntimeConfigurationProviders();

    if ((exoTomcatConfigProviders == null) || (pId == null))
    {
      return null;
    }

    for (int i = 0, sizei = exoTomcatConfigProviders.length; i < sizei; ++i)
    {
      if (exoTomcatConfigProviders[i].getConfigurationId().compareToIgnoreCase(pId) == 0)
      {
        return exoTomcatConfigProviders[i];
      }
    }

    return null;
  }

  /**
   * Returns the eXo Tomcat runtime Configuration provider that has the given name. Null
   * if no provider is found.
   * 
   * @param pName the name (String) of the configuration provider
   * 
   * @return the configuration provider or null if not found
   */
  public static IExoTomcatRuntimeConfigurationProvider getExoTomcatRuntimeConfigurationProviderWithName(
      String pName)
  {
    IExoTomcatRuntimeConfigurationProvider[] exoTomcatConfigProviders = getExoTomcatRuntimeConfigurationProviders();

    if ((exoTomcatConfigProviders == null) || (pName == null))
    {
      return null;
    }

    for (int i = 0, sizei = mExoTomcatConfigProviders.length; i < sizei; ++i)
    {
      if (exoTomcatConfigProviders[i].getConfigurationName().compareToIgnoreCase(pName) == 0)
      {
        return exoTomcatConfigProviders[i];
      }
    }

    return null;
  }

  /**
   * Returns the list of registered Tomcat runtime configuration providers. Tomcat runtime
   * configuration providers are registered via
   * <code>"org.exoplatform.eclipse.core.tomcatRuntimeConfigurationProviders"</code>
   * extension point. Returns an empty list if there are no registered Tomcat runtime
   * configuration providers.
   * 
   * @return the list of registered Tomcat Runtime Configuration Providers
   */
  public static ITomcatRuntimeConfigurationProvider[] getTomcatRuntimeConfigurationProviders()
  {
    if (mTomcatConfigProviders == null)
    {
      initializeTomcatRuntimeConfigurationProviders();
    }

    return mTomcatConfigProviders;
  }

  /**
   * Returns true if the given id matches the id of any of the configuration types
   * provided by this plugin
   * 
   * @param pId the id of the configuration type in question
   * 
   * @return true if the given id matches the id of any of the configuration types
   *         provided by this plugin
   */
  public static boolean isExoConfigurationType(String pId)
  {
    boolean result = false;

    if (pId == null)
    {
      return false;
    }
    if (EXO_TOMCAT_CONFIGURATION_TYPE_ID.equals(pId))
    {
      return true;
    }

    return result;
  }

  /**
   * Returns an array that contains both of the given IRuntimeClasspathEntry arrays.
   * 
   * @param pEntries1 the first array
   * @param pEntries2 the second array
   * 
   * @return an array that is the result of adding the first array to the second array
   */
  public static IRuntimeClasspathEntry[] appendRuntimeClasspathEntries(IRuntimeClasspathEntry[] pEntries1,
      IRuntimeClasspathEntry[] pEntries2)
  {
    List result = new ArrayList();

    if (pEntries1 != null)
    {
      for (int i = 0, sizei = pEntries1.length; i < sizei; ++i)
      {
        result.add(pEntries1[i]);
      }
    }

    if (pEntries2 != null)
    {
      for (int i = 0, sizei = pEntries2.length; i < sizei; ++i)
      {
        result.add(pEntries2[i]);
      }
    }

    IRuntimeClasspathEntry[] resultarray = new IRuntimeClasspathEntry[result.size()];

    return (IRuntimeClasspathEntry[]) result.toArray(resultarray);
  }

  /**
   * Returns all the open portlet projects that are listed in the launch configuration
   */
  public static IJavaProject[] getPortletProjects(ILaunchConfiguration pConfig, IProgressMonitor pMonitor)
      throws CoreException
  {
    List result = new ArrayList();

    // lets get the name of the portlet projects that are specified in the configuration
    List projects = pConfig.getAttribute(IExoLaunchingConfigurationConstants.ATTR_PORTLET_PROJECT_LIST, (List) null);

    if (projects == null)
    {
      return new IJavaProject[0];
    }

    IProject[] openProjects = null;
    try
    {
      openProjects = ExoCorePlugin.getAllOpenPortletProjects(pMonitor);
      if (openProjects == null)
      {
        return new IJavaProject[0];
      }
    }
    catch (InterruptedException e)
    {
      // the user wants to cancel so simply return empty list for now
      return new IJavaProject[0];
    }

    // now for each returned project check if its name appears in the configuration too
    for (int i = 0, sizei = openProjects.length; i < sizei; ++i)
    {
      String name = openProjects[i].getName();
      if (projects.contains(name))
      {
        IProject project = openProjects[i];
        if ((project != null) && project.exists() && project.isOpen())
        {

          IJavaProject javaproject = JavaCore.create(project);
          result.add(javaproject);
        }
      }
    }

    return (IJavaProject[]) result.toArray(new IJavaProject[result.size()]);
  }

}