/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.launching;


import org.eclipse.core.runtime.Path;

import java.util.ArrayList;
import java.util.List;

/**
 * This class is representation of a VM argument. Each argument has a key, and one or more
 * values (or no values at all). A typical usage of this class is to create it with a key
 * and then add values as needed and then add the whole class to a VM argument container.
 * For example, lets say that you want to represent
 * "-Dcatalina.home=C:\exo-workspace\tomcat5.0" using the class. The code should look
 * something like this:
 * 
 * <pre>
 * VMArgument arg1 = new VMArgument(&quot;-Dcatalina.home&quot;, VMArgumentType.parse(&quot;c=multi,p=path,s=assign&quot;));
 * arg1.appendValue(&quot;C:\\exo-workspace\\tomcat5.0&quot;);
 * </pre>
 * 
 * @author Hatim Khan
 * 
 * @see org.exoplatform.eclipse.core.launching.VMArgumentType
 */
public class VMArgument implements Cloneable
{
  private String mKey;
  private List mValues;
  private int mAllowedSize;
  private VMArgumentType mType;

  /**
   * The constructor method.
   * 
   * @param pKey
   * @param pType
   */
  public VMArgument(String pKey, VMArgumentType pType)
  {
    mKey = pKey;
    mType = pType;
    mValues = new ArrayList();
    mAllowedSize = pType.getParameterCount();
  }

  /**
   * Given a value, remove it from the existing values (if it exists), then add it again,
   * but at the end of the values list. Notice that if the type used to create this
   * VMArgument instance indicates that the parameters are of type 'PATH'. Then this
   * method will convert the value given to an OS relevant path string.
   * 
   * @param pValue the value to append to the list
   */
  public void appendValue(String pValue)
  {
    if (pValue == null)
    {
      return;
    }

    String value = prepareValue(pValue);

    mValues.remove(value);
    mValues.add(value);

    if (mValues.size() > mAllowedSize)
    {
      mValues.remove(value);
    }
  }

  /**
   * Given a list of values, one by one remove them (if they exist), then add them again,
   * but at the end of the existing values list. Notice that if the type used to create
   * this VMArgument instance indicates that the parameters are of type 'PATH'. Then this
   * method will convert the value given to an OS relevant path string.
   * 
   * @param pValues the values to append to the list
   */
  public void appendValues(String[] pValues)
  {
    if (pValues == null)
    {
      return;
    }

    for (int i = 0, sizei = pValues.length; i < sizei; ++i)
    {
      appendValue(pValues[i]);
    }
  }

  /**
   * Given a value, remove it from the existing values (if it exists), then add it again,
   * but at the begining of the values list. Notice that if the type used to create this
   * VMArgument instance indicates that the parameters are of type 'PATH'. Then this
   * method will convert the value given to an OS relevant path string.
   * 
   * @param pValue the value to prepend to the list
   */
  public void prependValue(String pValue)
  {
    if (pValue == null)
    {
      return;
    }

    String value = prepareValue(pValue);

    mValues.remove(value);
    mValues.add(0, value);

    if (mValues.size() > mAllowedSize)
    {
      mValues.remove(value);
    }
  }

  /**
   * Given a list of values, one by one remove them (if they exist), then add them again,
   * but at the begining of the existing values list. Notice that if the type used to
   * create this VMArgument instance indicates that the parameters are of type 'PATH'.
   * Then this method will convert the value given to an OS relevant path string.
   * 
   * @param pValues the values to prepend to the list
   */
  public void prependValues(String[] pValues)
  {
    if (pValues == null)
    {
      return;
    }
    for (int i = pValues.length - 1; i >= 0; --i)
    {
      prependValue(pValues[i]);
    }
  }

  private String prepareValue(String pValue)
  {
    String result = pValue;

    if (mType.getParameterType() == VMArgumentType.PARAMETER_TYPE_IS_PATH)
    {
      result = new Path(pValue).toOSString();
    }

    return result;
  }

  /**
   * @see java.lang.Object#clone()
   */
  public Object clone() throws CloneNotSupportedException
  {
    VMArgument copy = (VMArgument) super.clone();
    List values = new ArrayList();
    values.addAll(mValues);
    copy.mValues = values;

    return copy;
  }

  /**
   * The key part of this argument.
   * 
   * @return the key part of this argument
   */
  public String getKey()
  {
    return mKey;
  }

  /**
   * The type of this argument.
   * 
   * @return the type of this argument
   */
  public VMArgumentType getType()
  {
    return mType;
  }

  /**
   * The list of all values for this argument. Could be empty, but never null.
   * 
   * @return the list of all values for this argument. Could be empty, but never null
   */
  public String[] getValues()
  {
    String[] result = new String[mValues.size()];

    return (String[]) mValues.toArray(result);
  }
}