/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.launching;

import java.util.StringTokenizer;

/**
 * VM arguments usually come in different flavors. The class attempts to capture such behaviour in
 * terms of type.   There are three characteristics that distinguish one VMArgumentType instance
 * from another. They are: <br>
 * 1- Number of parameters : can be 0, 1, or multiple. <br>
 * 2- Type of parameters : if a parameter represents a directory or a file, then its type is  said
 * to be of 'PATH', all other parameters that do not fall into this category are said to be of
 * type 'UNKNOWN'. <br>
 * 3- The separator used between the key and its values: if the separator is '=' then it is
 * referred to as 'ASSIGN', if the separator is ':' then it is referred to as 'ATTACH', if the
 * separator is ' ' then it is referred to as 'SPACE', when there is no separator between the key
 * and its values, then it is referred to as 'NOSPACE'. <br>
 * <br>
 * <br>
 * <br>
 * This class has a private constructor, therefore you can not create your own type, instead you
 * should supply a string that represents the above mentioned characteristics to the parse()
 * method to get an instance of the VMArgumentType. <br>
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class VMArgumentType
{
  public static final String CLASS_VERSION = "$Id: VMArgumentType.java,v 1.1 2004/09/20 01:39:04 hatimk Exp $"; //$NON-NLS-1$
  public static final int SINGLE_PARAMETER = 1;
  public static final int MULTI_PARAMETER = 1000;
  public static final int NO_PARAMETER = 0;
  public static final int PARAMETER_TYPE_IS_PATH = 0;
  public static final int PARAMATER_TYPE_IS_UNKNOWN = 1;
  public static final int KEYVALUE_SEPARATOR_IS_ASSIGN = 0;
  public static final int KEYVALUE_SEPARATOR_IS_NOSPACE = 1;
  public static final int KEYVALUE_SEPARATOR_IS_ATTACH = 2;
  public static final int KEYVALUE_SEPARATOR_IS_SPACE = 3;
  private int mParameterCount;
  private int mParameterType;
  private int mKeyValueSeparator;

  private VMArgumentType(int pParameterCount, int pParameterType, int pKeyValueRelation)
  {
    mParameterCount = pParameterCount;
    mParameterType = pParameterType;
    mKeyValueSeparator = pKeyValueRelation;
  }

  /**   
   * This method is your only way to obtain an instance of the VMArgumentType class, and of course
   * you need an instance of this class to be able to instantiate the VMArgument class itself. The
   * string you supply to this method has to follow simple format. Here are few examples to give
   * you the idea:<br>
   * <br>
   * <br>
   * When you have a VM argument such as '-Xdebug' that has no parameters, then the string should
   * look like "c=0,p=unknown,s=nospace". <br>
   * <br>
   * <br>
   * When you have a VM argument such as '-Dcatalina.base=C:\tomcat5.0;C:\exo' that has one or
   * more parameters, and these parameters represent directories or files, then the string should
   * look like "c=multi,p=path,s=assign".<br>
   * <br>
   * <br>
   * When you have a VM argument such as '-Dsun.io.useCanonCaches=false' that has one parameter
   * that is not a directory or a file, then the string should look like
   * "c=single,p=unknown,s=assign". <br>
   * <br>
   * <br>
   * When you have a VM argument such as
   * '-Xrunjdwp:transport=dt_socket,suspend=y,address=localhost:12593' that has one or more
   * parameters that are not directories, and the key-value separator is ':', then the string
   * should look like "c=multi,p=unknown,s=attach". <br>
   *
   * @param pArgType a String that represents the type (example "c=multi,p=path,s=assign")
   *
   * @return an instance of the VMArgumentType
   */
  public static VMArgumentType parse(String pArgType)
  {
    if (pArgType == null)
    {
      return null;
    }

    int paramCount = NO_PARAMETER;
    int paramType = PARAMATER_TYPE_IS_UNKNOWN;
    int keyvalueSeparator = KEYVALUE_SEPARATOR_IS_ASSIGN;

    StringTokenizer tokenizer = new StringTokenizer(pArgType, ",");
    while (tokenizer.hasMoreTokens())
    {
      String token = tokenizer.nextToken();
      if (token.startsWith("C=") || token.startsWith("c="))
      {
        String count = token.substring(2);
        if ("0".compareToIgnoreCase(count) == 0)
        {
          paramCount = NO_PARAMETER;
        }
        else if ("SINGLE".compareToIgnoreCase(count) == 0)
        {
          paramCount = SINGLE_PARAMETER;
        }
        else if ("MULTI".compareToIgnoreCase(count) == 0)
        {
          paramCount = MULTI_PARAMETER;
        }
      }
      else if (token.startsWith("P=") || token.startsWith("p="))
      {
        String type = token.substring(2);
        if ("PATH".compareToIgnoreCase(type) == 0)
        {
          paramType = PARAMETER_TYPE_IS_PATH;
        }
        else
        {
          paramType = PARAMATER_TYPE_IS_UNKNOWN;
        }
      }
      else if (token.startsWith("s=") || token.startsWith("s="))
      {
        String separator = token.substring(2);
        if ("ASSIGN".compareToIgnoreCase(separator) == 0)
        {
          keyvalueSeparator = KEYVALUE_SEPARATOR_IS_ASSIGN;
        }
        else if ("NOSPACE".compareToIgnoreCase(separator) == 0)
        {
          keyvalueSeparator = KEYVALUE_SEPARATOR_IS_NOSPACE;
        }
        else if ("ATTACH".compareToIgnoreCase(separator) == 0)
        {
          keyvalueSeparator = KEYVALUE_SEPARATOR_IS_ATTACH;
        }
        else if ("SPACE".compareToIgnoreCase(separator) == 0)
        {
          keyvalueSeparator = KEYVALUE_SEPARATOR_IS_SPACE;
        }
      }
    }

    return new VMArgumentType(paramCount, paramType, keyvalueSeparator);
  }

  /**   
   * Only the following values are allowed: <br>
   * KEYVALUE_SEPARATOR_IS_ASSIGN  (when the separator is '=') <br>
   * KEYVALUE_SEPARATOR_IS_NOSPACE (when there is no separator) <br>
   * KEYVALUE_SEPARATOR_IS_ATTACH  (when the separator is ':') <br>
   * KEYVALUE_SEPARATOR_IS_SPACE   (when the separator is ' ') <br>
   *
   * @return the key value separator type
   */
  public int getKeyValueSeparator()
  {
    return mKeyValueSeparator;
  }

  /**   
   * Only three values are permitted here. They are:<br>
   * NO_PARAMETER <br>
   * SINGLE_PARAMETER <br>
   * MULTI_PARAMETER <br>
   *
   * @return the parameter count
   */
  public int getParameterCount()
  {
    return mParameterCount;
  }

  /**   
   * The type of the paratmer. When the parameter represents a directory or a file, then the type
   * returned will be PARAMETER_TYPE_IS_PATH otherwise it will be PARAMETER_TYPE_IS_UNKNOWN.
   *
   * @return the expected type of parameter
   */
  public int getParameterType()
  {
    return mParameterType;
  }
}
