/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.launching;

import org.exoplatform.eclipse.core.ExoCorePlugin;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * A container of unorderd set of VMArguments. There can be at most one VMArgument with the same
 * key in this container.  This container allows the addition of values to the contained
 * VMArguments. This container also provides a method to represent the whole set of VMArguments as
 * one String. A typical usage would be to create an empty container at the begining and then pass
 * it to different objects so they get the chance to add their VMArguments to it, and then after
 * all the VMArguments are added, the String representation is obtained from this container  which
 * can be used later to help start a VM.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class VMArgumentsContainer
{
  public static final String CLASS_VERSION = "$Id: VMArgumentsContainer.java,v 1.1 2004/09/20 01:39:04 hatimk Exp $"; //$NON-NLS-1$
  private Map mVMArgs;

  /**
   * The constructor method.
   */
  public VMArgumentsContainer()
  {
    mVMArgs = new HashMap();
  }

  /**   
   * Given a VMArgument, check to see if there is another VMArgument instance in the container with
   * the same key, if so, then just append the values of the given VMArgument to the existing
   * VMArgument, otherwise add the given VMArgument to the container.
   *
   * @param pVMArg the VMArgument to add to the container
   */
  public void appendVMArgumentValues(VMArgument pVMArg)
  {
    if (pVMArg == null)
    {
      return;
    }

    String key = pVMArg.getKey();
    if ((key == null) || (key.length() < 1))
    {
      return;
    }

    Object arg = mVMArgs.get(key);
    if (arg == null)
    {
      // lets first create a copy of the given argument, so that the caller does not have to worry about
      // the original argument being changed.
      mVMArgs.put(key, getCopy(pVMArg));
    }
    else
    {
      ((VMArgument) arg).appendValues(pVMArg.getValues());
    }
  }

  /**   
   * This method applies the 'appendVMArgumentValues()' method on each VMArgument element in the
   * given VMArgument array.
   *
   * @param pArgs an array of VMArgument to append
   */
  public void appendVMArguments(VMArgument[] pArgs)
  {
    if (pArgs == null)
    {
      return;
    }
    for (int i = 0, sizei = pArgs.length; i < sizei; ++i)
    {
      appendVMArgumentValues(pArgs[i]);
    }
  }

  /**   
   * Given a VMArgument, check to see if there is another VMArgument instance in the container with
   * the same key, if so, then just prepend the values of the given VMArgument to the existing
   * VMArgument, otherwise add the given VMArgument to the container.
   *
   * @param pVMArg the VMArgument to add to the container
   */
  public void prependVMArgumentValues(VMArgument pVMArg)
  {
    if (pVMArg == null)
    {
      return;
    }

    String key = pVMArg.getKey();
    if ((key == null) || (key.length() < 1))
    {
      return;
    }

    Object arg = mVMArgs.get(key);
    if (arg == null)
    {
			// lets first create a copy of the given argument, so that the caller does not have to worry about
			// the original argument being changed.
			mVMArgs.put(key, getCopy(pVMArg));
    }
    else
    {
      ((VMArgument) arg).prependValues(pVMArg.getValues());
    }
  }

  /**   
   * This method applies the 'prependVMArgumentValues()' method on each VMArgument element in the
   * given VMArgument array.
   *
   * @param pArgs an array of VMArgument to prepend
   */
  public void prependVMArguments(VMArgument[] pArgs)
  {
    if (pArgs == null)
    {
      return;
    }
    for (int i = 0, sizei = pArgs.length; i < sizei; ++i)
    {
      prependVMArgumentValues(pArgs[i]);
    }
  }

  /**   
   * Given an argument key, return its values. String[0] will be returned if the argument is not in
   * the container.
   *
   * @param pKey the argument key
   *
   * @return the values assoicated with the given argument key
   */
  public String[] getVMArgumentValues(String pKey)
  {
    if (pKey == null)
    {
      return new String[0];
    }

    Object arg = mVMArgs.get(pKey);
    if (arg == null)
    {
      return new String[0];
    }
    else
    {
      return ((VMArgument) arg).getValues();
    }
  }

  /**   
   * Returns an array of VMArgument instances contained in this container.  The array elements are copies
   * of the instances in the container, therefore changes in the returned instances will not affect the
   * container.
   *
   * @return an array of VMArgument instances contained in this container.
   */
  public VMArgument[] getVMArguments()
  {
    Collection args = mVMArgs.values();

    if (args == null)
    {
      return new VMArgument[0];
    }
		
		int i = 0;
		int size = args.size();
		VMArgument[] resultarray =  new VMArgument[size];
		
		Iterator iterator = args.iterator();
		while(iterator.hasNext())
		{
			if (i < size)
			{
				resultarray[i] = getCopy((VMArgument) iterator.next());
			}
			++i;
		}
		
    return resultarray;
  }

  private VMArgument getCopy(VMArgument pVMArg)
  {
    VMArgument copy = null;
    try
    {
      copy = (VMArgument) pVMArg.clone();
    }
    catch (CloneNotSupportedException e)
    {
      ExoCorePlugin.logError(e);
    }

    return copy;
  }

  /**   
   * This method produces a String representation of all VMArguments contained in this container.
   * The String format is very dependent on the type of each VMArgument. For a description of how
   * the format is done, please the VMArgumentFormatter class.
   *
   * @return the String representation of the content of all the contained VMArguments
   *
   * @see org.exoplatform.eclipse.core.launching.VMArgumentFormatter
   */
  public String toString()
  {
    StringBuffer sb = new StringBuffer();
    Collection args = mVMArgs.values();
    Iterator iterator = args.iterator();
    while (iterator.hasNext())
    {
      VMArgument vmarg = (VMArgument) iterator.next();
      sb.append(VMArgumentFormatter.format(vmarg));
      if (iterator.hasNext())
      {
        sb.append(" ");
      }
    }

    return sb.toString();
  }
}
