/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.operation;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * An operation which scans the given folder for any jar files, it then adds
 * the jar files to the classpath of the given project.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class AddFolderToClasspathOperation extends WorkspaceModifyOperation
{
  public static final String CLASS_VERSION = "$Id: AddFolderToClasspathOperation.java,v 1.1 2004/04/19 03:45:48 hatimk Exp $"; //$NON-NLS-1$
  private IJavaProject mJavaProject;
  private IFolder mFolder;
  private boolean mIgnoreReplica;

  /**
   * The constructor method.
   *
   * @param pJavaProject the java project to add the libraries found in the
   *        given folder to it
   * @param pFolder the folder to scan for jar files
   * @param pIgnoreReplica if true, the operation does not add the jar file if
   *        it already exists in the classpath. Two jar files are said to be
   *        the same if they have the same name.
   */
  public AddFolderToClasspathOperation(IJavaProject pJavaProject,
    IFolder pFolder, boolean pIgnoreReplica)
  {
    mJavaProject = pJavaProject;
    mFolder = pFolder;
    mIgnoreReplica = pIgnoreReplica;
  }

  /**   
   * @see org.eclipse.ui.actions.WorkspaceModifyOperation#execute(org.eclipse.core.runtime.IProgressMonitor)
   */
  protected void execute(IProgressMonitor pMonitor)
    throws CoreException, InvocationTargetException, InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    pMonitor.beginTask("Updating the classpath to include all jar files in "
      + mFolder.getName() + " : ", 2500);
    pMonitor.subTask("scan the folder (" + mFolder.getName()
      + ")to check for jar files");

    IPath[] paths = getJarFilesPath(mFolder, mIgnoreReplica);
    if (mIgnoreReplica)
    {
      paths = filterJars(mJavaProject, paths);
    }
    pMonitor.subTask("assign all found jar files to the project classpath");
    addToClasspath(mJavaProject, paths, new SubProgressMonitor(pMonitor, 2500));
    pMonitor.done();
  }

  /**   
   * Returns a list of full paths to all jar files in the given folder. If the
   * ignore replica flag is true, then jar files that have the same name will
   * be counted only once.
   *
   * @param pFolder the folder to scan for jar files
   * @param pIgnoreReplica ture if replica jar files should not count
   *
   * @return returns a list of full paths to all jar files in the given folder
   *
   * @throws CoreException if this request fails. Reasons include: <ul><li>
   *         This resource does not exist.</li> <li> This resource is a
   *         project that is not open.</li> </ul>
   */
  protected IPath[] getJarFilesPath(IFolder pFolder, boolean pIgnoreReplica)
    throws CoreException
  {
    List paths = new ArrayList();
    Set unique = new HashSet();
    if (pFolder == null)
    {
      return new IPath[0];
    }

    IResource[] resources = pFolder.members();
    if (resources == null)
    {
      return new IPath[0];
    }

    for (int i = 0, sizei = resources.length; i < sizei; ++i)
    {
      String extension = resources[i].getFileExtension();
      if (((extension != null) && (extension.length() > 0))
          && (extension.equalsIgnoreCase(IExoOperationConstant.JAR_EXTENSION)))
      {
        IPath fullPath = resources[i].getFullPath();

        if (unique.add(fullPath.lastSegment()))
        {
          paths.add(fullPath);
        }
        else if (!pIgnoreReplica)
        {
          paths.add(fullPath);
        }
      }
    }

    IPath[] result = new IPath[paths.size()];
    result = (IPath[]) paths.toArray(result);

    return result;
  }

  protected IPath[] filterJars(IJavaProject pJavaProject, IPath[] pPaths)
    throws JavaModelException
  {
    if (pPaths == null)
    {
      return new IPath[0];
    }

    IClasspathEntry[] ce = pJavaProject.getResolvedClasspath(true);

    if (ce == null)
    {
      return pPaths;
    }

    // create a set with all resolved paths last segement
    Set uniqueJarNames = new HashSet();
    for (int i = 0, sizei = ce.length; i < sizei; ++i)
    {
      uniqueJarNames.add(ce[i].getPath().lastSegment());
    }

    List paths = new ArrayList();
    for (int i = 0, sizei = pPaths.length; i < sizei; ++i)
    {
      if (uniqueJarNames.add(pPaths[i].lastSegment()))
      {
        paths.add(pPaths[i]);
      }
    }

    IPath[] result = new IPath[paths.size()];
    result = (IPath[]) paths.toArray(result);

    return result;
  }

  protected void addToClasspath(IJavaProject pJavaProject, IPath[] pPaths,
    IProgressMonitor pMonitor) throws JavaModelException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    try
    {
      if (pPaths == null)
      {
        pMonitor.beginTask("", 1); //$NON-NLS-1$

        return;
      }

      List paths = getExistingClasspath(pJavaProject);
      IClasspathEntry ce;

      for (int i = 0, sizei = pPaths.length; i < sizei; ++i)
      {
        ce = JavaCore.newLibraryEntry(pPaths[i], null, null);
        paths.add(ce);
      }

      IClasspathEntry[] entries = new IClasspathEntry[paths.size()];
      entries = (IClasspathEntry[]) paths.toArray(entries);

      pMonitor.beginTask("", 100); //$NON-NLS-1$

      // set the classpath
      pMonitor.subTask("set the classpath after adding more entries");
      if (entries.length > 0)
      {
        pJavaProject.setRawClasspath(entries,
          new SubProgressMonitor(pMonitor, 100));
      }
    }
    finally
    {
      pMonitor.done();
    }
  }

  protected List getExistingClasspath(IJavaProject pJavaProject)
    throws JavaModelException
  {
    List result = new ArrayList();

    IClasspathEntry[] ce = pJavaProject.getRawClasspath();

    if (ce == null)
    {
      return result;
    }

    for (int i = 0, sizei = ce.length; i < sizei; ++i)
    {
      result.add(ce[i]);
    }

    return result;
  }
}
