/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.operation;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;
import java.util.List;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.exoplatform.eclipse.core.common.IOverwriteQuery;
import org.exoplatform.eclipse.core.runtime.IExoProjectRuntimeProperties;

/**
 * This operation takes care of deploying a web application. It can deploy in the exploded
 * form or in the packaged form.
 * 
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class DeployWebappOperation extends WorkspaceModifyOperation
{
  public static final String CLASS_VERSION = "$Id: DeployWebappOperation.java,v 1.1 2004/04/19 03:45:48 hatimk Exp $"; //$NON-NLS-1$
  private File mDeploymentFile;
  private IPath mWebFolder; // full path relative to the workspace
  private int mDeploymentOption;
  private IOverwriteQuery mOverwriteQuery;
  private IProject mProject;

  /**
   * The constructor method.
   * 
   * @param pProject the project to consider for deployment
   * @param pWebFolder the full path of the web folder relative to the workspace
   * @param pDeploymentFile the full name of the location to deploy, it should not include
   *          ".war" in the name, because this operation will add it if needed.
   * @param pDeploymentOption the deployment option
   * @param pOverwriteQuery the implementation of this interface should answer one of the
   *          prescribed return codes when asked whether to overwrite a certain path
   *          string (which could represent a resource path, a file system path, etc). The
   *          above comment was copied from the IOverwriteQuery interface comment.
   */
  public DeployWebappOperation(IProject pProject, IPath pWebFolder, File pDeploymentFile,
      int pDeploymentOption, IOverwriteQuery pOverwriteQuery)
  {
    mProject = pProject;
    mDeploymentFile = pDeploymentFile;
    mWebFolder = pWebFolder;
    mDeploymentOption = pDeploymentOption;
    mOverwriteQuery = pOverwriteQuery;
  }

  protected void execute(IProgressMonitor pMonitor) throws CoreException, InvocationTargetException,
      InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    try
    {
      pMonitor.beginTask("", 2500);
      pMonitor.subTask("");

      IFolder webfolder = ResourcesPlugin.getWorkspace().getRoot().getFolder(mWebFolder);

      if (!webfolder.exists())
      {
        addError("Could not find the web folder", null);

        return;
      }

      switch (mDeploymentOption)
      {
        case IExoProjectRuntimeProperties.DEPLOYMENT_OPTION_EXPLODED:
          DeployExplodedOption(mProject, webfolder, mDeploymentFile, pMonitor);

          break;

        case IExoProjectRuntimeProperties.DEPLOYMENT_OPTION_PACKAGED:
          if (isOutputMatching(mProject, webfolder))
          {
            DeployPackagedOption(webfolder, mDeploymentFile, pMonitor);
          }
          else
          {
            addError("package deployment of portlets projects with classes folder other than WEB-INF/classes is not supported!", null);
          }

          break;
        default:
          throw new IllegalArgumentException("unrecognized deployment option");
      }
    }
    finally
    {
      pMonitor.done();
    }
  }

  protected void DeployExplodedOption(IProject pProject, IFolder pWebFolder, File pDeploymentFile,
      IProgressMonitor pMonitor) throws CoreException, InvocationTargetException, InterruptedException
  {
    pDeploymentFile.mkdir();

    if (!pDeploymentFile.exists())
    {
      addError("Could not create the deployment location", null);

      return;
    }

    List sources = Arrays.asList(pWebFolder.members());
    FileSystemExportOperation op = new FileSystemExportOperation(sources, pDeploymentFile.toString(),
        mOverwriteQuery);

    op.setCreateLeadupStructure(false);
    op.run(new SubProgressMonitor(pMonitor, 2000));

    IStatus status = op.getStatus();
    if (!status.isOK())
    {
      addToStatus(status);
    }

    // now if the output location does not match WEB-INF/classes we need to copy the
    // output location to the appropriate location on the deployment side
    if (!isOutputMatching(pProject, pWebFolder))
    {
      IPath outputPath = getOutputPath(pProject);
      IFolder outputFolder = ResourcesPlugin.getWorkspace().getRoot().getFolder(outputPath);

      // lets get the outputPath as elements
      sources = Arrays.asList(outputFolder.members());
      String targetClassesPath = new Path(pDeploymentFile.toString()).append("WEB-INF/classes").toString();
      // make sure the directory WEB-INF/classes exist if not create it in the deployment folder
      new File(targetClassesPath).mkdirs();
      
      op = new FileSystemExportOperation(sources, targetClassesPath, mOverwriteQuery);

      op.setCreateLeadupStructure(false);
      op.run(new SubProgressMonitor(pMonitor, 500));

      status = op.getStatus();
      if (!status.isOK())
      {
        addToStatus(status);
      }

    }
  }

  protected void DeployPackagedOption(IFolder pWebFolder, File pDeploymentFile, IProgressMonitor pMonitor)
      throws CoreException, InvocationTargetException, InterruptedException
  {
    IPath warFile = new Path(pDeploymentFile.toString() + ".war");

    WarFileExportOperation op = new WarFileExportOperation(pWebFolder, warFile.toFile(), mOverwriteQuery);
    op.run(new SubProgressMonitor(pMonitor, 2500));

    IStatus status = op.getStatus();
    if (!status.isOK())
    {
      addToStatus(status);
    }
  }

  /**
   * Check if the output location is the same as the WEB-INF/classes
   * 
   * @param pProject the project to inspect
   * @param pWebFolder the web content folder under which WEB-INF/classes should exist
   * @return true/false
   */
  protected boolean isOutputMatching(IProject pProject, IFolder pWebFolder)
  {
    IJavaProject javaProject = JavaCore.create(pProject);
    if (javaProject == null) return false;

    try
    {
      IPath outputPath = javaProject.getOutputLocation();
      IPath webOutputPath = pWebFolder.getFullPath().append("WEB-INF/classes");
      return webOutputPath.equals(outputPath);
    }
    catch (JavaModelException e)
    {
      addError(e.getMessage(), e);
      return false;
    }
  }

  protected IPath getOutputPath(IProject pProject)
  {
    IJavaProject javaProject = JavaCore.create(pProject);
    if (javaProject == null) return null;

    try
    {
      IPath outputPath = javaProject.getOutputLocation();
      return outputPath;
    }
    catch (JavaModelException e)
    {
      addError(e.getMessage(), e);
      return null;
    }
  }
}