/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.operation;


import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.exoplatform.eclipse.core.IPortletProject;
import org.exoplatform.eclipse.core.xmlvalidator.IPortletDescriptorValidator;
import org.exoplatform.eclipse.core.xmlvalidator.PortletDescriptorValidatorProvider;
import org.exoplatform.eclipse.core.xmlvalidator.PortletValidationProblemReporter;
import org.exoplatform.eclipse.core.xmlvalidator.helper.PortletDeploymentDescriptorInspector;

import java.lang.reflect.InvocationTargetException;

/**
 * Use this operation to validate portlet.xml files. This operation takes care
 * of choosing the appropriate validator for the appropriate portlet.xml
 * version.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class ValidatePortletDeploymentDescriptorOperation
  extends WorkspaceModifyOperation
{
  public static final String CLASS_VERSION = "$Id: ValidatePortletDeploymentDescriptorOperation.java,v 1.1 2004/04/19 03:45:48 hatimk Exp $"; //$NON-NLS-1$
  private IProject mProject;
  private IFile mPortletFile;

  /**
   * The constructor method. If you use this constructor then all projects will
   * be validated when you call the run method.
   */
  public ValidatePortletDeploymentDescriptorOperation()
  {
    mProject = null;
    mPortletFile = null;
  }

  /**
   * The constructor method. If you pass null, then all projects will be
   * validated.
   *
   * @param pProject the project to validate its portlet.xml, if null is given
   *        then all projects will be validated
   */
  public ValidatePortletDeploymentDescriptorOperation(IProject pProject)
  {
    mProject = pProject;
  }

  /**
   * The constructor method. If you pass null, then all projects will be
   * validated.
   *
   * @param pPortletFile the portlet.xml to validate if null is given then all
   *        projects will be validated
   */
  public ValidatePortletDeploymentDescriptorOperation(IFile pPortletFile)
  {
    mPortletFile = pPortletFile;
  }

  /**   
   * @see org.exoplatform.eclipse.core.operation.WorkspaceModifyOperation#execute(org.eclipse.core.runtime.IProgressMonitor)
   */
  protected void execute(IProgressMonitor pMonitor)
    throws CoreException, InvocationTargetException, InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    // check if we have a project given to this operation at construction time
    if (mProject != null)
    {
      validatePortletXML(mProject, pMonitor);

      return;
    }

    // check if we have portlet file given to this operation at construction time
    if (mPortletFile != null)
    {
      validatePortletXML(mPortletFile, pMonitor);

      return;
    }

    // otherwise validate all projects
    IProject[] allProjects =
      ResourcesPlugin.getWorkspace().getRoot().getProjects();

    if (allProjects == null)
    {
      return;
    }

    pMonitor.beginTask("", allProjects.length * 10);
    for (int i = 0, sizei = allProjects.length; i < sizei; i++)
    {
      IProject project = allProjects[i];
      if (!project.isOpen())
      {
        pMonitor.worked(10);

        continue;
      }
      if (!project.hasNature(ExoCorePlugin.PORTLET_NATURE_ID))
      {
        pMonitor.worked(10);

        continue;
      }
      if (pMonitor.isCanceled())
      {
        throw new InterruptedException();
      }

      validatePortletXML(project, new SubProgressMonitor(pMonitor, 10));
    }
    pMonitor.done();
  }

  private void validatePortletXML(IProject pProject, IProgressMonitor pMonitor)
    throws CoreException, InvocationTargetException, InterruptedException
  {
    IPortletProject portletProject =
      ExoCorePlugin.createPortletProject(pProject);

    IFile portletFile = pProject.getFile(portletProject.getPortletXML());
    validatePortletXML(portletFile, pMonitor);
  }

  private void validatePortletXML(IFile pPortletFile, IProgressMonitor pMonitor)
    throws CoreException, InvocationTargetException, InterruptedException
  {
    IProject project = pPortletFile.getProject();

    pMonitor.beginTask("Validating portlet.xml in project ["
      + project.getName() + "] : ", 100);

    // next, we need to determine the spec version that the portlet.xml is 
    // following.
    String version =
      PortletDeploymentDescriptorInspector.getVersion(pPortletFile);

    PortletDescriptorValidatorProvider provider =
      new PortletDescriptorValidatorProvider();
    IPortletDescriptorValidator validator =
      provider.getValidator(version, true);

    PortletValidationProblemReporter reporter =
      new PortletValidationProblemReporter(pPortletFile);
    reporter.deleteAllProblems();
    if (validator == null)
    {
      reporter.addOperationError(
        "Unable to find a validator for the portlet deployment descriptor");
    }
    else
    {
      validator.validate(reporter, new SubProgressMonitor(pMonitor, 100));
    }
    reporter.reportProblems();

    pMonitor.done();
  }
}
