/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.operation;


import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.IJavaModelMarker;
import org.exoplatform.eclipse.core.common.IOverwriteQuery;
import org.exoplatform.eclipse.core.warpackager.WarPackageData;
import org.exoplatform.eclipse.core.warpackager.WarWriter;

import java.io.File;

import java.lang.reflect.InvocationTargetException;

import java.util.zip.ZipException;

/**
 * An operation that generates a WAR file given a web folder.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class WarFileExportOperation extends WorkspaceModifyOperation
{
  public static final String CLASS_VERSION = "$Id: WarFileExportOperation.java,v 1.1 2004/04/19 03:45:48 hatimk Exp $"; //$NON-NLS-1$
  private File mDeploymentFile;
  private IFolder mWebFolder; // full path relative to the workspace
  private IOverwriteQuery mOverwriteQuery;

  /**
   * The constructor method.
   *
   * @param pWebFolder the full path of the web folder relative to the
   *        workspace
   * @param pDeploymentFile the full name of the WAR file to create for
   *        deployment. The deployment file should also have the ".war"
   *        extension.
   * @param pOverwriteQuery the implementation of this interface should answer
   *        one of the prescribed return codes when asked whether to overwrite
   *        a certain path string (which could represent a resource path, a
   *        file system path, etc). The above comment was copied from the
   *        IOverwriteQuery interface comment.
   */
  public WarFileExportOperation(IFolder pWebFolder, File pDeploymentFile,
    IOverwriteQuery pOverwriteQuery)
  {
    mDeploymentFile = pDeploymentFile;
    mWebFolder = pWebFolder;
    mOverwriteQuery = pOverwriteQuery;
  }

  /**   
   * @see org.exoplatform.eclipse.core.operation.WorkspaceModifyOperation#execute(org.eclipse.core.runtime.IProgressMonitor)
   */
  protected void execute(IProgressMonitor pMonitor)
    throws CoreException, InvocationTargetException, InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    WarWriter warWriter = null;
    try
    {
      IPath warFile = new Path(mDeploymentFile.toString());

      WarPackageData warData = new WarPackageData();
      warData.setWarLocation(warFile);
      warData.setElements(mWebFolder.members());

      if (!preconditionsOK(warData))
      {
        addError("JAR creation failed.", null);
      }

      int totalWork = countSelectedElements(warData);

      pMonitor.beginTask("", totalWork); //$NON-NLS-1$
      warWriter = warData.createWarWriter(mOverwriteQuery);
      exportSelectedElements(warWriter, warData,
        mWebFolder.getFullPath().segmentCount(), pMonitor);
    }
    finally
    {
      try
      {
        if (warWriter != null)
        {
          warWriter.close();
        }
      }
      catch (CoreException ex)
      {
        addToStatus(ex);
      }

      pMonitor.done();
    }
  }

  protected boolean preconditionsOK(WarPackageData pPackageData)
  {
    if ((pPackageData.getElements() == null)
        || (pPackageData.getElements().length == 0))
    {
      addError("No resources selected", null);

      return false;
    }

    if (pPackageData.getWarLocation() == null)
    {
      addError("Invalid WAR location", null);

      return false;
    }

    File targetFile = pPackageData.getWarLocation().toFile();
    if (targetFile.exists() && !targetFile.canWrite())
    {
      addError("WAR file already exists and cannot be overwritten", null);

      return false;
    }
    if (!pPackageData.isManifestAccessible())
    {
      addError("Manifest does not exist", null);

      return false;
    }

    return true;
  }

  /**   
   * Exports the resources as specified by the WAR package.
   *
   * @param pWriter
   * @param pPackageData
   * @param pLeadSegmentsToRemove
   * @param pMonitor
   *
   * @throws InterruptedException
   */
  protected void exportSelectedElements(WarWriter pWriter,
    WarPackageData pPackageData, int pLeadSegmentsToRemove,
    IProgressMonitor pMonitor) throws InterruptedException
  {
    int n = pPackageData.getElements().length;
    for (int i = 0; i < n; i++)
    {
      exportElement(pWriter, pPackageData,
        (IResource) pPackageData.getElements()[i], pLeadSegmentsToRemove,
        pMonitor);
    }
  }

  /**   
   * Exports the passed resource to the JAR file
   *
   * @param pWriter
   * @param pPackageData
   * @param pResource the resource to export
   * @param pLeadSegmentsToRemove
   * @param pMonitor
   *
   * @throws InterruptedException
   */
  protected void exportElement(WarWriter pWriter, WarPackageData pPackageData,
    IResource pResource, int pLeadSegmentsToRemove, IProgressMonitor pMonitor)
    throws InterruptedException
  {
    if (!pResource.isAccessible())
    {
      addWarning("Resource not found or not accessible: "
        + pResource.getFullPath(), null);

      return;
    }

    if (pResource.getType() == IResource.FILE)
    {
      if (!pResource.isLocal(IResource.DEPTH_ZERO))
      {
        try
        {
          pResource.setLocal(true, IResource.DEPTH_ZERO, pMonitor);
        }
        catch (CoreException ex)
        {
          addWarning("Resource could not be retrieved locally: "
            + pResource.getFullPath(), ex);

          return;
        }
      }

      IPath destinationPath =
        pResource.getFullPath().removeFirstSegments(pLeadSegmentsToRemove);

      exportResource(pWriter, pPackageData, pResource, destinationPath, pMonitor);

      pMonitor.worked(1);
      if (pMonitor.isCanceled())
      {
        throw new InterruptedException();
      }
    }
    else
    {
      exportContainer(pWriter, pPackageData, pLeadSegmentsToRemove,
        (IContainer) pResource, pMonitor);
    }
  }

  protected void reportPossibleCompileProblems(IFile file, boolean pHasErrors,
    boolean pHasWarnings)
  {
    if (pHasErrors)
    {
      addWarning("Exported with compile errors: " + file.getFullPath(), null);
    }
    if (pHasWarnings)
    {
      addWarning("Exported with compile warnings: " + file.getFullPath(), null);
    }
  }

  /**   
   * Answers the number of file resources specified by the WAR package.
   *
   * @param pWarPackage the War package to inspect
   *
   * @return the number of file resources specified by the WAR package
   */
  protected int countSelectedElements(WarPackageData pWarPackage)
  {
    int count = 0;
    int n = pWarPackage.getElements().length;
    for (int i = 0; i < n; i++)
    {
      Object element = pWarPackage.getElements()[i];
      IResource resource = null;
      if (element instanceof IResource)
      {
        resource = (IResource) element;
        if (resource.getType() == IResource.FILE)
        {
          count++;
        }
        else if (resource instanceof IContainer)
        {
          count += getTotalChildCount((IContainer) resource);
        }
      }
    }

    return count;
  }

  protected String getDefaultStatusErrorMessage()
  {
    return "Unexpected core error during deployment.";
  }

  /**   
   * Tells whether the given resource (or its children) have compile errors.
   * The method acts on the current build state and does not recompile.
   *
   * @param pResource the resource to check for errors
   *
   * @return <code>true</code> if the resource (and its children) are error
   *         free
   *
   * @throws CoreException org.eclipse.core.runtime.CoreException if there's a
   *         marker problem
   */
  private boolean hasCompileErrors(IResource pResource)
    throws CoreException
  {
    IMarker[] problemMarkers =
      pResource.findMarkers(IJavaModelMarker.JAVA_MODEL_PROBLEM_MARKER, true,
        IResource.DEPTH_INFINITE);
    for (int i = 0; i < problemMarkers.length; i++)
    {
      if (problemMarkers[i].getAttribute(IMarker.SEVERITY, -1) == IMarker.SEVERITY_ERROR)
      {
        return true;
      }
    }

    return false;
  }

  /**   
   * Tells whether the given resource (or its children) have compile errors.
   * The method acts on the current build state and does not recompile.
   *
   * @param pResource the resource to check for errors
   *
   * @return <code>true</code> if the resource (and its children) are error
   *         free
   *
   * @throws CoreException org.eclipse.core.runtime.CoreException if there's a
   *         marker problem
   */
  private boolean hasCompileWarnings(IResource pResource)
    throws CoreException
  {
    IMarker[] problemMarkers =
      pResource.findMarkers(IJavaModelMarker.JAVA_MODEL_PROBLEM_MARKER, true,
        IResource.DEPTH_INFINITE);
    for (int i = 0; i < problemMarkers.length; i++)
    {
      if (problemMarkers[i].getAttribute(IMarker.SEVERITY, -1) == IMarker.SEVERITY_WARNING)
      {
        return true;
      }
    }

    return false;
  }

  private void exportResource(WarWriter pWriter, WarPackageData pPackageData,
    IResource pResource, IPath pDestinationPath, IProgressMonitor pMonitor)
  {
    // Handle case where META-INF/MANIFEST.MF is part of the exported files
    if (pDestinationPath.toString().equals("META-INF/MANIFEST.MF"))
    { //$NON-NLS-1$
      if (pPackageData.isManifestGenerated())
      {
        addWarning(pResource.getFullPath()
          + " was replaced by the generated MANIFEST.MF and is no longer in the WAR. "+
          "This release does not support the use of existing MANIFEST.MF while deploying. If "+
          "you don't want to see this message again. Please remove the META-INF/MANIFEST.MF file from your "+
          "web content directory.",
          null);
      }

      return;
    }

    try
    {
      pMonitor.subTask("Deploying: " + pDestinationPath.toString());
      if (pResource instanceof IFile)
      {
        // currently this functionality is not completed, because the class file
        // itself does not have the warning or error marker. We need to trace
        // the class file to its source file first.
        boolean hasErrors = hasCompileErrors(pResource);
        boolean hasWarnings = hasCompileWarnings(pResource);
        reportPossibleCompileProblems((IFile) pResource, hasErrors, hasWarnings);
      }

      pWriter.write((IFile) pResource, pDestinationPath);
    }
    catch (CoreException ex)
    {
      Throwable realEx = ex.getStatus().getException();
      if (realEx instanceof ZipException && (realEx.getMessage() != null)
          && realEx.getMessage().startsWith("duplicate entry:")) //$NON-NLS-1$
      {
        addWarning(ex.getMessage(), realEx);
      }
      else
      {
        addToStatus(ex);
      }
    }
  }

  private void exportContainer(WarWriter pWriter, WarPackageData pPackageData,
    int pLeadSegmentsToRemove, IContainer pContainer, IProgressMonitor pMonitor)
    throws java.lang.InterruptedException
  {
    IResource[] children = null;
    try
    {
      children = pContainer.members();
    }
    catch (CoreException e)
    {
      // this should never happen because an #isAccessible check is done before #members is invoked
      addWarning("Error deploying " + pContainer.getFullPath(), e);
    }
    for (int i = 0; i < children.length; i++)
    {
      exportElement(pWriter, pPackageData, children[i], pLeadSegmentsToRemove,
        pMonitor);
    }
  }

  private int getTotalChildCount(IContainer container)
  {
    IResource[] members;
    try
    {
      members = container.members();
    }
    catch (CoreException ex)
    {
      return 0;
    }

    int count = 0;
    for (int i = 0; i < members.length; i++)
    {
      if (members[i].getType() == IResource.FILE)
      {
        count++;
      }
      else
      {
        count += getTotalChildCount((IContainer) members[i]);
      }
    }

    return count;
  }
}
