/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.operation.helper;


import org.eclipse.core.runtime.IPath;
import org.exoplatform.eclipse.core.filesprovider.JavaProjectFilesProvider;

import java.util.ArrayList;
import java.util.List;

/**
 * This class provides access to multiple java project files providers and to
 * the mounting point (path relative to the project) that they should exist
 * under.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class JavaProjectSettings extends ProjectSettings
{
  public static final String CLASS_VERSION = "$Id: JavaProjectSettings.java,v 1.1 2004/04/19 03:45:48 hatimk Exp $"; //$NON-NLS-1$
  private List mJavaProjectFilesProviders;
  private IPath mSourceFolder;
  private IPath mBinaryFolder;
  private IPath mLibraryFolder;

  /**
   * The constructor method.
   *
   * @param pParentFolder a relative path from the project that represents the
   *        mounting point. Since the path is relative to the project, then,
   *        for example, both the path "/test" and the path "test" are the
   *        same. A null value means the parent folder is the project folder
   *        itself.
   * @param pSourceFolder a relative path from the parent folder that
   *        represents the source folder
   * @param pBinaryFolder a relative path from the parent folder that
   *        represents the binary folder
   * @param pLibraryFolder a relative path from the parent folder that
   *        represents a folder that contains jar files to be added to the
   *        classpath, (can be null)
   */
  public JavaProjectSettings(IPath pParentFolder, IPath pSourceFolder,
    IPath pBinaryFolder, IPath pLibraryFolder)
  {
    super(pParentFolder);
    mSourceFolder = pSourceFolder;
    mBinaryFolder = pBinaryFolder;
    mLibraryFolder = pLibraryFolder;
  }

  /**   
   * Adds the given java project files provider to the collection of providers
   * that this class represents. Notice that this method also informs the java
   * project files provider about what the source folder and binary folder
   * names should look like. Also, in addition of being exposed in the
   * getJavaProjectFilesProviders(), the files provider will be exposed in the
   * getProjectFilesProviders() too.
   *
   * @param pProvider holdes information about an import structure provider
   *        that can be used later to supply content to interested operations
   *        (such as project create operation).
   */
  public void addJavaProjectFilesProvider(JavaProjectFilesProvider pProvider)
  {
    if (mJavaProjectFilesProviders == null)
    {
      mJavaProjectFilesProviders = new ArrayList();
    }

    addRenaming(pProvider);
    mJavaProjectFilesProviders.add(pProvider);
    addProjectFilesProvider(pProvider);
  }

  /**   
   * Returns all the java project files providers that this class represents.
   *
   * @return a java project files provider
   */
  public JavaProjectFilesProvider[] getJavaProjectFilesProviders()
  {
    if (mJavaProjectFilesProviders == null)
    {
      return new JavaProjectFilesProvider[0];
    }

    JavaProjectFilesProvider[] result =
      new JavaProjectFilesProvider[mJavaProjectFilesProviders.size()];
    result =
      (JavaProjectFilesProvider[]) mJavaProjectFilesProviders.toArray(result);

    return result;
  }

  /**   
   * Allows you to set all the java project files providers that this class
   * represents. This method removes any existing file providers that this
   * class represents and replaces them with the ones given.  Notice that this
   * method also informs the given java project files providers about what the
   * source folder and binary folder names should look like. Also, in addition
   * of being exposed in the getJavaProjectFilesProviders(), the files
   * provider will be exposed in the getProjectFilesProviders() too.
   *
   * @param pProviders an array of java project files providers
   */
  public void setJavaProjectFilesProviders(
    JavaProjectFilesProvider[] pProviders)
  {
    if (mJavaProjectFilesProviders == null)
    {
      mJavaProjectFilesProviders = new ArrayList();
    }
    else
    {
      mJavaProjectFilesProviders.clear();
    }

    if ((pProviders == null) || (pProviders.length < 1))
    {
      return;
    }

    for (int i = 0, sizei = pProviders.length; i < sizei; ++i)
    {
      addRenaming(pProviders[i]);
      mJavaProjectFilesProviders.add(pProviders[i]);
      addProjectFilesProvider(pProviders[i]);
    }
  }

  /**   
   * Given a provider, tell it to rename its source and binary folders to the
   * source and binary folders of this class
   *
   * @param pProvider to possibly telll it to rename its source and binary
   *        folders
   */
  protected void addRenaming(JavaProjectFilesProvider pProvider)
  {
    IPath providerSource = pProvider.getSourceFolder();
    if ((providerSource != null) && (mSourceFolder != null)
        && (!mSourceFolder.equals(providerSource)))
    {
      pProvider.getStructureProvider().addRenameEntry(providerSource,
        mSourceFolder);
    }

    IPath providerBinary = pProvider.getBinaryFolder();
    if ((providerBinary != null) && (mBinaryFolder != null)
        && (!mBinaryFolder.equals(providerBinary)))
    {
      pProvider.getStructureProvider().addRenameEntry(providerBinary,
        mBinaryFolder);
    }

    IPath providerLibrary = pProvider.getLibraryFolder();
    if ((providerLibrary != null) && (mLibraryFolder != null)
        && (!mLibraryFolder.equals(providerLibrary)))
    {
      pProvider.getStructureProvider().addRenameEntry(providerLibrary,
        mLibraryFolder);
    }
  }

  /**   
   * Gives you access to the intended binary folder
   *
   * @return a path for the binaries (relative to the parent folder)
   */
  public IPath getBinaryFolder()
  {
    return mBinaryFolder;
  }

  /**   
   * Gives you access to the intended source folder
   *
   * @return a path for the source code (relative to the parent folder)
   */
  public IPath getSourceFolder()
  {
    return mSourceFolder;
  }

  /**   
   * Gives you access to the intended binary folder
   *
   * @return a path for the library folder (relative to the parent folder)
   */
  public IPath getLibraryFolder()
  {
    return mLibraryFolder;
  }
}
