/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.operation.helper;


import org.eclipse.core.runtime.IPath;
import org.exoplatform.eclipse.core.filesprovider.ProjectFilesProvider;

import java.util.ArrayList;
import java.util.List;

/**
 * This class provides access to multiple project files providers and to the
 * mounting point (path relative to the project) that they should exist under.
 * For example, an instance of this class could contain two project files
 * providers. Lets say that the first one provides one directory 'test' with a
 * file 'readme.txt' in that directory, and lets say that the second one
 * provides one directory 'install'. If the mount point (the parent  folder)
 * is 'doc', then an operation (such as project create opertation), that uses
 * this instance, should result in a project with the directory 'doc' that has
 * two directories 'test' and 'install'. Furthermore, the 'test' directory
 * should contain the 'readme.txt' file.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class ProjectSettings
{
  public static final String CLASS_VERSION = "$Id: ProjectSettings.java,v 1.1 2004/04/19 03:45:48 hatimk Exp $"; //$NON-NLS-1$
  private List mProjectFilesProviders;
  private IPath mParentFolder;

  /**
   * The constructor method.
   *
   * @param pParentFolder a relative path from the project that represents the
   *        mounting point. Since the path is relative to the project, then,
   *        for example, both the path "/src" and the path "src" are the same.
   *        A null value means the parent folder is the project folder itself.
   */
  public ProjectSettings(IPath pParentFolder)
  {
    mParentFolder = pParentFolder;
  }

  /**   
   * Adds the given project files provider to the collection of providers that
   * this class represents.
   *
   * @param pProvider holdes information about an import structure provider
   *        that can be used later to supply content to interested operations
   *        (such as project create operation).
   */
  public void addProjectFilesProvider(ProjectFilesProvider pProvider)
  {
    if (mProjectFilesProviders == null)
    {
      mProjectFilesProviders = new ArrayList();
    }

    mProjectFilesProviders.add(pProvider);
  }

  /**   
   * Returns the mounting point (a relative path from the project). If the path
   * is null then it is assumed that the parent folder is nothing but the
   * project folder itself.
   *
   * @return a relative path from the project that represents the mounting
   *         point. Since the path is relative to the project, then, for
   *         example, both the path "/src" and the path "src" are the same. A
   *         null value means the parent folder is the project folder itself.
   */
  public IPath getParentFolder()
  {
    return mParentFolder;
  }

  /**   
   * Returns all the project files providers that this class represents.
   *
   * @return a project files provider
   */
  public ProjectFilesProvider[] getProjectFilesProviders()
  {
    if (mProjectFilesProviders == null)
    {
      return new ProjectFilesProvider[0];
    }

    ProjectFilesProvider[] result =
      new ProjectFilesProvider[mProjectFilesProviders.size()];
    result = (ProjectFilesProvider[]) mProjectFilesProviders.toArray(result);

    return result;
  }

  /**   
   * Allows you to set all the project files providers that this class
   * represents. This method removes any existing file providers that this
   * class represents and replaces them with the ones given.
   *
   * @param pProviders an array of project files providers
   */
  public void setProjectFilesProviders(ProjectFilesProvider[] pProviders)
  {
    if (mProjectFilesProviders == null)
    {
      mProjectFilesProviders = new ArrayList();
    }
    else
    {
      mProjectFilesProviders.clear();
    }

    if ((pProviders == null) || (pProviders.length < 1))
    {
      return;
    }

    for (int i = 0, sizei = pProviders.length; i < sizei; ++i)
    {
      mProjectFilesProviders.add(pProviders[i]);
    }
  }
}
