/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.core.portal;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.exoplatform.eclipse.core.exoservices.Configuration;
import org.exoplatform.eclipse.core.portal.model.INode;

/**
 * A class that holds all the portal settings. Notice that this class does not change the
 * value of the dirty boolean by itself, it is the responsibility of whoever is changing
 * and of its elements or subelements to inform this class by calling markDirty(). Also
 * this class will not set the dirty flag back to false by itself, someone else will have
 * to call markClean().
 * 
 * @author Hatim Khan
 */
public class PortalSettings
{
  private Configuration mRootConfiguration;
  private Configuration mPortalSessionConfiguration;
  private Configuration mDefaultPortalConfiguration;
  private Configuration mCustomPortalConfiguration;
  private String mJDKVersion;
  private String mInstallationDirectory;
  private String mContextRoot;
  private transient boolean mDirty = false;
  private transient Map mConfigurationChangedListenersMap = new HashMap();

  // key is an instance of Configuration, value is a Set of
  // IPortalSettingsConfigurationChangedListener

  /**
   * @return Returns the customPortalConfiguration.
   */
  public Configuration getCustomPortalConfiguration()
  {
    return mCustomPortalConfiguration;
  }

  /**
   * @param pCustomPortalConfiguration The customPortalConfiguration to set.
   */
  public void setCustomPortalConfiguration(Configuration pCustomPortalConfiguration)
  {
    mCustomPortalConfiguration = pCustomPortalConfiguration;
  }

  /**
   * @return Returns the defaultPortalConfiguration.
   */
  public Configuration getDefaultPortalConfiguration()
  {
    return mDefaultPortalConfiguration;
  }

  /**
   * @param pDefaultPortalConfiguration The defaultPortalConfiguration to set.
   */
  public void setDefaultPortalConfiguration(Configuration pDefaultPortalConfiguration)
  {
    mDefaultPortalConfiguration = pDefaultPortalConfiguration;
  }

  /**
   * @return Returns the portalSessionConfiguration.
   */
  public Configuration getPortalSessionConfiguration()
  {
    return mPortalSessionConfiguration;
  }

  /**
   * @param pPortalSessionConfiguration The portalSessionConfiguration to set.
   */
  public void setPortalSessionConfiguration(Configuration pPortalSessionConfiguration)
  {
    mPortalSessionConfiguration = pPortalSessionConfiguration;
  }

  /**
   * @return Returns the rootConfiguration.
   */
  public Configuration getRootConfiguration()
  {
    return mRootConfiguration;
  }

  /**
   * @param pRootConfiguration The rootConfiguration to set.
   */
  public void setRootConfiguration(Configuration pRootConfiguration)
  {
    mRootConfiguration = pRootConfiguration;
  }

  /**
   * @return Returns the installationDirectory.
   */
  public String getInstallationDirectory()
  {
    return mInstallationDirectory;
  }

  /**
   * @param pInstallationDirectory The installationDirectory to set.
   */
  public void setInstallationDirectory(String pInstallationDirectory)
  {
    mInstallationDirectory = pInstallationDirectory;
  }

  /**
   * @return Returns the jDKVersion.
   */
  public String getJDKVersion()
  {
    return mJDKVersion;
  }

  /**
   * @param pVersion The jDKVersion to set.
   */
  public void setJDKVersion(String pVersion)
  {
    mJDKVersion = pVersion;
  }

  /**
   * @return Returns the contextRoot.
   */
  public String getContextRoot()
  {
    return mContextRoot;
  }

  /**
   * @param pContextRoot The contextRoot to set.
   */
  public void setContextRoot(String pContextRoot)
  {
    mContextRoot = pContextRoot;
  }

  public void markDirty()
  {
    mDirty = true;
  }

  public boolean isDirty()
  {
    return mDirty;
  }

  public void markClean()
  {
    mDirty = false;
  }

  public void fireConfigurationElementAdded(Object pElement)
  {
    Configuration config = findConfiguration(pElement);
    if ((config == null) || (mConfigurationChangedListenersMap == null)) return;
    Set listeners = (Set) mConfigurationChangedListenersMap.get(config);
    if (listeners != null)
    {
      Iterator iterator = listeners.iterator();
      while (iterator.hasNext())
      {
        IPortalSettingsConfigurationChangedListener listener = (IPortalSettingsConfigurationChangedListener) iterator.next();
        listener.ConfigurationElementAdded(pElement);
      }
    }
  }

  public void fireConfigurationElementUpdated(Object pElement)
  {
    Configuration config = findConfiguration(pElement);
    if ((config == null) || (mConfigurationChangedListenersMap == null)) return;
    Set listeners = (Set) mConfigurationChangedListenersMap.get(config);
    if (listeners != null)
    {
      Iterator iterator = listeners.iterator();
      while (iterator.hasNext())
      {
        IPortalSettingsConfigurationChangedListener listener = (IPortalSettingsConfigurationChangedListener) iterator.next();
        listener.ConfigurationElementUpdated(pElement);
      }
    }
  }

  public void fireConfigurationElementRemoved(Object pParent, Object pChild)
  {
    Configuration config = findConfiguration(pParent);
    if ((config == null) || (mConfigurationChangedListenersMap == null)) return;
    Set listeners = (Set) mConfigurationChangedListenersMap.get(config);
    if (listeners != null)
    {
      Iterator iterator = listeners.iterator();
      while (iterator.hasNext())
      {
        IPortalSettingsConfigurationChangedListener listener = (IPortalSettingsConfigurationChangedListener) iterator.next();
        listener.ConfigurationElementRemoved(pParent, pChild);
      }
    }
  }

  public void addConfigurationChangedListener(Configuration pConfig,
      IPortalSettingsConfigurationChangedListener pListener)
  {
    if (pListener == null) return;
    if (!isManagedConfiguration(pConfig)) return;
    mConfigurationChangedListenersMap = addListener(mConfigurationChangedListenersMap, pConfig, pListener);
  }

  public void removeConfigurationChangedListener(Configuration pConfig,
      IPortalSettingsConfigurationChangedListener pListener)
  {
    if (pListener == null) return;
    if (!isManagedConfiguration(pConfig)) return;
    removeListener(mConfigurationChangedListenersMap, pConfig, pListener);
  }

  public void removeConfigurationChangedListener(IPortalSettingsConfigurationChangedListener pListener)
  {
    removeListener(mConfigurationChangedListenersMap, pListener);
  }

  public static Configuration findConfiguration(Object pNode)
  {
    if ((pNode == null) || !(pNode instanceof INode)) return null;
    if (pNode instanceof Configuration) return (Configuration) pNode;
    INode node = (INode) pNode;
    return findConfiguration(node.getParent());
  }

  private void removeListener(Map pMap, Object pListener)
  {
    if ((pMap == null) || (pListener == null)) return;
    Collection values = pMap.values();
    Iterator iterator = values.iterator();
    while (iterator.hasNext())
    {
      Set listeners = (Set) iterator.next();
      listeners.remove(pListener);
    }
  }

  private void removeListener(Map pMap, Object pKey, Object pListener)
  {
    if (pMap == null) return;
    Set listeners = (Set) pMap.get(pKey);
    if (listeners == null) return;
    listeners.remove(pListener);
  }

  private Map addListener(Map pMap, Object pKey, Object pListener)
  {
    Map result = (pMap != null) ? pMap : new HashMap();
    Set listeners = (Set) result.get(pKey);
    if (listeners == null) listeners = new HashSet();
    listeners.add(pListener);
    result.put(pKey, listeners);

    return result;
  }

  private boolean isManagedConfiguration(Configuration pConfig)
  {
    if (pConfig == null) return false;
    return ((pConfig == mRootConfiguration) || (pConfig == mDefaultPortalConfiguration)
        || (pConfig == mPortalSessionConfiguration) || (pConfig == mCustomPortalConfiguration));
  }
}