/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.warpackager;


import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.exoplatform.eclipse.core.common.IOverwriteQuery;

/**
 * Model for a WAR package. Describes a JAR package including a bunch of useful
 * options to generate a JAR file with a JAR writer. The idea and the code
 * came from org.eclipse.jdt.ui.jarpackager.JarPackageData
 */
public class WarPackageData
{
  private String mManifestVersion;

  /*
   * Leaf elements (no containers) to export
   */
  private Object[] mElements; // inside workspace
  private IPath mWarLocation; // external location
  private boolean mCompress;

  /*
   * A normal WAR has a manifest (mUsesManifest is true)
   * The manifest will be generated (mGenerateManifest is true)
   */
  private boolean mUsesManifest;
  private boolean mGenerateManifest;
  private String mComment; // the WAR comment

  // The provider for the manifest file
  private IManifestProvider mManifestProvider;

  /**
   * The constructor method.
   */
  public WarPackageData()
  {
    setCompress(true);
    setWarLocation(new Path("")); //$NON-NLS-1$
    setUsesManifest(true);
    setGenerateManifest(true);
  }

  // ----------- Accessors -----------

  /**   
   * Tells whether the WAR is compressed or not.
   *
   * @return <code>true</code> if the WAR is compressed
   */
  public boolean isCompressed()
  {
    return mCompress;
  }

  /**   
   * Set whether the WAR is compressed or not.
   *
   * @param pState a boolean indicating the new state
   */
  public void setCompress(boolean pState)
  {
    mCompress = pState;
  }

  /**   
   * Gets the location of the WAR file. This path is normally external to the
   * workspace.
   *
   * @return the path representing the location of the WAR file
   */
  public IPath getWarLocation()
  {
    return mWarLocation;
  }

  /**   
   * Set the WAR file location.
   *
   * @param pWarLocation a path denoting the location of the JAR file
   */
  public void setWarLocation(IPath pWarLocation)
  {
    mWarLocation = pWarLocation;
  }

  /**   
   * Tells whether the manifest file must be generated.
   *
   * @return <code>true</code> if the manifest has to be generated
   */
  public boolean isManifestGenerated()
  {
    return mGenerateManifest;
  }

  /**   
   * Set whether a manifest must be generated or not.
   *
   * @param pState the new state
   */
  public void setGenerateManifest(boolean pState)
  {
    mGenerateManifest = pState;
  }

  /**   
   * Gets the manifest version.
   *
   * @return a string containing the manifest version
   */
  public String getManifestVersion()
  {
    if (mManifestVersion == null)
    {
      return "1.0"; //$NON-NLS-1$
    }

    return mManifestVersion;
  }

  /**   
   * Set the manifest version.
   *
   * @param pManifestVersion the string which contains the manifest version
   */
  public void setManifestVersion(String pManifestVersion)
  {
    mManifestVersion = pManifestVersion;
  }

  /**   
   * Answers whether a manifest must be included in the JAR.
   *
   * @return <code>true</code> if a manifest has to be included
   */
  public boolean usesManifest()
  {
    return mUsesManifest;
  }

  /**   
   * Set whether a manifest must be included in the JAR.
   *
   * @param pState the new state
   */
  public void setUsesManifest(boolean pState)
  {
    mUsesManifest = pState;
  }

  /**   
   * Gets the manifest provider for this JAR package.
   *
   * @return the IManifestProvider
   */
  public IManifestProvider getManifestProvider()
  {
    if (mManifestProvider == null)
    {
      mManifestProvider = new ManifestProvider();
    }

    return mManifestProvider;
  }

  /**   
   * Set the manifest provider.
   *
   * @param pManifestProvider the ManifestProvider to set
   */
  public void setManifestProvider(IManifestProvider pManifestProvider)
  {
    mManifestProvider = pManifestProvider;
  }

  /**   
   * Returns the elements which will be exported. These elements are leaf
   * objects e.g. <code>IFile</code> and not containers.
   *
   * @return an array of leaf objects
   */
  public Object[] getElements()
  {
    if (mElements == null)
    {
      setElements(new Object[0]);
    }

    return mElements;
  }

  /**   
   * Set the elements which will be exported.  These elements are leaf objects
   * e.g. <code>IFile</code>. and not containers.
   *
   * @param pElements an array with leaf objects
   */
  public void setElements(Object[] pElements)
  {
    mElements = pElements;
  }

  /**   
   * Returns the WAR's comment.
   *
   * @return the comment string or <code>null</code> if the WAR does not
   *         contain a comment
   */
  public String getComment()
  {
    return mComment;
  }

  /**   
   * Sets the WAR's comment.
   *
   * @param pComment a string or <code>null</code> if the WAR does not contain
   *        a comment
   */
  public void setComment(String pComment)
  {
    mComment = pComment;
  }

  // ----------- Utility methods -----------  

  /**   
   * Creates and returns a WarWriter for this WAR package.
   *
   * @param pOverwriteQuery
   *
   * @return a WarWriter
   *
   * @throws CoreException
   *
   * @see WarWriter
   */
  public WarWriter createWarWriter(IOverwriteQuery pOverwriteQuery)
    throws CoreException
  {
    return new WarWriter(this, pOverwriteQuery);
  }

  /**   
   * Tells whether this WAR package data can be used to generate a valid WAR.
   *
   * @return <code>true</code> if the WAR Package info is valid
   */
  public boolean isValid()
  {
    return (getElements() != null) && (getElements().length > 0)
    && (getWarLocation() != null) && (isManifestAccessible());
  }

  /**   
   * Tells whether a manifest is available.
   *
   * @return <code>true</code> if the manifest is generated or the provided one
   *         is accessible
   */
  public boolean isManifestAccessible()
  {
    if (isManifestGenerated())
    {
      return true;
    }
    else
    {
      return false;
    }
  }
}
