/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.xmlvalidator;


import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.exoplatform.eclipse.core.extension.IExtensionProcessor;
import org.exoplatform.eclipse.core.extension.ProcessExtension;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * Provides access to all the portlet deployment descriptor validators
 * contributed via the extension point
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class PortletDescriptorValidatorProvider implements IExtensionProcessor
{
  public static final String CLASS_VERSION = "$Id: PortletDescriptorValidatorProvider.java,v 1.1 2004/04/19 03:45:50 hatimk Exp $"; //$NON-NLS-1$  
  public static final String EXTENSION_POINT_ID =
    ExoCorePlugin.PLUGIN_ID + ".exoPortletDescriptorValidators"; //$NON-NLS-1$  
  public static final String EXTENSION_ATTRIBUTE_VERSION = "portlet-version"; //$NON-NLS-1$
  public static final String EXTENSION_ATTRIBUTE_EXOVERSION = "exo-version"; //$NON-NLS-1$  
  public static final String EXTENSION_ATTRIBUTE_DEFAULTVALIDATOR =
    "default-validator"; //$NON-NLS-1$  
  public static final String EXTENSION_ATTRIBUTE_ORDER = "order"; //$NON-NLS-1$
  public static final String EXTENSION_ATTRIBUTE_ORDER_DEFAULT = "1"; //$NON-NLS-1$  
  public static final String EXTENSION_ATTRIBUTE_DEFAULTVALIDATOR_DEFAULT =
    "false"; //$NON-NLS-1$            
  private Map mValidatorsMap = null;
  private IPortletDescriptorValidator[] mValidators = null;

  /**   
   * Provides all available portlet deployment descriptor validators coming
   * from the extension point.  The validators will be sorted according to
   * their spec support version. Validators with the same spec support version
   * will be filtered out according to their order, only the lowest order is
   * considered.
   *
   * @return all validators contributed via the extension point
   */
  public IPortletDescriptorValidator[] getValidators()
  {
    if (mValidators != null)
    {
      return mValidators;
    }

    Map validatorsMap = getValidatorsMap();
    Iterator iterator = validatorsMap.values().iterator();
    List validators = new ArrayList();

    while (iterator.hasNext())
    {
      validators.add(iterator.next());
    }

    Collections.sort(validators,
      new Comparator()
      {
        public int compare(Object pLeft, Object pRight)
        {
          IPortletDescriptorValidator vLeft =
            (IPortletDescriptorValidator) pLeft;
          IPortletDescriptorValidator vRight =
            (IPortletDescriptorValidator) pRight;
          String verLeft = vLeft.getSupportedSpecVersion();
          String verRight = vRight.getSupportedSpecVersion();
          if (verLeft == null)
          {
            return -1;
          }
          if (verRight == null)
          {
            return 1;
          }

          double versionL = 0.0;
          double versionR = 0.0;
          try
          {
            versionL = Double.parseDouble(verLeft);
            versionR = Double.parseDouble(verRight);
          }
          catch (Exception e)
          {
          }

          if (versionL == versionR)
          {
            return 0;
          }

          return (versionL > versionR) ? 1 : (-1);
        }
      });

    IPortletDescriptorValidator[] validtorArray =
      new IPortletDescriptorValidator[validators.size()];
    validtorArray =
      (IPortletDescriptorValidator[]) validators.toArray(validtorArray);

    return validtorArray;
  }

  /**   
   * Considering the list of all available validtors returned by the
   * getValidators() method. This method returns the first validtor that is
   * declared as default.
   *
   * @return the default portlet deployment descriptor validator
   */
  public IPortletDescriptorValidator getDefaultValidator()
  {
    IPortletDescriptorValidator[] validtors = getValidators();
    for (int i = 0, sizei = validtors.length; i < sizei; ++i)
    {
      if (validtors[i].isDefault())
      {
        return validtors[i];
      }
    }

    return null;
  }

  /**   
   * Returns the validator associated with the given portlet spec version.  If
   * you want to get the default validator if no validator associated with the
   * given version is found, then set the return default flag to true.
   * Otherwise if no validator is found, null is returned.
   *
   * @param pSpecVersion the portlet.xml version
   * @param pReturnDefault true if you want the default validator to be
   *        returned if no other validtor is found.
   *
   * @return the validator associated with the given version
   */
  public IPortletDescriptorValidator getValidator(String pSpecVersion,
    boolean pReturnDefault)
  {
    if ((pSpecVersion == null) || (pSpecVersion.length() < 1))
    {
      return getDefaultValidator();
    }

    Map validatorsMap = getValidatorsMap();
    IPortletDescriptorValidator validator =
      (IPortletDescriptorValidator) validatorsMap.get(pSpecVersion);

    if ((validator == null) && (pReturnDefault))
    {
      validator = getDefaultValidator();
    }

    return validator;
  }

  /**   
   * @see org.exoplatform.eclipse.core.extension.IExtensionProcessor#process(org.eclipse.core.runtime.IExtension,
   *      org.eclipse.core.runtime.IConfigurationElement, java.lang.Object)
   */
  public void process(IExtension pExtension, IConfigurationElement pMember,
    Object pResultCollector)
  {
    if (pResultCollector == null)
    {
      return;
    }

    Map resultCollector = (Map) pResultCollector;

    IPortletDescriptorValidator currentDelegator = createDelegator(pMember);
    IPortletDescriptorValidator savedDelegator =
      (IPortletDescriptorValidator) resultCollector.get(currentDelegator
        .getSupportedSpecVersion());
    IPortletDescriptorValidator winnerDelegator = currentDelegator;
    if (savedDelegator != null)
    {
      // remember the less the order is the more important it is
      if (currentDelegator.getOrder() > savedDelegator.getOrder())
      {
        winnerDelegator = savedDelegator;
      }
    }

    resultCollector.put(winnerDelegator.getSupportedSpecVersion(),
      winnerDelegator);
  }

  private Map getValidatorsMap()
  {
    if (mValidatorsMap != null)
    {
      return mValidatorsMap;
    }

    Map resultCollector = new HashMap();
    ProcessExtension.process(EXTENSION_POINT_ID, this, resultCollector);

    mValidatorsMap = resultCollector;

    return mValidatorsMap;
  }

  private IPortletDescriptorValidator createDelegator(
    IConfigurationElement pElement)
  {
    String version = pElement.getAttribute(EXTENSION_ATTRIBUTE_VERSION);
    String exoVersion = pElement.getAttribute(EXTENSION_ATTRIBUTE_EXOVERSION);
    String defaultString =
      pElement.getAttribute(EXTENSION_ATTRIBUTE_DEFAULTVALIDATOR);
    String orderString = pElement.getAttribute(EXTENSION_ATTRIBUTE_ORDER);

    if (orderString == null)
    {
      orderString = EXTENSION_ATTRIBUTE_ORDER_DEFAULT;
    }

    int order = Integer.parseInt(orderString);

    if (defaultString == null)
    {
      defaultString = EXTENSION_ATTRIBUTE_DEFAULTVALIDATOR_DEFAULT;
    }

    boolean defaultValidator = (new Boolean(defaultString)).booleanValue();

    // now that all the attributes are extracted, create a delegator
    PortletDescriptorDelegator delegator =
      new PortletDescriptorDelegator(pElement);
    delegator.setDefault(defaultValidator);
    delegator.setOrder(order);
    delegator.setSupportedSpecVersion(version);
    delegator.setSupportedExoVersion(exoVersion);

    return (IPortletDescriptorValidator) delegator;
  }
}
