/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.xmlvalidator.helper;


import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.exoplatform.eclipse.core.xmlvalidator.PortletValidationProblemReporter;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * This class understands SAX events and passes issues found to the given
 * reporter.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class ExoSAXHandler extends DefaultHandler
{
  public static final String CLASS_VERSION = "$Id: ExoSAXHandler.java,v 1.1 2004/04/19 03:45:50 hatimk Exp $"; //$NON-NLS-1$
  private PortletValidationProblemReporter mReporter;
  private IProgressMonitor mMonitor;

  /**
   * The constructor method.
   *
   * @param pReporter the reporter that collects the problems
   * @param pMonitor to report progress
   */
  public ExoSAXHandler(PortletValidationProblemReporter pReporter,
    IProgressMonitor pMonitor)
  {
    super();
    mReporter = pReporter;
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }
    mMonitor = pMonitor;
    mMonitor.beginTask("", IProgressMonitor.UNKNOWN);
  }

  /**   
   * @see org.xml.sax.helpers.DefaultHandler
   */
  public void warning(SAXParseException pWarning) throws SAXException
  {
    if (mReporter != null)
    {
      mReporter.addWarning(pWarning.getMessage(), pWarning.getLineNumber(),
        pWarning.getColumnNumber());
    }
  }

  /**   
   * @see org.xml.sax.helpers.DefaultHandler
   */
  public void error(SAXParseException pError) throws SAXException
  {
    if (mReporter != null)
    {
      mReporter.addError(fixMessage(pError.getMessage()),
        pError.getLineNumber(), pError.getColumnNumber());
    }
  }

  /**   
   * @see org.xml.sax.helpers.DefaultHandler
   */
  public void fatalError(SAXParseException pError) throws SAXException
  {
    if (mReporter != null)
    {
      mReporter.addError(pError.getMessage(), pError.getLineNumber(),
        pError.getColumnNumber());
    }
  }

  /**   
   * @see org.xml.sax.helpers.DefaultHandler#startElement(java.lang.String,
   *      java.lang.String, java.lang.String, org.xml.sax.Attributes)
   */
  public void startElement(String uri, String localName, String qName,
    Attributes attributes) throws SAXException
  {
    if (mMonitor.isCanceled())
    {
      throw new SAXException(" user stopped validation operation.");
    }
    super.startElement(uri, localName, qName, attributes);
    mMonitor.subTask("processing tag <" + localName + ">");
    mMonitor.worked(5);
  }

  /**   
   * This method is used to just make the error messages look better to the
   * user, by removing the cvs-complex-type prefix that Xerces add to some of
   * the error messages.
   *
   * @param pMessage the messages to fix
   *
   * @return a message with the cvs-complex-type prefix removed
   */
  private String fixMessage(String pMessage)
  {
    if (pMessage == null)
    {
      return null;
    }

    String result = pMessage;

    if (pMessage.startsWith("cvc-complex-type."))
    {
      int index = pMessage.indexOf(":");
      if ((index > 0) && ((index + 1) < pMessage.length()))
      {
        String tempMessage = pMessage.substring(index + 1);
        if (tempMessage.length() > 4)
        {
          result = tempMessage;
        }
      }
    }

    if (result.startsWith(" "))
    {
      result = result.substring(1);
    }

    return result;
  }
}
