/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.xmlvalidator.helper;

import org.eclipse.core.resources.IFile;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

/**
 * This class provides utility methods to extract certain information
 * about/from the given portlet.xml. An example would be the namespace URI of
 * the portlet-app element and the version attribute value for the portlet-app
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class PortletDeploymentDescriptorInspector
{
  public static final String CLASS_VERSION = "$Id: PortletDeploymentDescriptorInspector.java,v 1.1 2004/04/19 03:45:50 hatimk Exp $"; //$NON-NLS-1$
  private static final String TAG_PORTLET_APP = "portlet-app"; //$NON-NLS-1$
  private static final String VERSION_ATTRIBUTE = "version"; //$NON-NLS-1$

  /**   
   * This method inspects the portlet-app element of the given portlet.xml and
   * returns the namespace URI bound to it. In case of any exception, or if
   * the namespace URI can not be determined, this method returns null.
   *
   * @param pFile must be the portlet.xml file.
   *
   * @return the namespace URI of the portlet-app element in the portlet.xml
   */
  public static String getNamespaceURI(IFile pFile)
  {
    InputStream in = null;
    PortletDeploymentDescriptorInspector instance =
      new PortletDeploymentDescriptorInspector();
    PortletNamespaceHandler handler = instance.new PortletNamespaceHandler();
    try
    {
      in = pFile.getContents();

      SAXParserFactory factory = SAXParserFactory.newInstance();
      factory.setNamespaceAware(true);

      // Parse the input 
      SAXParser saxParser = factory.newSAXParser();
      saxParser.parse(in, handler);
    }
    catch (Exception e)
    {
    }
    finally
    {
      try
      {
        if (in != null)
        {
          in.close();
        }
      }
      catch (IOException e)
      {
      }
    }

    return handler.getNamespaceURI();
  }

  /**   
   * This method inspects the portlet-app element of the given portlet.xml and
   * returns the value of the version attribute. In case of any exception, or
   * if the version can not be determined, this method returns null.
   *
   * @param pFile must be the portlet.xml file.
   *
   * @return the value of the version attribute of the portlet-app element in
   *         the portlet.xml
   */
  public static String getVersion(IFile pFile)
  {
    InputStream in = null;
    PortletDeploymentDescriptorInspector instance =
      new PortletDeploymentDescriptorInspector();
    PortletVersionHandler handler = instance.new PortletVersionHandler();
    try
    {
      in = pFile.getContents();

      SAXParserFactory factory = SAXParserFactory.newInstance();
      factory.setNamespaceAware(true);

      // Parse the input 
      SAXParser saxParser = factory.newSAXParser();
      saxParser.parse(in, handler);
    }
    catch (Exception e)
    {
    }
    finally
    {
      try
      {
        if (in != null)
        {
          in.close();
        }
      }
      catch (IOException e)
      {
      }
    }

    return handler.getVersion();
  }

  protected class PortletNamespaceHandler extends DefaultHandler
  {
    private String mNamespaceURI = null;

    /**   
     * @see org.xml.sax.helpers.DefaultHandler#startElement(java.lang.String,
     *      java.lang.String, java.lang.String, org.xml.sax.Attributes)
     */
    public void startElement(String pNamespaceURI, String pLocalName,
      String pQualifiedName, Attributes pAttributes) throws SAXException
    {
      super.startElement(pNamespaceURI, pLocalName, pQualifiedName, pAttributes);

      // the very first element should be portlet-app otherwise we can't continue
      if (TAG_PORTLET_APP.equalsIgnoreCase(pLocalName))
      {
        mNamespaceURI = pNamespaceURI;

        // since we got what we want, we should tell the parser to stop parsing
        // I don't know of any other way, other than throwing an exception to
        // stop the parsing.
        throw new SAXException(
          "Found the portlet-app tag. Should stop processing.");
      }
      else
      {
        throw new SAXException(
          "Did not find the portlet-app element. Should stop processing.");
      }
    }

    public String getNamespaceURI()
    {
      return mNamespaceURI;
    }
  }

  protected class PortletVersionHandler extends DefaultHandler
  {
    private String mVersion = null;

    /**   
     * @see org.xml.sax.helpers.DefaultHandler#startElement(java.lang.String,
     *      java.lang.String, java.lang.String, org.xml.sax.Attributes)
     */
    public void startElement(String pNamespaceURI, String pLocalName,
      String pQualifiedName, Attributes pAttributes) throws SAXException
    {
      super.startElement(pNamespaceURI, pLocalName, pQualifiedName, pAttributes);

      // the very first element should be portlet-app otherwise we can't continue
      if (TAG_PORTLET_APP.equalsIgnoreCase(pLocalName))
      {
        // now check the attributes
        if (pAttributes!= null)
        {
          for(int i=0, sizei= pAttributes.getLength(); i < sizei; ++i)
          {
            String localName = pAttributes.getLocalName(i);
            if (VERSION_ATTRIBUTE.equalsIgnoreCase(localName))
            {
              mVersion = pAttributes.getValue(i);
              break;
            }
          }
        }

        // since we got what we want, we should tell the parser to stop parsing
        // I don't know of any other way, other than throwing an exception to
        // stop the parsing.
        throw new SAXException(
          "Found the portlet-app tag. Should stop processing.");
      }
      else
      {
        throw new SAXException(
          "Did not find the portlet-app element. Should stop processing.");
      }
    }

    public String getVersion()
    {
      return mVersion;
    }
  }
}
