/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.internal.core.library;


import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.PluginVersionIdentifier;
import org.eclipse.jdt.core.ClasspathContainerInitializer;
import org.eclipse.jdt.core.IClasspathContainer;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.exoplatform.eclipse.core.extension.IExtensionProcessor;
import org.exoplatform.eclipse.core.extension.ProcessExtension;
import org.exoplatform.eclipse.core.library.ExoPortletLibraryContainer;
import org.osgi.framework.Bundle;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * This class prepares and exposes all available eXo portlet library containers. All
 * library entries are contributed via the extension point 'exoPortletLibraryProviders'.
 * This class also answers to the JavaCore (JDT plugin) questions about resolving library
 * containers that starts with the 'exo.platform.core.EXO_PORTLET_LIBRARY' segement.
 * 
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class ExoPortletLibraryInitializer extends ClasspathContainerInitializer implements
    IExtensionProcessor
{
  public static final String CLASS_VERSION = "$Id: ExoPortletLibraryInitializer.java,v 1.1 2004/04/19 17:48:25 hatimk Exp $"; //$NON-NLS-1$
  public static final String PORTLET_LIBRARY_DESC_PREFIX = "eXo Portlet Library";
  public static final String EXO_PORTLET_LIBRARY_PATH = ExoCorePlugin.PLUGIN_ID + ".EXO_PORTLET_LIBRARY"; //$NON-NLS-1$
  public static final String EXTENSION_POINT_ID = ExoCorePlugin.PLUGIN_ID + ".exoPortletLibraryProviders"; //$NON-NLS-1$  
  public static final String EXTENSION_ATTRIBUTE_VALUE = "value"; //$NON-NLS-1$
  public static final String EXTENSION_ATTRIBUTE_LOCATION = "location"; //$NON-NLS-1$  
  public static final String EXTENSION_ATTRIBUTE_SRC_LOCATION = "src-location"; //$NON-NLS-1$  
  public static final String EXTENSION_ATTRIBUTE_ORDER = "order"; //$NON-NLS-1$
  public static final String EXTENSION_ATTRIBUTE_ORDER_DEFAULT = "1"; //$NON-NLS-1$          
  public static final String EXTENSION_NAME_LIBRARY = "library"; //$NON-NLS-1$  
  private ExoPortletLibraryContainer[] mExoPortletLibraries = null;

  /**
   * @see org.eclipse.jdt.core.ClasspathContainerInitializer#initialize(org.eclipse.core.runtime.IPath,
   *      org.eclipse.jdt.core.IJavaProject)
   */
  public void initialize(IPath pContainerPath, IJavaProject pProject) throws CoreException
  {
    boolean initialized = false;

    ExoPortletLibraryContainer container = getPortletLibrary(pContainerPath);

    if (container == null)
    {
      return;
    }

    JavaCore.setClasspathContainer(pContainerPath, new IJavaProject[] { pProject }, new IClasspathContainer[] { container }, null);
  }

  /**
   * Given the library path (example, org.exoplatform.eclipse.core.EXO_PORTLET_LIBRARY),
   * returns its ExoPortletLibraryContainer representation. If the first segement matches,
   * but not the second one (or there was no second segment in the given path) then return
   * the latest version of the library container that is available.
   * 
   * @param pContainerPath the exo portlet library path
   * 
   * @return an instance of ExoPortletLibraryContainer that represents the given path or
   *         null if the first segement does not match
   */
  public ExoPortletLibraryContainer getPortletLibrary(IPath pContainerPath)
  {
    ExoPortletLibraryContainer result = null;
    boolean initialized = false;

    if ((pContainerPath == null) || (pContainerPath.segmentCount() < 1))
    {
      return result;
    }

    if ((pContainerPath.segment(0).equals(EXO_PORTLET_LIBRARY_PATH)))
    {
      ExoPortletLibraryContainer[] libraries = getPortletLibraries();

      if ((libraries == null) || (libraries.length < 1))
      {
        return result;
      }

      for (int i = 0, sizei = libraries.length; i < sizei; ++i)
      {
        IPath libraryPath = libraries[i].getPath();

        if (pContainerPath.equals(libraryPath))
        {
          result = libraries[i];
          initialized = true;
        }
      }

      // if the first segement matches but there is no match for the second
      // segement, then the latest version of the library container will be
      // assigned to the given project
      if (!initialized)
      {
        result = libraries[0];
        initialized = true;
      }
    }

    return result;
  }

  /**
   * Returns the description of the library container. This method returns a description
   * by inspecting the segements that are available from the given path. An example of the
   * description is 'eXo Library [1.0.0.beta4]'. If the path has only one segement that
   * matches the library id, then the description returned will be that of the latest
   * library available.
   * 
   * @see org.eclipse.jdt.core.ClasspathContainerInitializer#getDescription(org.eclipse.core.runtime.IPath,
   *      org.eclipse.jdt.core.IJavaProject)
   */
  public String getDescription(IPath pContainerPath, IJavaProject pProject)
  {
    String description = super.getDescription(pContainerPath, pProject);

    if ((pContainerPath == null) || (pContainerPath.segmentCount() < 1)
        || !(pContainerPath.segment(0).equals(EXO_PORTLET_LIBRARY_PATH)))
    {
      return description;
    }

    int size = pContainerPath.segmentCount();

    if (size == 2)
    {
      description = PORTLET_LIBRARY_DESC_PREFIX + " [" + pContainerPath.segment(1) + "]";
    }
    else
    {
      ExoPortletLibraryContainer[] containers = getPortletLibraries();

      if (containers.length > 0)
      {
        description = containers[0].getDescription();
      }
      else
      {
        description = PORTLET_LIBRARY_DESC_PREFIX;
      }
    }

    return description;
  }

  /**
   * @see org.eclipse.jdt.core.ClasspathContainerInitializer#canUpdateClasspathContainer(org.eclipse.core.runtime.IPath,
   *      org.eclipse.jdt.core.IJavaProject)
   */
  public boolean canUpdateClasspathContainer(IPath containerPath, IJavaProject project)
  {
    return false;
  }

  /**
   * @see org.exoplatform.eclipse.core.extension.IExtensionProcessor#process(org.eclipse.core.runtime.IExtension,
   *      org.eclipse.core.runtime.IConfigurationElement, Object)
   */
  public void process(IExtension pExtension, IConfigurationElement pMember, Object pResultCollector)
  {
    if (pResultCollector == null)
    {
      return;
    }

    Map resultCollector = (Map) pResultCollector;

    String version = pMember.getAttribute(EXTENSION_ATTRIBUTE_VALUE);
    IConfigurationElement[] elements = pMember.getChildren(EXTENSION_NAME_LIBRARY);

    List classPaths = (List) resultCollector.get(version);

    if (classPaths == null)
    {
      classPaths = new ArrayList();
      resultCollector.put(version, classPaths);
    }

    // For each member of the library-version ...
    for (int j = 0; j < elements.length; j++)
    {
      IConfigurationElement element = elements[j];
      PortletLibraryEntry entry = getPortletLibraryEntry(pExtension, element);

      if (entry != null)
      {
        classPaths.add(entry);
      }
    }
  }

  /**
   * Returns all available eXo portlet library containers. All library entries are
   * contributed via the extension point 'exoPortletLibraryProviders'
   * 
   * @return available eXo portlet libraries
   */
  public ExoPortletLibraryContainer[] getPortletLibraries()
  {
    if (mExoPortletLibraries != null)
    {
      return mExoPortletLibraries;
    }

    Map resultCollector = new HashMap();
    ProcessExtension.process(EXTENSION_POINT_ID, this, resultCollector);

    List containerList = new ArrayList();

    // the keys in the result collector are nothing but strings that represent
    // version numbers, while the value for each key is a list of LibraryEntry[]
    Iterator iterator = resultCollector.keySet().iterator();

    while (iterator.hasNext())
    {
      String version = (String) iterator.next();
      List classPaths = (List) resultCollector.get(version);

      if ((classPaths == null) || (classPaths.size() < 1))
      {
        continue;
      }

      // the classPaths should contain elements of type LibraryEntry
      // we need to sort them according to the order
      Collections.sort(classPaths, new Comparator()
      {
        public int compare(Object pLeft, Object pRight)
        {
          Integer leftInt = new Integer(((PortletLibraryEntry) pLeft).getOrder());
          Integer rightInt = new Integer(((PortletLibraryEntry) pRight).getOrder());

          return leftInt.compareTo(rightInt);
        }
      });

      // now that the library entries are sorted per version, create
      // the corresponding exo library container
      IClasspathEntry[] classpathEntryArray = new IClasspathEntry[classPaths.size()];
      Iterator iterator2 = classPaths.iterator();

      for (int i = 0; iterator2.hasNext(); ++i)
      {
        classpathEntryArray[i] = ((PortletLibraryEntry) iterator2.next()).getClasspathEntry();
      }

      String libraryContainerDesc = PORTLET_LIBRARY_DESC_PREFIX + " [" + version + "]";
      ExoPortletLibraryContainer container = new ExoPortletLibraryContainer(
          new Path(EXO_PORTLET_LIBRARY_PATH).append(version), libraryContainerDesc, classpathEntryArray,
          version);
      containerList.add(container);
    }

    // containerList now contains a list of ExoPortletLibraryContainer. However,
    // before lumping them as ExoPortletLibraryContainer[], we need to sort them
    // according to their version number. Because these version numbers should
    // follow plugin version numbers convention. We will use the plugin version
    // identifier class. Containers are sorted according to the latest version.
    // That is, the container with the latest version will appear first in the
    // list.
    Collections.sort(containerList, new Comparator()
    {
      public int compare(Object pLeft, Object pRight)
      {
        PluginVersionIdentifier leftVer = new PluginVersionIdentifier(
            ((ExoPortletLibraryContainer) pLeft).getVersion());
        PluginVersionIdentifier rightVer = new PluginVersionIdentifier(
            ((ExoPortletLibraryContainer) pRight).getVersion());

        // version numbers that don't have a qualifier (such as beta) should
        // be considered a higher version than those with qualifier. This
        // is different from the default behaviour of the plugin version
        // identifier. So, we will insert the character "z" in front of those
        // versions that don't have it to force them to be higher.
        String leftVerQ = leftVer.getQualifierComponent();

        if (leftVerQ.length() < 1)
        {
          leftVer = new PluginVersionIdentifier(leftVer.toString() + ".z");
        }

        String rightVerQ = rightVer.getQualifierComponent();

        if (rightVerQ.length() < 1)
        {
          rightVer = new PluginVersionIdentifier(rightVer.toString() + ".z");
        }

        int result = 0;

        if (leftVer.isPerfect(rightVer))
        {
          result = 0;
        }
        else if (leftVer.isGreaterThan(rightVer))
        {
          result = 1;
        }
        else
        {
          result = -1;
        }

        return -1 * result;
      }
    });

    ExoPortletLibraryContainer[] containerArray = new ExoPortletLibraryContainer[containerList.size()];
    containerArray = (ExoPortletLibraryContainer[]) containerList.toArray(containerArray);

    mExoPortletLibraries = containerArray;

    return mExoPortletLibraries;
  }

  private PortletLibraryEntry getPortletLibraryEntry(IExtension pExtension, IConfigurationElement pMember)
  {
    PortletLibraryEntry result = null;

    String location = pMember.getAttribute(EXTENSION_ATTRIBUTE_LOCATION);
    String srcLocation = pMember.getAttribute(EXTENSION_ATTRIBUTE_SRC_LOCATION);
    String orderString = pMember.getAttribute(EXTENSION_ATTRIBUTE_ORDER);

    try
    {
      String bundleName = pExtension.getNamespace();
      Bundle bundle = Platform.getBundle(bundleName);
      URL fileURL = bundle.getEntry(new Path(location).toString());
      String jarLocation = Platform.asLocalURL(fileURL).getFile();

      // since the src-location attribute is optional, handle it correctly
      IPath sourcePath = null;

      if ((srcLocation != null) && (srcLocation.length() > 0))
      {
        URL srcURL = bundle.getEntry(new Path(srcLocation).toString());
        sourcePath = new Path(Platform.asLocalURL(srcURL).getFile());
      }

      // since the order attribute is optional, handle it correctly
      int order = Integer.parseInt(EXTENSION_ATTRIBUTE_ORDER_DEFAULT);

      if ((orderString != null) && (orderString.length() > 0))
      {
        order = Integer.parseInt(orderString);
      }

      IClasspathEntry ce = JavaCore.newLibraryEntry(new Path(jarLocation), sourcePath, null);
      result = new PortletLibraryEntry(ce, order);
    }
    catch (Exception e)
    {
      ExoCorePlugin.logError("Encountered an error while trying to process the libraries entries for the exo portlet library providers extension", e);
    }

    return result;
  }

  private class PortletLibraryEntry
  {
    private IClasspathEntry mClasspathEntry;
    private int mOrder;

    public PortletLibraryEntry(IClasspathEntry pClasspathEntry, int pOrder)
    {
      mClasspathEntry = pClasspathEntry;
      mOrder = pOrder;
    }

    public int getOrder()
    {
      return mOrder;
    }

    public IClasspathEntry getClasspathEntry()
    {
      return mClasspathEntry;
    }
  }
}