/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.internal.ui.decorator;


import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.IDecoration;
import org.eclipse.jface.viewers.ILightweightLabelDecorator;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.LabelProviderChangedEvent;
import org.eclipse.swt.widgets.Display;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.exoplatform.eclipse.core.IPortletProject;
import org.exoplatform.eclipse.core.IPreferencesConstant;
import org.exoplatform.eclipse.ui.ExoUIPlugin;
import org.exoplatform.eclipse.ui.ExoUIPluginImages;

import java.util.HashMap;
import java.util.Map;

/**
 * Adds an icon decoration to web folders in a portlet project, and addes an icon to
 * display the version of the portlet deployment descriptor. If unable to determine the
 * version then no decoration is provided. This decorator observes changes to the related
 * resources (such as the webfolder, and portlet.xml) and informs its listeners (for
 * example, the decorator manager) about the changes if any. This allows this decorator to
 * react to changes to its deocrated resources, such as a modification to the portlet.xml.
 * 
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class PortletResourcesDecorator extends LabelProvider implements ILightweightLabelDecorator,
    IResourceChangeListener
{
  public static final String CLASS_VERSION = "$Id: PortletResourcesDecorator.java,v 1.1 2004/04/19 03:37:22 hatimk Exp $"; //$NON-NLS-1$
  private static Map mImageMap;

  static
  {
    mImageMap = new HashMap();
    mImageMap.put("1.0", ExoUIPluginImages.DESC_OVR_PORTLET_V10);
    mImageMap.put("1.1", ExoUIPluginImages.DESC_OVR_PORTLET_V11);
    mImageMap.put("2.0", ExoUIPluginImages.DESC_OVR_PORTLET_V20);
  }

  private IResourceDeltaVisitor mVisitor = new IResourceDeltaVisitor()
  {
    public boolean visit(IResourceDelta pDelta)
    {
      IResource resource = pDelta.getResource();
      if (resource == null)
      {
        return true;
      }

      if (resource instanceof IProject)
      {
        IProject project = (IProject) resource;
        try
        {
          if ((project.isOpen()) && (!project.hasNature(ExoCorePlugin.PORTLET_NATURE_ID)))
          {
            // simply, we are not interested in none portlet projects
            return false;
          }
        }
        catch (CoreException e)
        {
          // the project not exist, so ignore it
          return false;
        }
      }
      else if (resource instanceof IFolder)
      {
        IFolder folder = (IFolder) resource;
        String name = folder.getName();
        if (name.equals(IPreferencesConstant.PROJECT_SCOPE_DIRECTORY))
        {
          // this should detect changes made to the context root value
          fireLabelChanged();
          
          // no need to visit more resources
          return false;
        }
      }
      else if (resource instanceof IFile)
      {
        IFile file = (IFile) resource;
        String name = file.getName();
        if ((name.equals("portlet.xml")))        
        {
          // ok, we found the change
          fireLabelChanged();

          // no need to visit more resources
          return false;
        }
      }

      return true;
    }
  };

  /**
   * The constructor method.
   */
  public PortletResourcesDecorator()
  {
    super();
    ResourcesPlugin.getWorkspace().addResourceChangeListener(this, IResourceChangeEvent.POST_CHANGE);
  }

  /**
   * @see org.eclipse.core.resources.IResourceChangeListener#resourceChanged(org.eclipse.core.resources.IResourceChangeEvent)
   */
  public void resourceChanged(IResourceChangeEvent pEvent)
  {
    //we are only interested in POST_CHANGE events
    if (pEvent.getType() != IResourceChangeEvent.POST_CHANGE)
    {
      return;
    }

    IResourceDelta rootDelta = pEvent.getDelta();
    if (rootDelta == null)
    {
      // might mean there are too many changes to report.
      fireLabelChanged();
    }
    try
    {
      rootDelta.accept(mVisitor);
    }
    catch (CoreException e)
    {
      ExoUIPlugin.logError(e);
    }
  }

  /**
   * @see org.eclipse.jface.viewers.ILightweightLabelDecorator#decorate(java.lang.Object,
   *      org.eclipse.jface.viewers.IDecoration)
   */
  public void decorate(Object pElement, IDecoration pDecoration)
  {
    if ((pElement instanceof IFolder) || (pElement instanceof IFile))
    {
      IResource element = (IResource) pElement;
      IPath relativePath = element.getProjectRelativePath();
      if (relativePath == null)
      {
        return;
      }

      IProject project = element.getProject();
      if (project != null)
      {
        try
        {
          IPortletProject pp = ExoCorePlugin.createPortletProject(project);
          if (pElement instanceof IFolder)
          {
            IPath webfolder = pp.getWebFolder();
            if ((webfolder != null) && (webfolder.equals(relativePath)))
            {
              pDecoration.addOverlay(ExoUIPluginImages.DESC_OVR_WEBFOLDER);
              IPath contextRoot = pp.getContextRoot();
              if ((contextRoot != null) && (!contextRoot.isEmpty()))
              {
                pDecoration.addSuffix(" (" + getContextRoot(contextRoot) + ")");
              }
            }
          }
          else if (pElement instanceof IFile)
          {
            IPath portletXML = pp.getPortletXML();
            if ((portletXML != null) && (portletXML.equals(relativePath)))
            {
              String version = pp.getPortletXMLVersion();
              if (version != null)
              {
                ImageDescriptor imageDesc = (ImageDescriptor) mImageMap.get(version);
                if (imageDesc != null)
                {
                  pDecoration.addOverlay(imageDesc);
                }
              }
            }
          }
        }
        catch (CoreException e)
        {
          // don't care about this
        }
      }
    }
  }

  /**
   * @see org.eclipse.jface.viewers.IBaseLabelProvider#dispose()
   */
  public void dispose()
  {
    super.dispose();
    ResourcesPlugin.getWorkspace().removeResourceChangeListener(this);
  }

  /**
   * @see org.eclipse.jface.viewers.IBaseLabelProvider#isLabelProperty(java.lang.Object,
   *      java.lang.String)
   */
  public boolean isLabelProperty(Object pElement, String pProperty)
  {
    return false;
  }

  private void fireLabelChanged()
  {
    final LabelProviderChangedEvent event = new LabelProviderChangedEvent(this, null);

    // Just to be safe, we are assuming we might not be running in the UI thread
    Display.getDefault().asyncExec(new Runnable()
    {
      public void run()
      {
        // Fire a LabelProviderChangedEvent to notify the decorater manager
        fireLabelProviderChanged(event);
      }
    });
  }

  private String getContextRoot(IPath pContextRoot)
  {
    String contextRoot = pContextRoot.removeTrailingSeparator().toString();
    if (contextRoot.startsWith("/") || contextRoot.startsWith("\\"))
    {
      contextRoot = contextRoot.substring(1);
    }
    if (contextRoot.startsWith("/") || contextRoot.startsWith("\\"))
    {
      contextRoot = contextRoot.substring(1);
    }

    return contextRoot;
  }

}