/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.internal.ui.preference;


import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.help.WorkbenchHelp;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.exoplatform.eclipse.core.IPreferencesConstant;
import org.exoplatform.eclipse.core.operation.RemovePortletValidationMarkersOperation;
import org.exoplatform.eclipse.core.operation.ValidatePortletDeploymentDescriptorOperation;
import org.exoplatform.eclipse.internal.ui.preference.helper.GroupBooleanFieldEditor;
import org.exoplatform.eclipse.internal.ui.preference.helper.ValidatorVersionTableFieldEditor;
import org.exoplatform.eclipse.ui.ExoUIPlugin;
import org.exoplatform.eclipse.ui.IExoHelpContextIds;
import org.exoplatform.eclipse.ui.action.ExoRunnableAdapter;
import org.exoplatform.eclipse.ui.common.PreferencesWrapper;
import org.exoplatform.eclipse.ui.dialog.ProblemDialog;

import java.lang.reflect.InvocationTargetException;

/**
 * The preference page for the portlet validation
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class ExoPluginPreferencePage extends FieldEditorPreferencePage
  implements IWorkbenchPreferencePage
{
  public static final String CLASS_VERSION = "$Id: ExoPluginPreferencePage.java,v 1.1 2004/04/19 03:37:23 hatimk Exp $"; //$NON-NLS-1$
  private static final String ENABLE_VALIDATION_GROUP_TITLE =
    "Portlet validation settings";
  private static final String ENABLE_VALIDATION_GROUP_DESCRIPTION =
    "If you uncheck the box, then the portlet validator will not run when projects "
    + "are built.  However, if you disable portlet "
    + "validation, you still can manually validate portlet.xml file by right clicking "
    + "on it and choosing 'validate portlet' option from the popup context menu.";
  private static final String REMOVAL_VALIDATION_TITLE =
    "Regarding portlet validation errors";
  private static final String REMOVAL_VALIDATION_MESSAGE =
    "Because you chose to disable portlet validation, "
    + "would you like to clear all existing portlet "
    + "validation error messages?";
  private static final String VALIDATE_ALL_TITLE =
    "Regarding portlet validation";
  private static final String VALIDATE_ALL_MESSAGE =
    "Because you chose to enable portlet validation after it was disabled, "
    + "would you like to validate all portlet.xml across all projects, right now?";
  private static final String PROBLEM_TITLE =
    "Portlet Validation Error Removal Problem";
  private static final String VALIDATION_PROBLEM_TITLE =
    "Portlet Validation Error Removal Problem";
  private IWorkbench mWorkbench;
  private boolean mStartStateOfValidation = true;
  private boolean mDoneRemoval = false;
  private boolean mDoneValidation = false;

  /**
   * The constructor method.
   */
  public ExoPluginPreferencePage()
  {
    super(FieldEditorPreferencePage.GRID);

    // Set the preference store for the preference page.
    IPreferenceStore store = new PreferencesWrapper(ExoCorePlugin.getDefault());
    setPreferenceStore(store);
    mStartStateOfValidation =
      store.getBoolean(IPreferencesConstant.PORTLET_VALIDATION_ENABLE);
  }

  /**   
   * @see org.eclipse.jface.preference.FieldEditorPreferencePage#createFieldEditors()
   */
  protected void createFieldEditors()
  {
    GroupBooleanFieldEditor validationSettings =
      new GroupBooleanFieldEditor(IPreferencesConstant.PORTLET_VALIDATION_ENABLE,
        "&Perform portlet validation automatically",
        ENABLE_VALIDATION_GROUP_TITLE, ENABLE_VALIDATION_GROUP_DESCRIPTION,
        getFieldEditorParent());
    ValidatorVersionTableFieldEditor versionTable =
      new ValidatorVersionTableFieldEditor(getFieldEditorParent());
    addField(validationSettings);
    addField(versionTable);
    WorkbenchHelp.setHelp(getControl(),
      IExoHelpContextIds.VALIDATION_PREFERENCE_PAGE);
  }

  /**   
   * @see org.eclipse.jface.preference.IPreferencePage#performOk()
   */
  public boolean performOk()
  {
    // give the user a chance to clear any existing portlet validation error messages
    // if they chose to diable validation
    // However, if the enable the validation after it was disable, they get a chance
    // to validate all portlets
    boolean result = super.performOk();
    boolean currentState =
      getPreferenceStore().getBoolean(IPreferencesConstant.PORTLET_VALIDATION_ENABLE);
    if (currentState != mStartStateOfValidation)
    {
      if ((currentState == false) && (!mDoneRemoval))
      {
        if (MessageDialog.openQuestion(getShell(), REMOVAL_VALIDATION_TITLE,
              REMOVAL_VALIDATION_MESSAGE))
        {
          handleValidationErrorRemoval();
        }
        mDoneRemoval = true;
        mDoneValidation = false;
        mStartStateOfValidation = currentState;
      }
      else if ((currentState == true) && (!mDoneValidation))
      {
        if (MessageDialog.openQuestion(getShell(), VALIDATE_ALL_TITLE,
              VALIDATE_ALL_MESSAGE))
        {
          handleValidationForAllProjects();
        }
        mDoneValidation = true;
        mDoneRemoval = false;
        mStartStateOfValidation = currentState;
      }
    }

    return result;
  }

  private void handleValidationErrorRemoval()
  {
    Shell shell = getShell();
    RemovePortletValidationMarkersOperation op = null;
    try
    {
      op = new RemovePortletValidationMarkersOperation();
      new ProgressMonitorDialog(shell).run(true, true,
        new ExoRunnableAdapter(op));

      IStatus status = op.getStatus();
      if (!status.isOK())
      {
        ProblemDialog.open(shell, PROBLEM_TITLE, null, status);
      }
    }
    catch (InvocationTargetException target)
    {
      Throwable cause = target.getTargetException();
      if (cause instanceof CoreException)
      {
        IStatus status = ((CoreException) cause).getStatus();
        ProblemDialog.open(shell, PROBLEM_TITLE, null, status);
        ExoUIPlugin.log(status);
      }
      else
      {
        String message =
          (cause == null) ? target.getLocalizedMessage() : cause.toString();
        MessageDialog.openError(shell, PROBLEM_TITLE,
          "Encountered a problem while trying to remove validaton errors\n"
          + message);
        ExoUIPlugin.logError(cause);
      }
    }
    catch (InterruptedException e)
    {
      // no action is required if user requests cancelation
    }
  }

  private void handleValidationForAllProjects()
  {
    Shell shell = getShell();
    ValidatePortletDeploymentDescriptorOperation op = null;
    try
    {
      op = new ValidatePortletDeploymentDescriptorOperation();
      new ProgressMonitorDialog(shell).run(true, true,
        new ExoRunnableAdapter(op));

      IStatus status = op.getStatus();
      if (!status.isOK())
      {
        ProblemDialog.open(shell, VALIDATION_PROBLEM_TITLE, null, status);
      }
    }
    catch (InvocationTargetException target)
    {
      Throwable cause = target.getTargetException();
      if (cause instanceof CoreException)
      {
        IStatus status = ((CoreException) cause).getStatus();
        ProblemDialog.open(shell, VALIDATION_PROBLEM_TITLE, null, status);
        ExoUIPlugin.log(status);
      }
      else
      {
        String message =
          (cause == null) ? target.getLocalizedMessage() : cause.toString();
        MessageDialog.openError(shell, VALIDATION_PROBLEM_TITLE,
          "Encountered a problem while trying to validate all portlets\n"
          + message);
        ExoUIPlugin.logError(cause);
      }
    }
    catch (InterruptedException e)
    {
      // no action is required if user requests cancelation
    }
  }

  /**   
   * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
   */
  public void init(IWorkbench pWorkbench)
  {
    mWorkbench = pWorkbench;
  }

  /**   
   * Returns the shell associated with this window
   *
   * @return returns the shell associated with this window
   */
  public Shell getShell()
  {
    if (mWorkbench == null)
    {
      return null;
    }
    if (mWorkbench.getActiveWorkbenchWindow() == null)
    {
      return null;
    }

    return mWorkbench.getActiveWorkbenchWindow().getShell();
  }
}
