/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.internal.ui.sample;


import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.exoplatform.eclipse.core.extension.IExtensionProcessor;
import org.exoplatform.eclipse.core.extension.ProcessExtension;
import org.exoplatform.eclipse.core.filesprovider.WebappProjectFilesProvider;
import org.exoplatform.eclipse.core.filesprovider.ZipFileWithRenameStructureProvider;
import org.exoplatform.eclipse.core.operation.helper.JavaProjectConfiguration;
import org.exoplatform.eclipse.core.operation.helper.WebappProjectSettings;
import org.exoplatform.eclipse.ui.ExoUIPlugin;
import org.exoplatform.eclipse.ui.ExoUIPluginImages;
import org.osgi.framework.Bundle;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.zip.ZipFile;

/**
 * This class encapsulates the logic to parse the extensions that provide the
 * sample webapp projects. The class exposes convenient methods to return
 * SampleWebappProjects.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class SampleWebappInitializer implements IExtensionProcessor
{
  public static final String CLASS_VERSION =
    "$Id: SampleWebappInitializer.java,v 1.1 2004/04/19 03:37:23 hatimk Exp $"; //$NON-NLS-1$
  public static final String EXTENSION_POINT_ID =
    ExoUIPlugin.PLUGIN_ID + ".exoSampleWebappProviders"; //$NON-NLS-1$
  public static final String EXTENSION_ELEMENT_WEBAPPSAMPLE = "webapp-sample"; //$NON-NLS-1$              
  public static final String EXTENSION_ATTRIBUTE_WEBAPPSAMPLE_KIND = "kind"; //$NON-NLS-1$
  public static final String EXTENSION_ATTRIBUTE_WEBAPPSAMPLE_TITLE = "title"; //$NON-NLS-1$
  public static final String EXTENSION_ATTRIBUTE_WEBAPPSAMPLE_ICON = "icon"; //$NON-NLS-1$
  public static final String EXTENSION_ELEMENT_DESCRIPTION = "description"; //$NON-NLS-1$                    
  public static final String EXTENSION_ELEMENT_GROUP = "group"; //$NON-NLS-1$          
  public static final String EXTENSION_ATTRIBUTE_GROUP_MOUNTPOINT =
    "mount-point"; //$NON-NLS-1$    
  public static final String EXTENSION_ELEMENT_ZIPENTRY = "zip-entry"; //$NON-NLS-1$      
  public static final String EXTENSION_ATTRIBUTE_ZIP_LOCATION = "location"; //$NON-NLS-1$  
  public static final String EXTENSION_ATTRIBUTE_ZIP_SRCFOLDER = "src-folder"; //$NON-NLS-1$  
  public static final String EXTENSION_ATTRIBUTE_ZIP_WEBFOLDER = "web-folder"; //$NON-NLS-1$  

  /**   
   * @see org.exoplatform.eclipse.core.extension.IExtensionProcessor#process(org.eclipse.core.runtime.IExtension,
   *      org.eclipse.core.runtime.IConfigurationElement, java.lang.Object)
   */
  public void process(IExtension pExtension, IConfigurationElement pMember,
    Object pResultCollector)
  {
    List resultCollector = (List) pResultCollector;
    WebappSampleTag webapptag;
    try
    {
      webapptag = new WebappSampleTag(pExtension, pMember);
      resultCollector.add(webapptag);
    }
    catch (IOException e)
    {
      ExoUIPlugin.logError(
        "Couldn't process one of the web application sample provider .. skiping this one .. :"
        + e.getMessage(), e);
    }
  }

  /**   
   * Returns all available sample webapp projects that match the given kind.
   *
   * @param pKind to match with the webapp projects
   *
   * @return webapp projects that match the given kind
   */
  public static SampleWebappProject[] getSampleWebappProjects(String pKind)
  {
    SampleWebappInitializer sampleInit = new SampleWebappInitializer();
    List resultCollector = new ArrayList();
    ProcessExtension.process(EXTENSION_POINT_ID, sampleInit, resultCollector);

    List sampleResult = new ArrayList();

    // now the resultCollector contains a list of WebappSampleTag
    Iterator isample = resultCollector.iterator();
    while (isample.hasNext())
    {
      WebappSampleTag sampletag = (WebappSampleTag) isample.next();
      if (!sampletag.getKind().equals(pKind))
      {
        continue;
      }

      SampleWebappProject webProject =
        new SampleWebappProject(sampletag, sampletag.getKind(),
          sampletag.getTitle(), sampletag.getDescription(), sampletag.getIcon());
      sampleResult.add(webProject);
    }

    SampleWebappProject[] result = new SampleWebappProject[sampleResult.size()];
    result = (SampleWebappProject[]) sampleResult.toArray(result);

    return result;
  }

  /**   
   * Returns a java project configuration object that can be used to create the
   * corresponding given sample webapp project.
   *
   * @param pSample the sample webapp project that we need its corresponding
   *        java configuration object
   * @param pParentFolder the folder that this webapp should be installed
   *        under. The path should be relative to the project. A null value
   *        means install on the project directory.
   * @param pSourceFolder the source folder relative to the parent folder
   * @param pWebFolder the web folder relative to the parent folder
   *
   * @return a fully populated java project configuration object that can be
   *         used to create the corresponding given sample webapp project.
   */
  public static JavaProjectConfiguration getJavaProjectConfiguration(
    SampleWebappProject pSample, IPath pParentFolder, IPath pSourceFolder,
    IPath pWebFolder)
  {
    JavaProjectConfiguration javaConfig = new JavaProjectConfiguration();
    IPath defaultBinaryFolder = null;
    Iterator igroup =
      ((WebappSampleTag) pSample.getCachedToken()).getGroupTags().iterator();
    while (igroup.hasNext())
    {
      GroupTag grouptag = (GroupTag) igroup.next();
      IPath parentFolder = grouptag.getMountPoint();
      if (parentFolder.equals(new Path(IExoSampleConstant.PROJECT_MOUNTPOINT)))
      {
        parentFolder = pParentFolder;
      }
      if (parentFolder == null)
      {
        defaultBinaryFolder = pWebFolder.append("WEB-INF/classes");
      }
      else
      {
        defaultBinaryFolder =
          parentFolder.append(pWebFolder).append("WEB-INF/classes");
      }

      WebappProjectSettings webappSettings =
        new WebappProjectSettings(parentFolder, pSourceFolder, pWebFolder);

      Iterator izip = grouptag.getZipEntryTags().iterator();
      while (izip.hasNext())
      {
        ZipEntryTag ziptag = (ZipEntryTag) izip.next();
        ZipFileWithRenameStructureProvider zipProvider =
          new ZipFileWithRenameStructureProvider(ziptag.getZipFile());
        WebappProjectFilesProvider webProvider =
          new WebappProjectFilesProvider(zipProvider, zipProvider.getRoot(),
            ziptag.getSourceFolder(), ziptag.getWebFolder());
        webappSettings.addJavaProjectFilesProvider(webProvider);
      }
      javaConfig.addJavaProjectSettings(webappSettings);
    }

    // set up the default binary folder and the libraries
    javaConfig.setDefaultBinaryFolder(defaultBinaryFolder);

    IClasspathEntry cpe1 =
      JavaCore.newContainerEntry(new Path(IExoSampleConstant.JRE_LIBRARY_PATH));
    IClasspathEntry cpe2 =
      JavaCore.newContainerEntry(new Path(
          IExoSampleConstant.EXO_PORTLET_LIBRARY_PATH));
    javaConfig.setProjectLibraries(new IClasspathEntry[] { cpe1, cpe2 });

    return javaConfig;
  }

  /**   
   * Returns a web project setttings object that can be used while upgrading to
   * a portlet project or while mounting the web content to a different folder
   *
   * @param pSample the sample webapp project that we need its corresponding
   *        web project settings object, the sample webapp should be of kind
   *        portlet-upgrade.
   * @param pParentFolder the folder that this webapp should be installed
   *        under. The path should be relative to the project. A null value
   *        means install on the project directory.
   * @param pSourceFolder the source folder relative to the parent folder, can
   *        be null
   * @param pWebFolder the web folder relative to the parent folder
   *
   * @return a populated  web project setttings object that can be used while
   *         upgrading to a portlet project or while mounting the web content
   *         to a different folder
   */
  public static WebappProjectSettings getUpgradeWebappSettings(
    SampleWebappProject pSample, IPath pParentFolder, IPath pSourceFolder,
    IPath pWebFolder)
  {
    List lgroup = ((WebappSampleTag) pSample.getCachedToken()).getGroupTags();
    if ((lgroup == null) || (lgroup.size() < 1))
    {
      return null;
    }

    GroupTag grouptag = (GroupTag) lgroup.get(0);
    IPath parentFolder = grouptag.getMountPoint();
    if (parentFolder.equals(new Path(IExoSampleConstant.PROJECT_MOUNTPOINT)))
    {
      parentFolder = pParentFolder;
    }

    WebappProjectSettings webappSettings =
      new WebappProjectSettings(parentFolder, pSourceFolder, pWebFolder);

    Iterator izip = grouptag.getZipEntryTags().iterator();
    while (izip.hasNext())
    {
      ZipEntryTag ziptag = (ZipEntryTag) izip.next();
      ZipFileWithRenameStructureProvider zipProvider =
        new ZipFileWithRenameStructureProvider(ziptag.getZipFile());
      WebappProjectFilesProvider webProvider =
        new WebappProjectFilesProvider(zipProvider, zipProvider.getRoot(),
          ziptag.getSourceFolder(), ziptag.getWebFolder());
      webappSettings.addJavaProjectFilesProvider(webProvider);
    }

    return webappSettings;
  }

  private class WebappSampleTag
  {
    private String mKind;
    private String mTitle;
    private String mDescription;
    private ImageDescriptor mIcon = ExoUIPluginImages.DESC_OBJ_SAMPLE_WEB_APP;
    private List mGroupTags = new ArrayList();

    WebappSampleTag(IExtension pExtension, IConfigurationElement pMember)
      throws IOException
    {
      mKind = pMember.getAttribute(EXTENSION_ATTRIBUTE_WEBAPPSAMPLE_KIND);
      mTitle = pMember.getAttribute(EXTENSION_ATTRIBUTE_WEBAPPSAMPLE_TITLE);

      String icon = pMember.getAttribute(EXTENSION_ATTRIBUTE_WEBAPPSAMPLE_ICON);
      if (icon != null)
      {
        String bundleName = pExtension.getNamespace();
        Bundle bundle = Platform.getBundle(bundleName);
        URL iconURL = bundle.getEntry(icon);        
        mIcon = ImageDescriptor.createFromURL(iconURL);
      }

      IConfigurationElement[] elements =
        pMember.getChildren(EXTENSION_ELEMENT_DESCRIPTION);
      if (elements.length > 0)
      {
        mDescription = elements[0].getValueAsIs();
      }

      elements = pMember.getChildren(EXTENSION_ELEMENT_GROUP);
      for (int i = 0, sizei = elements.length; i < sizei; ++i)
      {
        GroupTag grouptag = new GroupTag(pExtension, elements[i]);
        mGroupTags.add(grouptag);
      }
    }

    public String getDescription()
    {
      return mDescription;
    }

    public List getGroupTags()
    {
      return mGroupTags;
    }

    public ImageDescriptor getIcon()
    {
      return mIcon;
    }

    public String getKind()
    {
      return mKind;
    }

    public String getTitle()
    {
      return mTitle;
    }
  }

  private class GroupTag
  {
    private IPath mMountPoint;
    private List mZipEntryTags = new ArrayList();

    GroupTag(IExtension pExtension, IConfigurationElement pMember)
      throws IOException
    {
      mMountPoint =
        new Path(pMember.getAttribute(EXTENSION_ATTRIBUTE_GROUP_MOUNTPOINT));

      IConfigurationElement[] elements =
        pMember.getChildren(EXTENSION_ELEMENT_ZIPENTRY);
      for (int i = 0, sizei = elements.length; i < sizei; ++i)
      {
        ZipEntryTag ziptag = new ZipEntryTag(pExtension, elements[i]);
        mZipEntryTags.add(ziptag);
      }
    }

    public IPath getMountPoint()
    {
      return mMountPoint;
    }

    public List getZipEntryTags()
    {
      return mZipEntryTags;
    }
  }

  private class ZipEntryTag
  {
    private IPath mSourceFolder;
    private IPath mWebFolder;
    private ZipFile mZipFile;

    ZipEntryTag(IExtension pExtension, IConfigurationElement pMember)
      throws IOException
    {      
      String ziploc = pMember.getAttribute(EXTENSION_ATTRIBUTE_ZIP_LOCATION);
      String bundleName = pExtension.getNamespace();
      Bundle bundle = Platform.getBundle(bundleName);
      URL fileURL = bundle.getEntry(new Path(ziploc).toString());
      String zipLocation = Platform.asLocalURL(fileURL).getFile();
      mZipFile = new ZipFile(zipLocation);

      mSourceFolder =
        new Path(pMember.getAttribute(EXTENSION_ATTRIBUTE_ZIP_SRCFOLDER));
      mWebFolder =
        new Path(pMember.getAttribute(EXTENSION_ATTRIBUTE_ZIP_WEBFOLDER));
    }

    public IPath getSourceFolder()
    {
      return mSourceFolder;
    }

    public IPath getWebFolder()
    {
      return mWebFolder;
    }

    public ZipFile getZipFile()
    {
      return mZipFile;
    }
  }
}
