/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.internal.ui.wizard;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.exoplatform.eclipse.core.exoservices.ObjectParam;
import org.exoplatform.eclipse.core.exoservices.PropertiesParam;
import org.exoplatform.eclipse.core.exoservices.Property;
import org.exoplatform.eclipse.core.portal.model.INode;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;

/**
 * @author Hatim Khan
 */
public class EditParameterPropertyValuePage extends DidYouKnowPage
{

  public static final int ADD_MODE = 0;
  public static final int EDIT_MODE = 1;
  public static final String TITLE_EDIT = "Edit an eXo Platform service configuration parameter value";
  public static final String TITLE_ADD = "Add an eXo Platform service configuration parameter value";
  public static final String MESSAGE_EDIT = "You are attempting to edit a parameter value of type 'Property'";
  public static final String MESSAGE_ADD = "You are attempting to create a new parameter value of type 'Property'";
  public static final String PAGE_NAME = "EditParameterPropertyValuePage";

  protected int mMode = EDIT_MODE;
  protected boolean mDirty;

  private Object mPropertyParent;
  private List mPropertiesKeys;
  private Property mProperty;
  private String mOriginalName;
  private String mOriginalValue;
  
  private Text mNameText;
  private Text mValueText;
  
  public EditParameterPropertyValuePage(int pMode, String pDidYouKnow, Object pPropertyParent, Property pProperty)
  {
    super(PAGE_NAME, pDidYouKnow);
    setTitle((pMode == ADD_MODE) ? TITLE_ADD : TITLE_EDIT);
    setMessage((pMode == ADD_MODE) ? MESSAGE_ADD : MESSAGE_EDIT);    
    mDirty = false;
    mMode = pMode;    
    mProperty = (pProperty == null) ? new Property() : pProperty;
    mPropertyParent = pPropertyParent;
  }

  /**
   * @see org.exoplatform.eclipse.internal.ui.wizard.DidYouKnowPage#createActualContent(org.eclipse.swt.widgets.Composite)
   */
  public void createActualContent(Composite pParent)
  {
    Composite container = UICompositeUtil.createFillBothComposite(pParent, 2, 1);
    GridLayout gl = (GridLayout) container.getLayout();
    gl.makeColumnsEqualWidth = false;
    Dialog.applyDialogFont(container);

    createNameInput(container);
    createValueInput(container);

    updateFromModel();
    attachListeners();

    setPageComplete((mMode == EDIT_MODE));    
  }
  
  protected void createNameInput(Composite pParent)
  {
    UICompositeUtil.addLabel(pParent, "Name : ", 1);

    mNameText = UICompositeUtil.addText(pParent, "", 1);
    GridData gd = new GridData(GridData.FILL_HORIZONTAL);
    mNameText.setLayoutData(gd);
  }
  
  protected void createValueInput(Composite pParent)
  {
    UICompositeUtil.addLabel(pParent, "Value : ", 1);

    mValueText = UICompositeUtil.addText(pParent, "", 1);
    GridData gd = new GridData(GridData.FILL_HORIZONTAL);
    mValueText.setLayoutData(gd);
  }
  
  protected void updateFromModel()
  {
    String name = "";
    String value = "";
    if (mProperty != null)
    {
      name = UICompositeUtil.emptyIfNull(mProperty.getName()).trim();
      value = UICompositeUtil.emptyIfNull(mProperty.getValue()).trim();
    }

    mOriginalName = name;
    mOriginalValue = value;

    mPropertiesKeys = new ArrayList();

    if (mPropertyParent instanceof PropertiesParam)
    {
      PropertiesParam param = (PropertiesParam) mPropertyParent;
      mPropertiesKeys.addAll(param.getPropertiesKeys());
    }
    else if (mPropertyParent instanceof ObjectParam)
    {
      ObjectParam param = (ObjectParam) mPropertyParent;
      mPropertiesKeys.addAll(param.getPropertiesKeys());
    }

    // remove the original key from the list so we don't confuse it with the other keys
    if (name.length() > 0) mPropertiesKeys.remove(name);

    mNameText.setText(name);
    mValueText.setText(value);
    mDirty = false;
  }

  protected boolean validateInput()
  {
    String name = UICompositeUtil.emptyIfNull(mNameText.getText()).trim();

    if ((name.length() < 1))
    {
      setErrorMessage("The name can NOT be empty.");
      return false;
    }

    boolean existingName = mPropertiesKeys.contains(name);
    // check if the proposed name matches an existing name (except the original parameter)
    if (existingName)
    {
      setErrorMessage("There is an existing Property with the same name.");
      return false;
    }

    // if we get to this point then validation is complete and we don't have any errors
    // so update the error message
    setErrorMessage(null);
    return true;
  }
 
  protected void attachListeners()
  {
    mNameText.addListener(SWT.Modify, new Listener()
    {
      public void handleEvent(Event pEvent)
      {
        setPageComplete(validateInput());
      }
    });
    mValueText.addListener(SWT.Modify, new Listener()
    {
      public void handleEvent(Event pEvent)
      {
        setPageComplete(validateInput());
      }
    });
  }

  public void finish()
  {
    // first lets detect if the user changed either the name and/or the value, if
    // so, then we need to make the model dirty and fire an event
    String name = UICompositeUtil.emptyIfNull(mNameText.getText()).trim();
    String value = UICompositeUtil.emptyIfNull(mValueText.getText()).trim();
    if ((!name.equals(mOriginalName)) || (!value.equals(mOriginalValue)))
    {
      if (mProperty != null)
      {
        mProperty.setName(name);
        mProperty.setValue(value);
        mDirty = true;
      }
    }

    if (mDirty)
    {
      if (mMode == EDIT_MODE)
      {
        // we dont need to do anything here since we already updated the name and value of the property
      }
      else if (mMode == ADD_MODE)
      {
        if (mPropertyParent instanceof INode)
        {
          INode parentNode = (INode) mPropertyParent;
          parentNode.addChild(mProperty);
        }
      }
    }
  }
  
  public boolean isDirty()
  {
    return mDirty;
  }
}