/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.internal.ui.wizard;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.exoplatform.eclipse.core.exoservices.Configuration;
import org.exoplatform.eclipse.core.exoservices.ObjectParam;
import org.exoplatform.eclipse.core.exoservices.Parameter;
import org.exoplatform.eclipse.core.exoservices.PropertiesParam;
import org.exoplatform.eclipse.core.exoservices.Property;
import org.exoplatform.eclipse.core.exoservices.ServiceConfiguration;
import org.exoplatform.eclipse.core.exoservices.ValueParam;
import org.exoplatform.eclipse.core.exoservices.ValuesParam;
import org.exoplatform.eclipse.core.portal.PortalSettings;
import org.exoplatform.eclipse.core.portal.model.INode;
import org.exoplatform.eclipse.core.portal.model.ValueHolder;
import org.exoplatform.eclipse.ui.common.DidYouKnowMessages;
import org.exoplatform.eclipse.ui.common.PortalConfigurationContentProvider;
import org.exoplatform.eclipse.ui.common.PortalConfigurationLabelProvider;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;

/**
 * A wizard page that allows the user to either edit or add a service configuration
 * key/type. This page will update the model and fire the appropriate model change event.
 * If this page changes the model, it will tell the outside world about it by setting the
 * isDirty() to true.
 * 
 * @author Hatim Khan
 */
public class EditServicePage extends DidYouKnowPage
{
  public static final int ADD_MODE = 0;
  public static final int EDIT_MODE = 1;
  public static final String PAGE_NAME = "EditServicePage";
  public static final String TITLE_EDIT = "Edit an eXo Platform service configuration";
  public static final String TITLE_ADD = "Add an eXo Platform service configuration";
  public static final String DID_YOU_KNOW = DidYouKnowMessages.getString("EditServicePage");//$NON-NLS-1$
  public static final Class[] EDIT_ACCEPT_CLASSES = new Class[] { ValueHolder.class, Property.class,
      ValueParam.class, ValuesParam.class, PropertiesParam.class, ObjectParam.class };

  private int mMode = EDIT_MODE;
  private PortalSettings mPortalSettings;
  private Configuration mConfiguration;
  private ServiceConfiguration mOriginalServiceConfiguration;
  private ServiceConfiguration mCopyServiceConfiguration;
  private List mServicesKeys;
  private String mOriginalKey;
  private String mOriginalType;
  private boolean mDirty;

  private Text mKeyText;
  private Text mTypeText;
  private StructuredViewer mViewer;
  private Button mAddValueParamButton;
  private Button mAddValuesParamButton;
  private Button mAddPropertiesParamButton;
  private Button mAddObjectParamButton;
  private Button mEditButton;
  private Button mRemoveButton;
  private Button mExpandAllButton;
  private Button mCollapseAllButton;

  public EditServicePage(PortalSettings pPortalSettings, Configuration pConfiguration,
      ServiceConfiguration pServiceConfiguration)
  {
    this(EDIT_MODE, pPortalSettings, pConfiguration, pServiceConfiguration);
  }

  public EditServicePage(PortalSettings pPortalSettings, Configuration pConfiguration)
  {
    this(ADD_MODE, pPortalSettings, pConfiguration, null);
  }

  private EditServicePage(int pMode, PortalSettings pPortalSettings, Configuration pConfiguration,
      ServiceConfiguration pServiceConfiguration)
  {
    super(PAGE_NAME, DID_YOU_KNOW);
    mMode = pMode;
    mPortalSettings = pPortalSettings;
    mConfiguration = pConfiguration;
    mDirty = false;
    setTitle((pMode == ADD_MODE) ? TITLE_ADD : TITLE_EDIT);
    if ((pServiceConfiguration == null) || (pMode == ADD_MODE))
    {
      mOriginalServiceConfiguration = new ServiceConfiguration();
    }
    else
    {
      mOriginalServiceConfiguration = pServiceConfiguration;
    }
  }

  public void createActualContent(Composite pParent)
  {
    createKeyTypeContent(pParent);
    updateFromModel();
    attachListeners();

    setMessage("You have the option of changing the key, the type or both.");
    setPageComplete((mMode == EDIT_MODE));
  }

  private void updateFromModel()
  {
    String key = UICompositeUtil.emptyIfNull(mOriginalServiceConfiguration.getKey()).trim();
    String type = UICompositeUtil.emptyIfNull(mOriginalServiceConfiguration.getType()).trim();

    mOriginalKey = key;
    mOriginalType = type;
    mCopyServiceConfiguration = (ServiceConfiguration) mOriginalServiceConfiguration.getCopy();

    Configuration config = PortalSettings.findConfiguration(mOriginalServiceConfiguration);
    if (config != null)
    {
      mServicesKeys = config.getServicesKeys();
      // remove the original key from the list so we don't confuse it with the other keys
      if (key.length() > 0) mServicesKeys.remove(key);
      else mServicesKeys.remove(type);
    }
    else
    {
      mServicesKeys = new ArrayList();
    }

    mKeyText.setText(key);
    mTypeText.setText(type);
    mDirty = false;

    populateViewer(mViewer, mCopyServiceConfiguration);
    updateButtonsState(null);
  }

  public void finish()
  {
    // first lets detect if the user changed either the key and/or the type, or
    // updated/added parameters, if so, then we need to make the model dirty and fire an
    // event
    if (updateToModel())
    {
      mDirty = true;

      if (mPortalSettings != null) mPortalSettings.markDirty();
      if (mMode == EDIT_MODE)
      {
        if (mPortalSettings != null)
            mPortalSettings.fireConfigurationElementUpdated(mOriginalServiceConfiguration);
      }
      else if (mMode == ADD_MODE)
      {
        if (mConfiguration != null)
        {
          mConfiguration.addChild(mOriginalServiceConfiguration);
          if (mPortalSettings != null)
              mPortalSettings.fireConfigurationElementAdded(mOriginalServiceConfiguration);
        }
      }
    }
  }

  private boolean updateToModel()
  {
    // will return true if we updated the mOriginalServiceConfiguration
    boolean updated = false;
    String key = UICompositeUtil.emptyIfNull(mKeyText.getText()).trim();
    String type = UICompositeUtil.emptyIfNull(mTypeText.getText()).trim();
    if ((!key.equals(mOriginalKey)) || (!type.equals(mOriginalType)))
    {
      mOriginalServiceConfiguration.setKey((key.length() > 0) ? key : null);
      mOriginalServiceConfiguration.setType((type.length() > 0) ? type : null);
      updated = true;
    }

    if (!Parameter.isEquivalentParameterList(mOriginalServiceConfiguration.getValueParams(), mCopyServiceConfiguration.getValueParams()))
    {
      updated = true;
      mOriginalServiceConfiguration.setValueParams(Parameter.getCopyParameterList(mCopyServiceConfiguration.getValueParams()));
    }

    if (!Parameter.isEquivalentParameterList(mOriginalServiceConfiguration.getValuesParams(), mCopyServiceConfiguration.getValuesParams()))
    {
      updated = true;
      mOriginalServiceConfiguration.setValuesParams(Parameter.getCopyParameterList(mCopyServiceConfiguration.getValuesParams()));
    }

    if (!Parameter.isEquivalentParameterList(mOriginalServiceConfiguration.getPropertiesParams(), mCopyServiceConfiguration.getPropertiesParams()))
    {
      updated = true;
      mOriginalServiceConfiguration.setPropertiesParams(Parameter.getCopyParameterList(mCopyServiceConfiguration.getPropertiesParams()));
    }

    if (!Parameter.isEquivalentParameterList(mOriginalServiceConfiguration.getObjectParams(), mCopyServiceConfiguration.getObjectParams()))
    {
      updated = true;
      mOriginalServiceConfiguration.setObjectParams(Parameter.getCopyParameterList(mCopyServiceConfiguration.getObjectParams()));
    }

    //    if (updated)
    //    {
    //      mOriginalServiceConfiguration.setAppearIn(null);
    //      mOriginalServiceConfiguration.getChildren(); // just to update the parent links
    //    }

    return updated;
  }

  private boolean validateInput()
  {
    String key = UICompositeUtil.emptyIfNull(mKeyText.getText()).trim();
    String type = UICompositeUtil.emptyIfNull(mTypeText.getText()).trim();

    // check if both are empty, if so then display an error
    if ((key.length() < 1) && (type.length() < 1))
    {
      setErrorMessage("The key and the value can not be empty at the same time.");
      return false;
    }

    String proposedKey = (key.length() > 0) ? key : type;
    boolean existingKey = mServicesKeys.contains(proposedKey);
    // check if the proposed key matches an existing key (except the original service)
    if (existingKey)
    {
      setErrorMessage("There is an existing service configuration that conflicts with this one.");
      return false;
    }

    // type can not be empty
    if (type.length() < 1)
    {
      setErrorMessage("The type can NOT be empty.");
      return false;
    }

    // if we get to this point then validation is complete and we don't have any errors
    // so update the error message
    setErrorMessage(null);
    return true;
  }

  private Composite createKeyTypeContent(Composite pParent)
  {
    Composite container = UICompositeUtil.createFillBothComposite(pParent, 2, 1);
    GridLayout gl = (GridLayout) container.getLayout();
    gl.makeColumnsEqualWidth = false;
    Dialog.applyDialogFont(container);

    UICompositeUtil.addLabel(container, "Key : ", 1);
    mKeyText = UICompositeUtil.addText(container, "", 1);

    UICompositeUtil.addLabel(container, "Type : ", 1);
    mTypeText = UICompositeUtil.addText(container, "", 1);
    UICompositeUtil.setWidthHint(mTypeText, IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);

    addParametersContent(container);

    return container;
  }

  private void attachListeners()
  {
    mTypeText.addListener(SWT.Modify, new Listener()
    {

      public void handleEvent(Event pEvent)
      {
        setPageComplete(validateInput());
      }
    });

    mKeyText.addListener(SWT.Modify, new Listener()
    {

      public void handleEvent(Event pEvent)
      {
        setPageComplete(validateInput());
      }
    });

    attachListenersToButtons();
  }

  private void attachListenersToButtons()
  {
    mAddValueParamButton.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent pEvent)
      {
        addButtonPressed(ValueParam.class);
      }
    });

    mAddValuesParamButton.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent pEvent)
      {
        addButtonPressed(ValuesParam.class);
      }
    });

    mAddPropertiesParamButton.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent pEvent)
      {
        addButtonPressed(PropertiesParam.class);
      }
    });

    mAddObjectParamButton.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent pEvent)
      {
        addButtonPressed(ObjectParam.class);
      }
    });

    mEditButton.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent pEvent)
      {
        Object selection = UICompositeUtil.getSelection(mViewer.getSelection(), EDIT_ACCEPT_CLASSES);
        if (selection == null)
        {
          MessageDialog.openError(mEditButton.getShell(), "Error while trying to edit the selection", "Either the selection is empty or you are trying to edit a selection that is not supported. Please make a different selection and try again.");
        }
        else
        {
          editButtonPressed(selection);
        }
      }
    });

    mRemoveButton.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent pEvent)
      {
        Object selection = UICompositeUtil.getSelection(mViewer.getSelection(), EDIT_ACCEPT_CLASSES);
        if (selection == null)
        {
          MessageDialog.openError(mRemoveButton.getShell(), "Error while trying to remove the selection", "Either the selection is empty or you are trying to remove a selection that is not supported. Please make a different selection and try again.");
        }
        else
        {
          removeButtonPressed(selection);
        }
      }
    });

    mExpandAllButton.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent pEvent)
      {
        ((TreeViewer) mViewer).expandAll();
      }
    });

    mCollapseAllButton.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent pEvent)
      {
        ((TreeViewer) mViewer).collapseAll();
      }
    });

  }

  /**
   * @return Returns the changeOccurred.
   */
  public boolean isDirty()
  {
    return mDirty;
  }

  protected void addParametersContent(Composite pClient)
  {
    Group group = UICompositeUtil.createGroup(pClient, "Parameters", 2, 2);
    mViewer = createTreeViewer(group);
    createActionButtons(group);
    createAddButtons(group);
  }

  private StructuredViewer createTreeViewer(Composite parent)
  {
    int style = SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER;
    TreeViewer treeViewer = new TreeViewer(parent, style);
    Control tree = treeViewer.getControl();
    GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
    gridData.horizontalSpan = 1;
    tree.setLayoutData(gridData);
    UICompositeUtil.setWidthHeight(tree, -1, 150);

    return treeViewer;
  }

  private void createActionButtons(Composite pParent)
  {
    Composite container = UICompositeUtil.createFillBothComposite(pParent, 1, 1);
    Dialog.applyDialogFont(container);
    GridLayout layout = (GridLayout) container.getLayout();
    layout.makeColumnsEqualWidth = false;
    layout.marginHeight = 0;
    layout.marginWidth = 5;

    container.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_BEGINNING));

    mEditButton = createButton("Edit...", container);
    mRemoveButton = createButton("Remove", container);
    mExpandAllButton = createButton("Expand All", container);
    mCollapseAllButton = createButton("Collapse All", container);
  }

  private void createAddButtons(Composite pParent)
  {
    Composite container = UICompositeUtil.createFillBothComposite(pParent, 4, 2);
    Dialog.applyDialogFont(container);
    GridLayout layout = (GridLayout) container.getLayout();
    layout.makeColumnsEqualWidth = false;
    layout.marginHeight = 0;
    layout.marginWidth = 0;

    container.setLayoutData(new GridData());

    mAddValueParamButton = createButton("Add ValueParam...", container);
    mAddValuesParamButton = createButton("Add ValuesParam...", container);
    mAddPropertiesParamButton = createButton("Add PropertiesParam...", container);
    mAddObjectParamButton = createButton("Add ObjectParam...", container);
  }

  private Button createButton(String pLabel, Composite pContainer)
  {
    Button button = new Button(pContainer, SWT.PUSH);
    button.setText(pLabel);
    button.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_BEGINNING | GridData.FILL_HORIZONTAL));

    return button;
  }

  private void populateViewer(StructuredViewer pViewer, Object pInput)
  {
    pViewer.setContentProvider(new PortalConfigurationContentProvider());
    pViewer.setLabelProvider(new PortalConfigurationLabelProvider());
    pViewer.setInput(pInput);
    pViewer.addSelectionChangedListener(new ISelectionChangedListener()
    {
      public void selectionChanged(SelectionChangedEvent pEvent)
      {
        updateButtonsState(pEvent.getSelection());
      }
    });
    pViewer.addDoubleClickListener(new IDoubleClickListener()
    {

      public void doubleClick(DoubleClickEvent pEvent)
      {
        updateButtonsState(pEvent.getSelection());
        editButtonPressed(UICompositeUtil.getSelection(pEvent.getSelection(), EDIT_ACCEPT_CLASSES));
      }
    });
  }

  private void updateButtonsState(ISelection pSelection)
  {
    Object selectedObject = UICompositeUtil.getSelection(pSelection, EDIT_ACCEPT_CLASSES);
    updateButtonsState(selectedObject);
  }

  private void updateButtonsState(Object pModelObject)
  {
    mAddValueParamButton.setEnabled(true);
    mAddValuesParamButton.setEnabled(true);
    mAddPropertiesParamButton.setEnabled(true);
    mAddObjectParamButton.setEnabled(true);

    mEditButton.setEnabled(false);
    mRemoveButton.setEnabled(false);
    mExpandAllButton.setEnabled(false);
    mCollapseAllButton.setEnabled(false);

    // lets first determine if the expand all or collapse all need to be enabled
    if (mOriginalServiceConfiguration != null)
    {
      Object[] values = mOriginalServiceConfiguration.getChildren();
      if ((values != null) && (values.length > 0))
      {
        mExpandAllButton.setEnabled(true);
        mCollapseAllButton.setEnabled(true);
      }
    }
    Object selection = UICompositeUtil.getSelection((mViewer != null) ? mViewer.getSelection() : null);
    if ((pModelObject == null) || (selection == null) || (pModelObject instanceof ServiceConfiguration))
    {
      mEditButton.setEnabled(false);
      mRemoveButton.setEnabled(false);
      return;
    }

    if (pModelObject instanceof Parameter)
    {
      mEditButton.setEnabled(true);
      mRemoveButton.setEnabled(true);
      return;
    }
  }

  private void editButtonPressed(Object pSelection)
  {
    if (pSelection instanceof ValueHolder)
    {
      pSelection = ((ValueHolder) pSelection).getParent();
    }

    if (pSelection instanceof Property)
    {
      pSelection = ((Property) pSelection).getParent();
    }

    if (pSelection instanceof Parameter)
    {
      Parameter parameter = (Parameter) pSelection;
      EditServiceParameterWizard wizard = new EditServiceParameterWizard(null, parameter);
      WizardDialog container = new WizardDialog(getShell(), wizard);
      container.open();
      if (wizard.isDirty())
      {
        mViewer.refresh(parameter);
        mViewer.getControl().setFocus();
      }
    }
  }

  private void addButtonPressed(Class pType)
  {
    EditServiceParameterWizard wizard = new EditServiceParameterWizard(pType, null, mCopyServiceConfiguration);
    WizardDialog container = new WizardDialog(getShell(), wizard);
    container.open();
    if (wizard.isDirty())
    {
      mViewer.refresh();
      mViewer.getControl().setFocus();
    }
  }

  private void removeButtonPressed(Object pSelection)
  {
    if (pSelection instanceof INode)
    {
      INode node = (INode) pSelection;
      Object object = node.getParent();
      if ((object != null) && (object instanceof INode))
      {
        INode parent = (INode) object;
        boolean removed = parent.removeChild(node);
        if (removed)
        {
          mViewer.refresh();
        }
      }
    }
  }

}