/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.internal.ui.wizard;


import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.dialogs.NewFolderDialog;
import org.eclipse.ui.help.WorkbenchHelp;
import org.eclipse.ui.model.WorkbenchContentProvider;
import org.eclipse.ui.model.WorkbenchLabelProvider;
import org.exoplatform.eclipse.ui.ExoUIPlugin;
import org.exoplatform.eclipse.ui.IExoHelpContextIds;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;

import java.util.ArrayList;
import java.util.List;

/**
 * A wizard page that presents the user with the option of selecting a folder,
 * along with two options, 1- make WEB-INF/classes the default binary folder,
 * 2- add all jars in WEB-INF/lib to the classpath.  To figure out what
 * options and folder the user selected. Use getResult(),
 * getMakeDefaultBinaryOption(), and getAddJarsToClasspathOption()
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class SelectWebFolderPage extends WizardPage
{
  public static final String CLASS_VERSION = "$Id: SelectWebFolderPage.java,v 1.1 2004/04/19 03:37:24 hatimk Exp $"; //$NON-NLS-1$
  private int mTreeWidth = 60;
  private int mTreeHeight = 18;
  private String mTreeTitle = "Select a folder";
  private String mGroupTitle = "";
  private List mFilters;
  private IResource mResult;
  private IProject mProject;
  private TreeViewer mTreeViewer;
  private Button mAddBinaryFolderOption;
  private Button mJarsToClasspathOption;
  private Composite mMiscOptionsGroup;

  /**
   * Creates a wizard page.
   *
   * @param pPageName the name of this page
   */
  public SelectWebFolderPage(String pPageName)
  {
    super(pPageName);
    setPageComplete(true);
  }

  /**   
   * The project to select a folder from
   *
   * @param pProject to select a folder from
   */
  public void init(IProject pProject)
  {
    mProject = pProject;
  }

  /**   
   * Get the selected resource. The resource could be the project itself. Use
   * getResult().getProjectRelativePath() to figure out the relative path to
   * the project.
   *
   * @return the selected resource (could be null)
   */
  public IResource getResult()
  {
    return mResult;
  }

  /**   
   * True if the user wants to make the WEB-INF/classes folder the default
   * binary folder, false otherwise.
   *
   * @return true if the user wants to make the WEB-INF/classes folder the
   *         default binary folder, false otherwise.
   */
  public boolean getMakeDefaultBinaryOption()
  {
    return (mAddBinaryFolderOption != null)
    ? mAddBinaryFolderOption.getSelection() : false;
  }

  /**   
   * True if the user wants to add all jars in WEB-INF/lib to the classpath,
   * false otherwise.
   *
   * @return true if the user wants to add all jars in WEB-INF/lib to the
   *         classpath, false otherwise.
   */
  public boolean getAddJarsToClasspathOption()
  {
    return (mJarsToClasspathOption != null)
    ? mJarsToClasspathOption.getSelection() : false;
  }

  /**   
   * The title to display on top of the resource tree
   *
   * @param pTitle the title to display on top of the resource tree
   */
  public void setSelectTitle(String pTitle)
  {
    mTreeTitle = pTitle;
  }

  /**   
   * The title to display on top of the resource tree group
   *
   * @param pTitle the title to display on top of the resource tree group
   */
  public void setGroupTitle(String pTitle)
  {
    mGroupTitle = pTitle;
  }

  /**   
   * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
   */
  public void createControl(Composite pParent)
  {
    Composite mainComposite =
      UICompositeUtil.createFillBothComposite(pParent, 1, 1);
    initializeDialogUnits(pParent);
    createNewFolderGroup(mainComposite);
    createMiscOptionGroup(mainComposite);

    setPageComplete(true);
    WorkbenchHelp.setHelp(mainComposite,
      IExoHelpContextIds.UPGRADE_PORTLET_WEBFOLDER_PAGE);
    setControl(mainComposite);
  }

  /**   
   * Adds a filter to the tree viewer.
   *
   * @param pFilter a filter.
   */
  public void addFilter(ViewerFilter pFilter)
  {
    if (mFilters == null)
    {
      mFilters = new ArrayList(4);
    }

    mFilters.add(pFilter);
  }

  private void createNewFolderGroup(Composite pParent)
  {
    Composite group =
      UICompositeUtil.createGroupWithNoGrapping(pParent, mGroupTitle, 1, 1);
    createResourceTree(group);
    createNewFolderButton(group);
  }

  private void createResourceTree(Composite pParent)
  {
    UICompositeUtil.addLabelWithWrapping(pParent, mTreeTitle, 1);
    mTreeViewer = createTreeViewer(pParent);

    GridData data = new GridData(GridData.FILL_BOTH);

    data.widthHint = convertWidthInCharsToPixels(mTreeWidth);
    data.heightHint = convertHeightInCharsToPixels(mTreeHeight);

    Tree treeWidget = mTreeViewer.getTree();
    treeWidget.setLayoutData(data);
    treeWidget.setFont(pParent.getFont());
  }

  private void createMiscOptionGroup(Composite pParent)
  {
    Composite group =
      UICompositeUtil.createGroupWithNoGrapping(pParent, "Misc. Options", 1, 1);
    mAddBinaryFolderOption =
      UICompositeUtil.createCheckboxButton(group,
        "&Make the WEB-INF\\classes folder under the selected web content the default binary folder.");
    mAddBinaryFolderOption.setSelection(false);
    mJarsToClasspathOption =
      UICompositeUtil.createCheckboxButton(group,
        "&Add the jars files in WEB-INF\\lib folder under the selected web content to the classpath.");
    mJarsToClasspathOption.setSelection(false);
    mMiscOptionsGroup = group;
  }

  /**   
   * Creates the tree viewer.
   *
   * @param pParent the parent composite
   *
   * @return the tree viewer
   */
  private TreeViewer createTreeViewer(Composite pParent)
  {
    int style = SWT.BORDER | SWT.SINGLE;

    TreeViewer viewer = new TreeViewer(new Tree(pParent, style));
    viewer.setContentProvider(new WorkbenchContentProvider());
    viewer.setLabelProvider(new WorkbenchLabelProvider());
    viewer.addSelectionChangedListener(new ISelectionChangedListener()
      {
        public void selectionChanged(SelectionChangedEvent pEvent)
        {
          Object selection =
            ((IStructuredSelection) pEvent.getSelection()).getFirstElement();
          if ((selection != null) && (selection instanceof IResource))
          {
            mResult = (IResource) selection;
          }
          updateMiscOptionsGroup();
        }
      });

    addFilter(new FileFilter());
    if (mFilters != null)
    {
      for (int i = 0; i != mFilters.size(); i++)
      {
        viewer.addFilter((ViewerFilter) mFilters.get(i));
      }
    }

    viewer.setInput(mProject.getParent());

    return viewer;
  }

  private void createNewFolderButton(Composite pParent)
  {
    Button button = new Button(pParent, SWT.PUSH);
    button.setText("Create New Folder...");
    button.addSelectionListener(new SelectionAdapter()
      {
        public void widgetSelected(SelectionEvent event)
        {
          newFolderButtonPressed();
        }
      });
    button.setFont(pParent.getFont());

    GridData data = new GridData();
    data.heightHint =
      convertVerticalDLUsToPixels(IDialogConstants.BUTTON_HEIGHT);
    button.setLayoutData(data);
  }

  protected void newFolderButtonPressed()
  {
    NewFolderDialog dialog =
      new NewFolderDialog(getShell(), (IContainer) mResult);
    if (dialog.open() == NewFolderDialog.OK)
    {
      TreeViewer treeViewer = mTreeViewer;
      treeViewer.refresh(mResult);

      Object createdFolder = dialog.getResult()[0];
      treeViewer.reveal(createdFolder);
      treeViewer.setSelection(new StructuredSelection(createdFolder));
    }
  }

  protected void updateMiscOptionsGroup()
  {
    if ((mResult == null) || (mResult instanceof IProject)
        || !(mResult instanceof IFolder))
    {
      mAddBinaryFolderOption.setSelection(false);
      mJarsToClasspathOption.setSelection(false);
      UICompositeUtil.disableAllWidgets(mMiscOptionsGroup);
    }
    else
    {
      UICompositeUtil.enableAllWidgets(mMiscOptionsGroup);
      mAddBinaryFolderOption.setSelection(true);
      mJarsToClasspathOption.setSelection(true);

      // check if we have any jar files in the lib directory
      if (!containsJars(mResult.getProjectRelativePath().append("WEB-INF/lib")))
      {
        mJarsToClasspathOption.setSelection(false);
        mJarsToClasspathOption.setEnabled(false);
      }
    }
  }

  private boolean containsJars(IPath pFolder)
  {
    if (!mProject.exists(pFolder))
    {
      return false;
    }

    try
    {
      IFolder folder = mProject.getFolder(pFolder);
      IResource[] resources = folder.members();
      if ((resources == null) || (resources.length < 1))
      {
        return false;
      }
      for (int i = 0, sizei = resources.length; i < sizei; ++i)
      {
        String ext = resources[i].getFileExtension();
        if ("jar".equalsIgnoreCase(ext))
        {
          return true;
        }
      }
    }
    catch (CoreException e)
    {
      ExoUIPlugin.log(e.getStatus());
    }

    return false;
  }

  /**
   * Filter to remove any IFile resources.
   */
  private class FileFilter extends ViewerFilter
  {
    /* (non-Javadoc)
     * Method declared on ViewerFilter.
     */
    public boolean select(Viewer viewer, Object parentElement, Object element)
    {
      IResource resource = null;
      if (element instanceof IResource)
      {
        resource = (IResource) element;
      }
      else
      {
        if (element instanceof IAdaptable)
        {
          IAdaptable adaptable = (IAdaptable) element;
          resource = (IResource) adaptable.getAdapter(IResource.class);
        }
      }

      if (resource != null)
      {
        if (mProject != resource.getProject())
        {
          return false;
        }

        return resource.getType() != IResource.FILE;
      }
      else
      {
        return false;
      }
    }
  }
}
