/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.internal.ui.wizard;


import java.io.File;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.help.WorkbenchHelp;
import org.exoplatform.eclipse.core.runtime.IExoProjectRuntimeProperties;
import org.exoplatform.eclipse.ui.ExoUIPlugin;
import org.exoplatform.eclipse.ui.IExoHelpContextIds;
import org.exoplatform.eclipse.ui.IExoUIPreferencesConstant;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;
import org.exoplatform.eclipse.ui.validator.InputTextValidator;

/**
 * This wizard page allows the user to input the following settings: 1- Context
 * root name  2- Deployment directory  3- Deployment overwrite options
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class UpgradeToPortletDeploymentPage extends WizardPage
{
  public static final String CLASS_VERSION =
    "$Id: UpgradeToPortletDeploymentPage.java,v 1.1 2004/04/19 03:37:24 hatimk Exp $"; //$NON-NLS-1$
  public static final String DEFAULT_CONTEXTROOT_TEXT = ""; //$NON-NLS-1$    
  public static final String DEFAULT_DEPLOYMENTDIR_TEXT = ""; //$NON-NLS-1$      
  private IProject mProject;
  private Text mContextRootText;
  private Text mDeploymentDirText;
  private Button mWebFolderBrowseButton;
  private Button mDeploymentOption1;
  private Button mDeploymentOption2;
  private Button mDeploymentDirBrowseButton;
  private Button mDeploymentOverwriteButton;
  private Button mDetectWebXMLButton;
  private Button mWebXMLOverwriteButton;
  private Label mWebXMLOverwriteDesc;
  private Label mWebXMLOverwriteDesc2;
  private SelectionListener mWebXMLOptionsListener =
    new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent pEvent)
      {
        updateWebXMLOptions();
      }
    };

  private Listener mTextModifyListener =
    new Listener()
    {
      public void handleEvent(Event e)
      {
        setPageComplete(validatePage());
      }
    };

  /**
   * Creates a wizard page.
   *
   * @param pPageName the name of this page
   */
  public UpgradeToPortletDeploymentPage(String pPageName)
  {
    super(pPageName);
    setPageComplete(true);
  }

  /**   
   * The project in question
   *
   * @param pSelectedProject the project in question
   */
  public void init(IProject pSelectedProject)
  {
    mProject = pSelectedProject;
  }

  /**   
   * Returns the value of the contextroot with leading and trailing spaces
   * removed.
   *
   * @return the contextroot
   */
  public String getContextRoot()
  {
    if (mContextRootText == null)
    {
      return ""; //$NON-NLS-1$
    }
    else
    {
      return mContextRootText.getText().trim();
    }
  }

  /**   
   * Returns the value of the deployment directory location with leading and
   * trailing spaces removed.
   *
   * @return the deployment directory location
   */
  public String getDeploymentFolder()
  {
    if (mDeploymentDirText == null)
    {
      return ""; //$NON-NLS-1$
    }
    else
    {
      return mDeploymentDirText.getText().trim();
    }
  }

  /**   
   * Returns the deployment option. The exploded option is returned by default
   * if none is selected by the user. To see all possible values please visit
   * IExoProjectRuntimeProperties interface.
   *
   * @return the deployment option
   */
  public int getDeploymentOption()
  {
    if (mDeploymentOption1 == null)
    {
      return IExoProjectRuntimeProperties.DEPLOYMENT_OPTION_EXPLODED;
    }
    if (mDeploymentOption1.getSelection())
    {
      return IExoProjectRuntimeProperties.DEPLOYMENT_OPTION_EXPLODED;
    }
    else
    {
      return IExoProjectRuntimeProperties.DEPLOYMENT_OPTION_PACKAGED;
    }
  }

  /**   
   * Returns the deployment overwrite option.
   *
   * @return the deployment overwrite option
   */
  public boolean getDeploymentOverwriteOption()
  {
    return (mDeploymentOverwriteButton != null)
    ? mDeploymentOverwriteButton.getSelection()
    : IExoProjectRuntimeProperties.OVERWRITE_PROMPT_DEFAULT;
  }

  /**   
   * Returns the web.xml display-name overwrite option.
   *
   * @return the web.xml display-name overwrite option
   */
  public boolean getWebXMLOverwriteOption()
  {
    return (mWebXMLOverwriteButton != null)
    ? mWebXMLOverwriteButton.getSelection()
    : IExoProjectRuntimeProperties.OVERWRITE_WEBXML_PROMPT_DEFAULT;
  }

  /**   
   * Returns the detect web.xml display-name option.
   *
   * @return the detect web.xml display-name option
   */
  public boolean getDetectWebXMLOption()
  {
    return (mDetectWebXMLButton != null) ? mDetectWebXMLButton.getSelection()
                                         : IExoProjectRuntimeProperties.DETECT_WEBXML_FORMODIFICATION_OPTION_DEFAULT;
  }

  /**   
   * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
   */
  public void createControl(Composite pParent)
  {
    Composite mainComposite =
      UICompositeUtil.createFillBothComposite(pParent, 1, 1);
    initializeDialogUnits(pParent);
    createContextRootGroup(mainComposite);
    createDeploymentDirectoryGroup(mainComposite);
    createDeploymentOptionGroup(mainComposite);
    createOverwritePromptGroup(mainComposite);
    createOverwriteWebXMLPromptGroup(mainComposite);
    setPageComplete(validatePage());

    WorkbenchHelp.setHelp(mainComposite,
      IExoHelpContextIds.UPGRADE_PORTLET_DEPLOYMENT_PAGE);
    setControl(mainComposite);
  }

  /*
   * see @DialogPage.setVisible(boolean)
   */
  public void setVisible(boolean visible)
  {
    super.setVisible(visible);
    if ((visible) && (mContextRootText != null))
    {
      mContextRootText.setFocus();
    }
  }

  private void createContextRootGroup(Composite pParent)
  {
    Group group =
      UICompositeUtil.createGroupWithNoGrapping(pParent,
        "Context root setting", 2, 1);

    UICompositeUtil.addLabelWithWrapping(group,
      "The context root to use during deployment : ", 2);
    mContextRootText =
      UICompositeUtil.addText(group, DEFAULT_CONTEXTROOT_TEXT, 2);
    mContextRootText.addListener(SWT.Modify, mTextModifyListener);
  }

  private void createDeploymentDirectoryGroup(Composite pParent)
  {
    Font font = pParent.getFont();

    Composite group =
      UICompositeUtil.createGroupWithNoGrapping(pParent,
        "Deployment Directory", 3, 1);
    UICompositeUtil.addLabelWithWrapping(group,
      "The directory to use to deploy this web application to : ", 3);
    mDeploymentDirText =
      UICompositeUtil.addText(group, DEFAULT_DEPLOYMENTDIR_TEXT, 2);

    // browse button
    mDeploymentDirBrowseButton = new Button(group, SWT.PUSH);
    mDeploymentDirBrowseButton.setText("Br&owse..");
    mDeploymentDirBrowseButton.addSelectionListener(new SelectionAdapter()
      {
        public void widgetSelected(SelectionEvent event)
        {
          String loc = ""; //$NON-NLS-1$
          if (mDeploymentDirText != null)
          {
            loc = mDeploymentDirText.getText();
          }

          handleDeploymentDirBrowseButtonPressed(loc);
        }
      });

    mDeploymentDirBrowseButton.setEnabled(true);
    mDeploymentDirBrowseButton.setFont(font);
    setButtonLayoutData(mDeploymentDirBrowseButton);

    // Set the initial value first before listener
    // to avoid handling an event during the creation.
    String initialLocation =
      ExoUIPlugin.getDefault().getPreferenceStore().getString(IExoUIPreferencesConstant.PREF_DEPLOYMENT_DIR);
    mDeploymentDirText.setText(initialLocation);
    mDeploymentDirText.addListener(SWT.Modify, mTextModifyListener);
  }

  private void createDeploymentOptionGroup(Composite pParent)
  {
    Font font = pParent.getFont();

    Composite group =
      UICompositeUtil.createGroupWithNoGrapping(pParent, "Deployment Option",
        1, 1);
    UICompositeUtil.addLabel(group, "The type of deployment that you prefer :",
      1);
    mDeploymentOption1 =
      UICompositeUtil.createRadioButton(group, "&Exploded Archive");
    mDeploymentOption2 =
      UICompositeUtil.createRadioButton(group, "&Packaged Archive");

    switch (IExoProjectRuntimeProperties.DEPLOYMENT_OPTION_DEFAULT)
    {
      case IExoProjectRuntimeProperties.DEPLOYMENT_OPTION_PACKAGED:
        mDeploymentOption1.setSelection(false);
        mDeploymentOption2.setSelection(true);

        break;

      default:
        mDeploymentOption1.setSelection(true);
        mDeploymentOption2.setSelection(false);
    }
  }

  private void createOverwritePromptGroup(Composite pParent)
  {
    Font font = pParent.getFont();

    Composite group =
      UICompositeUtil.createGroupWithNoGrapping(pParent,
        "Deployment Overwrite Option", 1, 1);
    mDeploymentOverwriteButton =
      UICompositeUtil.createCheckboxButton(group,
        "A&lways prompt me before overwriting any deployment file.");
    mDeploymentOverwriteButton.setSelection(IExoProjectRuntimeProperties.OVERWRITE_PROMPT_DEFAULT);

    String overwriteDesc =
      "If you uncheck the box, then deployment files will always be "
      + "overwritten. The selection you make for this checkbox only "
      + "takes effect for the selected project. Each project has its "
      + "own deployment overwrite option.";
    UICompositeUtil.addLabelWithWrapping(group, overwriteDesc, 1);
  }

  private void createOverwriteWebXMLPromptGroup(Composite pParent)
  {
    Font font = pParent.getFont();

    Composite group =
      UICompositeUtil.createGroupWithNoGrapping(pParent,
        "eXo web.xml Modification Dialog Option", 1, 1);
    mDetectWebXMLButton =
      UICompositeUtil.createCheckboxButton(group,
        "Al&ways detect if the web.xml file needs modification.");

    mWebXMLOverwriteButton =
      UICompositeUtil.createCheckboxButton(group,
        "Alw&ays prompt me before modifying the web.xml file.");

    String overwriteDesc =
      "If you uncheck the box, then web.xml file will be modified without prompting you first. "
      + "The selection you make for this checkbox only "
      + "takes effect for the selected project. Each project has its "
      + "own eXo web.xml modification option.";

    mWebXMLOverwriteDesc =
      UICompositeUtil.addLabelWithWrapping(group, overwriteDesc, 1);
    UICompositeUtil.addLabelWithWrapping(group, "", 1);

    String warningDesc =
      "WARNING: The existing format of web.xml will NOT be preserved. However, all the comments in the "
      + "web.xml will not be affected.";
    mWebXMLOverwriteDesc2 =
      UICompositeUtil.addLabelWithWrapping(group, warningDesc, 1);

    mDetectWebXMLButton.setSelection(IExoProjectRuntimeProperties.DETECT_WEBXML_FORMODIFICATION_OPTION_DEFAULT);
    mWebXMLOverwriteButton.setSelection(IExoProjectRuntimeProperties.OVERWRITE_WEBXML_PROMPT_DEFAULT);
    mDetectWebXMLButton.addSelectionListener(mWebXMLOptionsListener);
    mWebXMLOverwriteButton.addSelectionListener(mWebXMLOptionsListener);
  }

  /**   
   * Returns whether this page's controls currently all contain valid  values.
   *
   * @return <code>true</code> if all controls are valid, and
   *         <code>false</code> if at least one is invalid
   */
  protected boolean validatePage()
  {
    String contextRoot = DEFAULT_CONTEXTROOT_TEXT;
    String deploymentDir = DEFAULT_DEPLOYMENTDIR_TEXT;
    if (mContextRootText != null)
    {
      contextRoot = mContextRootText.getText();
    }
    if (mDeploymentDirText != null)
    {
      deploymentDir = mDeploymentDirText.getText();
    }
    
    // validate context root
    IStatus validationStatus =
      InputTextValidator.validateContextRootName(contextRoot);
    if (validationStatus.getSeverity() == IStatus.INFO)
    {
      setErrorMessage(null);
      setMessage(validationStatus.getMessage());

      return false;
    }
    else if (validationStatus.getSeverity() != IStatus.OK)
    {
      setMessage(null);
      setErrorMessage(validationStatus.getMessage());

      return false;
    }

    
    // validate deployment directory
    validationStatus =
      InputTextValidator.validateDeploymentDirectory(deploymentDir);
    if (validationStatus.getSeverity() == IStatus.INFO)
    {
      setErrorMessage(null);
      setMessage(validationStatus.getMessage());

      return false;
    }
    else if (validationStatus.getSeverity() != IStatus.OK)
    {
      setMessage(null);
      setErrorMessage(validationStatus.getMessage());

      return false;
    }

    setErrorMessage(null);
    setMessage(null);

    return true;
  }

  private void updateWebXMLOptions()
  {
    if ((mDetectWebXMLButton != null) && (mWebXMLOverwriteButton != null))
    {
      if (mDetectWebXMLButton.getSelection())
      {
        mWebXMLOverwriteButton.setEnabled(true);
        mWebXMLOverwriteDesc.setEnabled(true);
        mWebXMLOverwriteDesc2.setEnabled(true);        
      }
      else
      {
        mWebXMLOverwriteButton.setEnabled(false);
        mWebXMLOverwriteDesc.setEnabled(false);
        mWebXMLOverwriteDesc2.setEnabled(false);        
      }
    }
  }

  /**   
   * Open an appropriate directory browser
   *
   * @param pLocation the location to start with
   */
  private void handleDeploymentDirBrowseButtonPressed(String pLocation)
  {
    DirectoryDialog dialog = new DirectoryDialog(mDeploymentDirText.getShell());
    dialog.setMessage("Please choose the deployment directory location");

    String dirName = pLocation;

    if (!dirName.equals("")) //$NON-NLS-1$
    {
      File path = new File(dirName);

      if (path.exists())
      {
        dialog.setFilterPath(new Path(dirName).toOSString());
      }
    }

    String selectedDirectory = dialog.open();

    if (selectedDirectory != null)
    {
      mDeploymentDirText.setText(selectedDirectory);
    }
  }

  /**
   * Filter to remove any IFile resources.
   */
  private class FileFilter extends ViewerFilter
  {
    /* (non-Javadoc)
     * Method declared on ViewerFilter.
     */
    public boolean select(Viewer viewer, Object parentElement, Object element)
    {
      IResource resource = null;
      if (element instanceof IResource)
      {
        resource = (IResource) element;
      }
      else
      {
        if (element instanceof IAdaptable)
        {
          IAdaptable adaptable = (IAdaptable) element;
          resource = (IResource) adaptable.getAdapter(IResource.class);
        }
      }

      if (resource != null)
      {
        if (mProject != resource.getProject())
        {
          return false;
        }

        return resource.getType() != IResource.FILE;
      }
      else
      {
        return false;
      }
    }
  }
}
