/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.internal.ui.wizard;


import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.ui.actions.WorkbenchRunnableAdapter;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.wizard.Wizard;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.exoplatform.eclipse.core.IPreferencesConstant;
import org.exoplatform.eclipse.core.common.IOverwriteQuery;
import org.exoplatform.eclipse.core.operation.ChangeWebFolderOperation;
import org.exoplatform.eclipse.core.operation.SimplePortletProjectStoreOperation;
import org.exoplatform.eclipse.core.operation.helper.WebappProjectSettings;
import org.exoplatform.eclipse.core.runtime.IExoProjectRuntimeProperties;
import org.exoplatform.eclipse.internal.ui.sample.IExoSampleConstant;
import org.exoplatform.eclipse.internal.ui.sample.SampleWebappInitializer;
import org.exoplatform.eclipse.internal.ui.sample.SampleWebappProject;
import org.exoplatform.eclipse.ui.ExoUIPlugin;
import org.exoplatform.eclipse.ui.ExoUIPluginImages;
import org.exoplatform.eclipse.ui.IExoUIPreferencesConstant;

import java.lang.reflect.InvocationTargetException;

/**
 * A wizard the helps upgrade an existing java project to a portlet project.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class UpgradeToPortletProjectWizard extends Wizard
{
  public static final String CLASS_VERSION = "$Id: UpgradeToPortletProjectWizard.java,v 1.1 2004/04/19 03:37:24 hatimk Exp $"; //$NON-NLS-1$
  private boolean mFatalErrorOccurred = false;
  private SelectWebFolderPage mWebFolderPage;
  private UpgradeToPortletDeploymentPage mDeploymentPage;
  private IProject mProject;

  /**
   * The constructor method.
   */
  public UpgradeToPortletProjectWizard()
  {
    super();
    setWindowTitle("Upgrade an existing Java project to a Portlet project");
    setNeedsProgressMonitor(true);
    setDefaultPageImageDescriptor(ExoUIPluginImages.DESC_WIZBAN_UPGRADE_TO_PORTLET);
  }

  /**   
   * The project to upgrade
   *
   * @param pSelectedProject the project to upgrade
   */
  public void init(IProject pSelectedProject)
  {
    mProject = pSelectedProject;
  }

  /**   
   * @see org.eclipse.jface.wizard.Wizard#addPages()
   */
  public void addPages()
  {
    super.addPages();

    mWebFolderPage = new SelectWebFolderPage("SelectWebFolderPage"); //$NON-NLS-1$
    mWebFolderPage.setTitle("Web Content Settings Page");
    mWebFolderPage.setDescription(
      "Specify the web content folder from the given list, or create a new one.");
    mWebFolderPage.init(mProject);
    mWebFolderPage.setSelectTitle(
      "Select the folder that contains the web content (jsp pages, WEB-INF directory, etc.):");
    mWebFolderPage.setGroupTitle("Web content folder setting");

    mDeploymentPage =
      new UpgradeToPortletDeploymentPage("UpgradeToPortletDeploymentPage"); //$NON-NLS-1$
    mDeploymentPage.init(mProject);
    mDeploymentPage.setTitle("Deployment Settings Page");
    mDeploymentPage.setDescription(
      "Specify the deployment settings such as deployment directory.");

    addPage(mWebFolderPage);
    addPage(mDeploymentPage);
  }

  /**   
   * This method is run in the wizard container's context as a workspace
   * runnable.
   *
   * @param pMonitor to report progress
   *
   * @throws CoreException something went wrong
   * @throws InvocationTargetException
   * @throws InterruptedException we got interrupted
   */
  protected void finishPage(IProgressMonitor pMonitor)
    throws CoreException, InvocationTargetException, InterruptedException
  {
    try
    {
      pMonitor.beginTask("Upgrading to a portlet project : ", 900);

      // gather all the information from all the pages in preparation of 
      // the project upgrade
      String deploymentFolder = mDeploymentPage.getDeploymentFolder();
      String deploymentOption =
        String.valueOf(mDeploymentPage.getDeploymentOption());
      boolean deploymentOverwritePrompt =
        mDeploymentPage.getDeploymentOverwriteOption();
      boolean detectWebXMLOption = 
        mDeploymentPage.getDetectWebXMLOption();
      boolean webxmlOverwritePrompt =
        mDeploymentPage.getWebXMLOverwriteOption();
      IPath contextRootPath = new Path(mDeploymentPage.getContextRoot());

      pMonitor.subTask(
        "process the project description and adds the portlet nature");

      IProjectDescription description = mProject.getDescription();
      String[] natures = description.getNatureIds();
      String[] newNatures = new String[natures.length + 1];
      System.arraycopy(natures, 0, newNatures, 1, natures.length);
      newNatures[0] = ExoCorePlugin.PORTLET_NATURE_ID;
      description.setNatureIds(newNatures);
      mProject.setDescription(description, new SubProgressMonitor(pMonitor, 100));

      if (pMonitor.isCanceled())
      {
        throw new InterruptedException();
      }

      pMonitor.subTask("updating the web settings for the project");

      // now add the context root and the web folder to the portlet project
      // xml store
      SimplePortletProjectStoreOperation storeOp =
        new SimplePortletProjectStoreOperation(mProject);
      storeOp.addPropertyToModify(IPreferencesConstant.PORTLET_PROJECT_STORE_CONTEXTROOT,
        contextRootPath.toString());

      IResource webfolder = mWebFolderPage.getResult();
      if (webfolder != null)
      {
        IPath webpath = webfolder.getProjectRelativePath();
        storeOp.addPropertyToModify(IPreferencesConstant.PORTLET_PROJECT_STORE_WEBFOLDER,
          webpath.toString());
      }
      storeOp.run(new SubProgressMonitor(pMonitor, 100));

      // now add the deployment directory as a resource property for this project      
      mProject.setPersistentProperty(IExoProjectRuntimeProperties.DEPLOYMENT_DIR,
        deploymentFolder);
      pMonitor.worked(100);

      // now add the deployment option as a resource property for this project      
      mProject.setPersistentProperty(IExoProjectRuntimeProperties.DEPLOYMENT_OPTION,
        deploymentOption);
      pMonitor.worked(100);

      // now add the deployment overwrite option as a resource property for this project
      mProject.setPersistentProperty(IExoProjectRuntimeProperties.OVERWRITE_PROMPT,
        String.valueOf(deploymentOverwritePrompt));
      pMonitor.worked(100);

      // now add the detect web.xml display-name tag option as a resource property for this project
      mProject.setPersistentProperty(IExoProjectRuntimeProperties.DETECT_WEBXML_FORMODIFICATION_OPTION,
        String.valueOf(detectWebXMLOption));
      pMonitor.worked(100);

      // now add the web.xml display-name overwrite option as a resource property for this project
      mProject.setPersistentProperty(IExoProjectRuntimeProperties.OVERWRITE_WEBXML_PROMPT,
        String.valueOf(webxmlOverwritePrompt));
      pMonitor.worked(100);
      
      // remember the deployment directory 
      ExoUIPlugin.getDefault().getPreferenceStore().setValue(IExoUIPreferencesConstant.PREF_DEPLOYMENT_DIR,
        deploymentFolder);

      // add the exolibrary (if not there already) to the classpath
      addToExoLibToClasspath(mProject, new SubProgressMonitor(pMonitor, 100));

      // see if we need to upgrade the web content folder too
      if (webfolder != null)
      {
        IPath webpath = webfolder.getProjectRelativePath();
        if (!webpath.isEmpty())
        {
          pMonitor.subTask("modifying web content folder");

          SampleWebappProject[] webapps =
            SampleWebappInitializer.getSampleWebappProjects(IExoSampleConstant.KIND_PORTLET_UPGRADE);
          WebappProjectSettings websettings =
            SampleWebappInitializer.getUpgradeWebappSettings(webapps[0], null,
              null, webpath);
          ChangeWebFolderOperation op =
            new ChangeWebFolderOperation(mProject, websettings,
              mWebFolderPage.getMakeDefaultBinaryOption(),
              mWebFolderPage.getAddJarsToClasspathOption(),
              new NoOverwriteQuery());
          op.run(new SubProgressMonitor(pMonitor, 100));
        }
        else
        {
          pMonitor.worked(100);
        }
      }
      else
      {
        pMonitor.worked(100);
      }
    }
    finally
    {
      pMonitor.done();
    }
  }

  /**   
   * @see Wizard#performFinish
   */
  public boolean performFinish()
  {
    IWorkspaceRunnable op =
      new IWorkspaceRunnable()
      {
        public void run(IProgressMonitor monitor)
          throws CoreException, OperationCanceledException
        {
          try
          {
            finishPage(monitor);
          }
          catch (InvocationTargetException eTarget)
          {
            Throwable e = eTarget.getTargetException();
            throw new CoreException(ExoUIPlugin.createErrorStatus(
                e.getMessage(), e));
          }
          catch (InterruptedException e)
          {
            throw new OperationCanceledException(e.getMessage());
          }
        }
      };

    try
    {
      getContainer().run(false, true, new WorkbenchRunnableAdapter(op));
      mFatalErrorOccurred = false;
    }
    catch (InvocationTargetException e)
    {
      handleException(e.getTargetException());
      mFatalErrorOccurred = true;

      return false;
    }
    catch (InterruptedException e)
    {
      mFatalErrorOccurred = true;

      return false;
    }

    return true;
  }

  /**   
   * @see org.eclipse.jface.wizard.IWizard#canFinish()
   */
  public boolean canFinish()
  {
    return super.canFinish() && !mFatalErrorOccurred;
  }

  protected void handleException(Throwable pTarget)
  {
    String title = "Error while upgrading to a portlet project";

    if (pTarget instanceof CoreException)
    {
      IStatus status = ((CoreException) pTarget).getStatus();
      ExoUIPlugin.log(status);
      ErrorDialog.openError(getShell(), title, null, status);
    }
    else
    {
      ExoUIPlugin.logError(pTarget);
      MessageDialog.openError(getShell(), title, title + " : " + pTarget);
    }
  }

  protected void addToExoLibToClasspath(IProject pProject,
    IProgressMonitor pMonitor) throws JavaModelException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }
    try
    {
      pMonitor.beginTask("adding the exo lib to the classpath", 1000);

      IJavaProject javaProject = JavaCore.create(pProject);
      IPath exoLib = new Path(IExoSampleConstant.EXO_PORTLET_LIBRARY_PATH);
      IClasspathEntry[] oldEntries = javaProject.getRawClasspath();

      // check if the exo library is already there 
      if (oldEntries != null)
      {
        for (int i = 0, sizei = oldEntries.length; i < sizei; ++i)
        {
          if (exoLib.equals(oldEntries[i].getPath()))
          {
            pMonitor.worked(1000);

            return;
          }
        }
      }
      else
      {
        return;
      }

      IClasspathEntry[] newEntries = new IClasspathEntry[oldEntries.length + 1];
      System.arraycopy(oldEntries, 0, newEntries, 0, oldEntries.length);
      newEntries[oldEntries.length] = JavaCore.newContainerEntry(exoLib);
      javaProject.setRawClasspath(newEntries,
        new SubProgressMonitor(pMonitor, 1000));
    }
    finally
    {
      pMonitor.done();
    }
  }

  // always answer not to overwrite any files
  protected class NoOverwriteQuery implements IOverwriteQuery
  {
    public String queryOverwrite(String file)
    {
      String[] returnCodes = { YES, NO, ALL, NO_ALL, CANCEL };
      int returnVal = 3;

      return (returnVal < 0) ? CANCEL : returnCodes[returnVal];
    }
  }

  // overwrite dialog
  protected class PromptOverwriteQuery implements IOverwriteQuery
  {
    public String queryOverwrite(String file)
    {
      String[] returnCodes = { YES, NO, ALL, NO_ALL, CANCEL };
      int returnVal = openDialog(file);

      return (returnVal < 0) ? CANCEL : returnCodes[returnVal];
    }

    private int openDialog(final String file)
    {
      final int[] result = { IDialogConstants.CANCEL_ID };
      getShell().getDisplay().syncExec(new Runnable()
        {
          public void run()
          {
            String title = "Overwrite";
            String msg = "Do you want to overwrite " + file + "?";
            String[] options =
            {
              IDialogConstants.YES_LABEL, IDialogConstants.NO_LABEL,
              IDialogConstants.YES_TO_ALL_LABEL,
              IDialogConstants.NO_TO_ALL_LABEL, IDialogConstants.CANCEL_LABEL
            };
            MessageDialog dialog =
              new MessageDialog(getShell(), title, null, msg,
                MessageDialog.QUESTION, options, 0);
            result[0] = dialog.open();
          }
        });

      return result[0];
    }
  }
}
