/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.ui;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.ui.plugin.AbstractUIPlugin;

/**
 * The plugin runtime class for the eXo ui plugin. At this time this class does
 * not take advantage of any life cycle requests issued by the platform. It is
 * here just to provide convenient logging methods.
 * 
 * <p>
 * Like all plugin runtime classes (subclasses of <code>Plugin</code>), this
 * class is automatically instantiated by the platform when the plug-in gets
 * activated. Clients must not attempt to instantiate plugin runtime classes
 * directly.
 * </p>
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class ExoUIPlugin extends AbstractUIPlugin
{
  public static final String CLASS_VERSION =
    "$Id: ExoUIPlugin.java,v 1.1 2004/04/19 03:37:25 hatimk Exp $"; //$NON-NLS-1$
  public static final String PLUGIN_ID = "org.exoplatform.eclipse.ui"; //$NON-NLS-1$
  private static final String EMPTY_STRING = ""; //$NON-NLS-1$
  private static ExoUIPlugin mPlugin;

  /**
   * The constructor. It needs to be public, so that the platform can
   * instantiate it. No other clients should directly do that.
   *
   * @param pDescriptor the plugin descriptor provided by the platform
   */
  public ExoUIPlugin()
  {
    super();
    mPlugin = this;
  }

  /**   
   * Returns the shared instance.
   *
   * @return plugin the eXo ui plugin instance
   */
  public static ExoUIPlugin getDefault()
  {
    return mPlugin;
  }

  /**   
   * Writes an error message to the plugin's log
   *
   * @param pMessage the text to write to the log
   * @param pException the exception to print its stack trace to the log
   */
  public static void logError(String pMessage, Throwable pException)
  {
    IStatus status = createErrorStatus(pMessage, pException);
    getDefault().getLog().log(status);
  }

  /**   
   * Writes an IStatus to the plugin's log
   *
   * @param pStatus the status that contains the information to log
   */
  public static void log(IStatus pStatus)
  {
    getDefault().getLog().log(pStatus);
  }

  /**   
   * Writes an error message to the plugin's log. The error message text itself
   * is hardcoded in this method.
   *
   * @param pException the exception to print its stack trace to the log
   */
  public static void logError(Throwable pException)
  {
    //this message is intentionally not internationalized, as an exception may
    // be due to the resource bundle itself
    String message = EMPTY_STRING;
    if (pException != null)
    {
      message = pException.getMessage();
    }
    getDefault().getLog().log(createErrorStatus(
        "Internal error logged from eXo ui plugin : " + message, pException)); //$NON-NLS-1$
  }

  /**   
   * Returns a new <code>IStatus</code> for this plugin that encapsulates the
   * given error message and the given exception.
   *
   * @param pMessage the message to encapsulate in this status
   * @param pException the exception to encapsulate in this status
   *
   * @return return the status object populated with the given message and
   *         exception
   */
  public static IStatus createErrorStatus(String pMessage, Throwable pException)
  {
    if (pMessage == null)
    {
      pMessage = EMPTY_STRING;
    }

    return new Status(IStatus.ERROR, PLUGIN_ID, 0, pMessage, pException);
  }
}
