/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.ui.common;


import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontMetrics;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.exoplatform.eclipse.core.common.IOverwriteQuery;
import org.exoplatform.eclipse.core.runtime.IExoProjectRuntimeProperties;

import java.io.File;

/**
 * Implements the IOverwriteQuery. It will represent the user with the
 * overwrite dialog only if the given resource allows for overwrite prompt.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class DeploymentOverwriteQuery implements IOverwriteQuery
{
  public static final String CLASS_VERSION =
    "$Id: DeploymentOverwriteQuery.java,v 1.1 2004/04/19 03:37:38 hatimk Exp $"; //$NON-NLS-1$
  private IResource mResource;
  private Shell mShell;
  private boolean mAllSelected = false;
  private static final int MAX_FILE_NAME_LENGTH = 70;

  /**
   * The constructor method.
   *
   * @param pResource
   * @param pShell
   */
  public DeploymentOverwriteQuery(IResource pResource, Shell pShell)
  {
    mResource = pResource;
    mShell = pShell;
  }

  /**   
   * Returns one of the return code constants declared on IOverwriteQuery,
   * indicating whether the entity represented by the passed String should be
   * overwritten.
   *
   * @param pFile should this entity be overwritten
   *
   * @return see IOverwriteQuery.YES|NO|ALL|CANCEL
   */
  public String queryOverwrite(String pFile)
  {
    String[] returnCodes = { YES, NO, ALL, CANCEL };
    boolean promptAgain = getOverwritePrompt(mResource);
    if (!promptAgain)
    {
      return ALL;
    }
    
    if (mAllSelected)
    {
      return ALL;
    }

    int returnVal = openDialog(pFile);
    if (returnVal == 2) mAllSelected = true;

    return (returnVal < 0) ? CANCEL : returnCodes[returnVal];
  }

  private int openDialog(final String pFileName)
  {
    final int[] result = { IDialogConstants.CANCEL_ID };
    mShell.getDisplay().syncExec(new Runnable()
      {
        public void run()
        {
          String title = "Overwrite";
          String msg =
            "Do you want to overwrite\n" + multiLineFileName(pFileName) + "?";
          DeploymentOverwriteDialog dialog =
            new DeploymentOverwriteDialog(mShell, title, null, msg,
              DeploymentOverwriteDialog.QUESTION, 0, mResource);
          result[0] = dialog.open();
        }
      });

    return result[0];
  }

  private static boolean getOverwritePrompt(IResource pResource)
  {
    boolean result = IExoProjectRuntimeProperties.OVERWRITE_PROMPT_DEFAULT;
    String resultString = null;
    try
    {
      resultString =
        pResource.getPersistentProperty(IExoProjectRuntimeProperties.OVERWRITE_PROMPT);
      if ((resultString == null) || (resultString.length() < 1))
      {
        pResource.setPersistentProperty(IExoProjectRuntimeProperties.OVERWRITE_PROMPT,
          String.valueOf(result));
      }
      else
      {
        result = Boolean.valueOf(resultString).booleanValue();
      }
    }
    catch (Exception e)
    {
    }

    return result;
  }

  private static String multiLineFileName(String pFileName)
  {
    if ((pFileName == null) || (pFileName.length() < MAX_FILE_NAME_LENGTH))
    {
      return pFileName;
    }

    IPath path = new Path(pFileName);

    StringBuffer sb = new StringBuffer();
    sb.append(path.getDevice());

    int currentLineLength = path.getDevice().length();
    for (int i = 0, sizei = path.segmentCount(); i < sizei; ++i)
    {
      sb.append(File.separator);
      ++currentLineLength;

      String currentSegment = path.segment(i);
      currentLineLength += currentSegment.length();
      if (currentLineLength > MAX_FILE_NAME_LENGTH)
      {
        sb.append("\n");
        currentLineLength = currentSegment.length();
      }
      sb.append(currentSegment);
    }

    return sb.toString();
  }

  private class DeploymentOverwriteDialog extends MessageDialog
    implements SelectionListener
  {
    public static final String CLASS_VERSION =
      "$Id: DeploymentOverwriteQuery.java,v 1.1 2004/04/19 03:37:38 hatimk Exp $"; //$NON-NLS-1$
    private IResource mResource;
    private Button mOverwritePrompt;
    private boolean mCurrentSelection =
      IExoProjectRuntimeProperties.OVERWRITE_PROMPT_DEFAULT;
    private static final int YES_BUTTON_INDEX = 0;
    private static final int NO_BUTTON_INDEX = 1;
    private static final int YES_ALL_BUTTON_INDEX = 2;
    private static final int CANCEL_BUTTON_INDEX = 3;

    private DeploymentOverwriteDialog(Shell pParentShell, String pDialogTitle,
      Image pDialogTitleImage, String pDialogMessage, int pDialogImageType,
      int pDefaultIndex, IResource pResource)
    {
      super(pParentShell, pDialogTitle, pDialogTitleImage, pDialogMessage,
        pDialogImageType,
        new String[]
        {
          IDialogConstants.YES_LABEL, IDialogConstants.NO_LABEL,
          IDialogConstants.YES_TO_ALL_LABEL, IDialogConstants.CANCEL_LABEL
        }, pDefaultIndex);
      mResource = pResource;
    }

    /**   
     * @see org.eclipse.jface.window.Window#open()
     */
    public int open()
    {
      int select = super.open();
      saveOverwritePrompt(mResource);

      return select;
    }

    /**   
     * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
     */
    public void widgetSelected(SelectionEvent e)
    {
      Button button = (Button) e.widget;
      mCurrentSelection = button.getSelection();
      updateButtonsEnablement(mCurrentSelection);
    }

    /**   
     * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
     */
    public void widgetDefaultSelected(SelectionEvent e)
    {
    }

    protected void updateButtonsEnablement(boolean pOverwritePromptSelection)
    {
      if (!pOverwritePromptSelection)
      {
        getButton(YES_BUTTON_INDEX).setEnabled(false);
        getButton(NO_BUTTON_INDEX).setEnabled(false);
        getButton(YES_ALL_BUTTON_INDEX).setEnabled(true);
        getButton(YES_ALL_BUTTON_INDEX).setFocus();
        getButton(CANCEL_BUTTON_INDEX).setEnabled(false);
      }
      else
      {
        getButton(YES_BUTTON_INDEX).setEnabled(true);
        getButton(NO_BUTTON_INDEX).setEnabled(true);
        getButton(YES_ALL_BUTTON_INDEX).setEnabled(true);
        getButton(CANCEL_BUTTON_INDEX).setEnabled(true);
      }
    }

    /**   
     * @see org.eclipse.jface.dialogs.IconAndMessageDialog#createButtonBar(org.eclipse.swt.widgets.Composite)
     */
    protected Control createButtonBar(Composite pParent)
    {
      Composite main = UICompositeUtil.createFillBothComposite(pParent, 1, 2);
      Composite parent = (Composite) super.createButtonBar(main);
      createOverwritePromptGroup(main, 1);

      return main;
    }

    private void createOverwritePromptGroup(Composite pParent,
      int pHorizontalSpan)
    {
      Font font = pParent.getFont();

      Composite group =
        createGroup(pParent, "Overwrite Option", 1, pHorizontalSpan);
      mOverwritePrompt =
        createCheckboxButton(group,
          "A&lways prompt me before overwriting any deployment file.");

      boolean selection = getOverwritePrompt(mResource);
      mOverwritePrompt.setSelection(selection);
      mCurrentSelection = selection;
      updateButtonsEnablement(mCurrentSelection);

      String overwriteDesc =
        "If you uncheck the box, then deployment files will always be overwritten and this dialog "
        + "will never appear again when deploying this project. If you wish to enable this dialog "
        + "again, you can find an option in the property page of this project. You can access the "
        + "property page by selecting the 'Project -> Properties' menu option.";
      addLabel(group, overwriteDesc, 1);

      addLabel(group, "", 1);

      String noteDesc =
        "NOTE: The selection you make for this checkbox only takes effect for the selected project. "
        + "Each project has its own deployment overwrite option.";
      addLabel(group, noteDesc, 1);
    }

    /**   
     * Utility method that creates a checkbox button instance and sets the
     * default layout data.
     *
     * @param pParent the parent for the new button
     * @param pLabel the label for the new button
     *
     * @return the newly-created button
     */
    private Button createCheckboxButton(Composite pParent, String pLabel)
    {
      Button button = new Button(pParent, SWT.CHECK | SWT.LEFT);
      button.setText(pLabel);

      GridData data = new GridData();
      button.setLayoutData(data);
      button.addSelectionListener(this);

      return button;
    }

    private Group createGroup(Composite pParent, String pTitle,
      int pNumColumns, int pHorizontalSpan)
    {
      Group group = new Group(pParent, SWT.NONE);
      group.setText(pTitle);

      GridLayout layout = new GridLayout();
      layout.marginHeight = 5;
      layout.marginWidth = 5;
      layout.numColumns = pNumColumns;

      GridData data = new GridData();
      data.horizontalAlignment = GridData.FILL;
      data.verticalAlignment = GridData.FILL;

      data.grabExcessHorizontalSpace = true;
      data.grabExcessVerticalSpace = true;
      data.horizontalSpan = pHorizontalSpan;
      group.setLayout(layout);
      group.setLayoutData(data);
      group.setFont(pParent.getFont());

      return group;
    }

    private Label addLabel(Composite pComp, String pText, int pHorizontalSpan)
    {
      return addLabel(pComp, pText, pHorizontalSpan, 0);
    }

    private Label addLabel(Composite pComp, String pText, int pHorizontalSpan,
      int pHorizontalIndent)
    {
      Label label = null;

      if (pText != null)
      {
        label = new Label(pComp, SWT.WRAP);

        // Compute a font metric to be used to figure out the width hint for the label
        GC gc = new GC(label);
        gc.setFont(pComp.getFont());

        FontMetrics fontMetrics = gc.getFontMetrics();
        gc.dispose();

        // knowing the font metrics start building the label      
        label.setText(pText);

        GridData data =
          new GridData(GridData.HORIZONTAL_ALIGN_FILL
            | GridData.VERTICAL_ALIGN_BEGINNING);
        data.widthHint =
          org.eclipse.jface.dialogs.Dialog.convertHorizontalDLUsToPixels(fontMetrics,
            IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
        label.setFont(pComp.getFont());
        data.horizontalSpan = pHorizontalSpan;
        data.horizontalIndent = pHorizontalIndent;
        label.setLayoutData(data);
      }

      return label;
    }

    private void saveOverwritePrompt(IResource pResource)
    {
      boolean overwriteSelection = mCurrentSelection;
      try
      {
        pResource.setPersistentProperty(IExoProjectRuntimeProperties.OVERWRITE_PROMPT,
          String.valueOf(overwriteSelection));
      }
      catch (CoreException e)
      {
      }
    }
  }
}
