/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.ui.dialog;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.exoplatform.eclipse.core.exoservices.Configuration;
import org.exoplatform.eclipse.core.exoservices.ServiceConfiguration;
import org.exoplatform.eclipse.core.portal.PortalSettings;
import org.exoplatform.eclipse.ui.ExoUIPluginImages;
import org.exoplatform.eclipse.ui.common.DidYouKnowMessages;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;

/**
 * @author Hatim Khan
 */
public class EditServiceDialog extends ExplainFormDialog
{
  public static final String TITLE = "Edit an eXo Platform service configuration";
  public static final String DID_YOU_KNOW = DidYouKnowMessages.getString("EditServicePage");//$NON-NLS-1$

  private PortalSettings mPortalSettings;
  private ServiceConfiguration mServiceConfiguration;
  private List mServicesKeys;
  private String mOriginalKey;
  private String mOriginalType;
  private boolean mChangeOccurred;
  private Text mKeyText;
  private Text mTypeText;

  public EditServiceDialog(Shell pParentShell, PortalSettings pPortalSettings,
      ServiceConfiguration pServiceConfiguration)
  {
    super(pParentShell, TITLE, DID_YOU_KNOW, ExoUIPluginImages.DESC_WIZBAN_EXO_SERVICES);
    mPortalSettings = pPortalSettings;
    mServiceConfiguration = pServiceConfiguration;
    mChangeOccurred = false;
  }

  /**
   * @see org.exoplatform.eclipse.ui.dialog.ExplainFormDialog#createActualContent(org.eclipse.swt.widgets.Composite)
   */
  public void createActualContent(Composite pParent)
  {
    setTitle("Edit an eXo Platform service configuration");
    setMessage("You have the option of changing the key, the type or both.");
    createKeyTypeContent(pParent);
    updateFromModel();
    attachListeners();
  }

  private void updateFromModel()
  {
    String key = "";
    String type = "";
    if (mServiceConfiguration != null)
    {
      key = UICompositeUtil.emptyIfNull(mServiceConfiguration.getKey()).trim();
      type = UICompositeUtil.emptyIfNull(mServiceConfiguration.getType()).trim();
    }

    mOriginalKey = key;
    mOriginalType = type;

    Configuration config = PortalSettings.findConfiguration(mServiceConfiguration);
    if (config != null)
    {
      mServicesKeys = config.getServicesKeys();
      // remove the original key from the list so we don't confuse it with the other keys
      if (key.length() > 0) mServicesKeys.remove(key);
      else mServicesKeys.remove(type);
    }
    else
    {
      mServicesKeys = new ArrayList();
    }

    mKeyText.setText(key);
    mTypeText.setText(type);
    mChangeOccurred = false;
  }

  protected void okPressed()
  {
    // first lets detect if the user changed either the key and/or the type, if so,
    // then we need to make the model dirty and fire an event
    String key = UICompositeUtil.emptyIfNull(mKeyText.getText()).trim();
    String type = UICompositeUtil.emptyIfNull(mTypeText.getText()).trim();
    if ((!key.equals(mOriginalKey)) || (!type.equals(mOriginalType)))
    {
      mServiceConfiguration.setKey((key.length() > 0) ? key : null);
      mServiceConfiguration.setType((type.length() > 0) ? type : null);
      mServiceConfiguration.setAppearIn(null);
      mChangeOccurred = true;
      mPortalSettings.markDirty();
      mPortalSettings.fireConfigurationElementUpdated(mServiceConfiguration);
    }
    super.okPressed();
  }

  private void validateInput()
  {
    String key = UICompositeUtil.emptyIfNull(mKeyText.getText()).trim();
    String type = UICompositeUtil.emptyIfNull(mTypeText.getText()).trim();

    // check if both are empty, if so then display an error
    if ((key.length() < 1) && (type.length() < 1))
    {
      reportError("The key and the value can not be empty at the same time.");
      return;
    }

    String proposedKey = (key.length() > 0) ? key : type;
    boolean existingKey = mServicesKeys.contains(proposedKey);
    // check if the proposed key matches an existing key (except the original service)
    if (existingKey)
    {
      reportError("There is an existing service configuration that conflicts with this one.");
      return;
    }

    // type can not be empty
    if (type.length() < 1)
    {
      reportError("The type can NOT be empty.");
      return;
    }

    // if we get to this point then validation is complete and we don't have any errors
    // so update the error message and the ok button
    setErrorMessage(null);
    Button button = getButton(IDialogConstants.OK_ID);
    if (button != null) button.setEnabled(true);
  }

  private void reportError(String pMessage)
  {
    setErrorMessage(pMessage);
    Button button = getButton(IDialogConstants.OK_ID);
    if (button != null) button.setEnabled(false);
  }

  private void createKeyTypeContent(Composite pParent)
  {
    Composite container = UICompositeUtil.createFillBothComposite(pParent, 2, 1);
    GridLayout gl = (GridLayout) container.getLayout();
    gl.makeColumnsEqualWidth = false;
    Dialog.applyDialogFont(container);

    UICompositeUtil.addLabel(container, "Key : ", 1);
    mKeyText = UICompositeUtil.addText(container, "", 1);

    UICompositeUtil.addLabel(container, "Type : ", 1);
    mTypeText = UICompositeUtil.addText(container, "", 1);
    UICompositeUtil.setWidthHint(mTypeText, IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
  }

  private void attachListeners()
  {
    mTypeText.addListener(SWT.Modify, new Listener()
    {

      public void handleEvent(Event pEvent)
      {
        validateInput();
      }
    });

    mKeyText.addListener(SWT.Modify, new Listener()
    {

      public void handleEvent(Event pEvent)
      {
        validateInput();
      }
    });
  }

  /**
   * @return Returns the changeOccurred.
   */
  public boolean isChangeOccurred()
  {
    return mChangeOccurred;
  }
}