/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.ui.validator;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.exoplatform.eclipse.ui.ExoUIPlugin;

/**
 * This class exposes static validation methods that are commonly used across wizards,
 * dialogs, property pages, and preferences pages.
 * 
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class InputTextValidator
{
  public static final String CLASS_VERSION = "$Id: InputTextValidator.java,v 1.1 2004/04/19 03:37:39 hatimk Exp $"; //$NON-NLS-1$

  /**
   * Given a proposed source folder name (relative to the project), this method validates
   * it syntax. Notice that a source folder that has a parent is considered invalid.
   * 
   * @param pSouceFolder the source folder name to validate its syntax
   * 
   * @return a status object with serverity and message the reflect validation result
   */
  public static IStatus validateSourceFolderName(String pSouceFolder)
  {
    int statusLevel = IStatus.ERROR;
    String message;

    if (pSouceFolder.equals(""))
    {
      return createStatus(IStatus.INFO, "Source folder must be specified");
    }

    IPath path = new Path(""); //$NON-NLS-1$
    if (!path.isValidPath(pSouceFolder))
    {
      return createStatus(IStatus.ERROR, "The source folder contains characters that are not allowed for a folder name");
    }

    path = new Path(pSouceFolder);
    //    if (path.segmentCount() > 1)
    //    {
    //      return createStatus(IStatus.ERROR,
    //        "The source folder can not have a parent folder that is not the project folder");
    //    }

    if (path.isRoot())
    {
      return createStatus(IStatus.ERROR, "Root folder is not allowd");
    }

    return createStatus(IStatus.OK, ""); //$NON-NLS-1$
  }

  /**
   * Given a proposed web folder name (relative to the project), this method validates it
   * syntax. Notice that a web folder that has a parent is considered invalid.
   * 
   * @param pWebFolder the web folder name to validate its syntax
   * 
   * @return a status object with serverity and message the reflect validation result
   */
  public static IStatus validateWebFolderName(String pWebFolder)
  {
    int statusLevel = IStatus.ERROR;
    String message;

    if (pWebFolder.equals(""))
    {
      return createStatus(IStatus.INFO, "Web content folder must be specified");
    }

    IPath path = new Path(""); //$NON-NLS-1$
    if (!path.isValidPath(pWebFolder))
    {
      return createStatus(IStatus.ERROR, "The web content folder contains characters that are not allowed for a folder name");
    }

    path = new Path(pWebFolder);
    // if ((path.segmentCount() > 1))
    //    {
    //      return createStatus(IStatus.ERROR,
    //        "The web content folder can not have a parent folder that is not the project
    // folder");
    //    }

    if (path.isRoot())
    {
      return createStatus(IStatus.ERROR, "Root folder is not allowd");
    }
    return createStatus(IStatus.OK, ""); //$NON-NLS-1$
  }

  /**
   * Given a proposed context root name, this method validates it syntax. Notice that a
   * context root that has a parent other than the root is considered invalid.
   * 
   * @param pContextRoot the context root to validate its syntax
   * 
   * @return a status object with serverity and message the reflect validation result
   */
  public static IStatus validateContextRootName(String pContextRoot)
  {
    int statusLevel = IStatus.ERROR;
    String message;

    // context root is optional, it can be empty
    if (pContextRoot.equals(""))
    {
      return createStatus(IStatus.OK, ""); //$NON-NLS-1$
    }

    IPath path = new Path(""); //$NON-NLS-1$
    if (!path.isValidPath(pContextRoot))
    {
      return createStatus(IStatus.ERROR, "The context root contains characters that are not allowed");
    }

    path = new Path(pContextRoot);
    if ((path.segmentCount() > 1))
    {
      return createStatus(IStatus.ERROR, "The context root should be a direct child of the root");
    }

    if (path.isRoot())
    {
      return createStatus(IStatus.ERROR, "Root is not allowd");
    }
    return createStatus(IStatus.OK, ""); //$NON-NLS-1$
  }

  /**
   * Given a proposed deployment directory, this method validates it syntax. Notice that
   * deployment directory that does not exist is considered invalid.
   * 
   * @param pDeploymentDir deployment directory to validate its syntax
   * 
   * @return a status object with serverity and message the reflect validation result
   */
  public static IStatus validateDeploymentDirectory(String pDeploymentDir)
  {
    int statusLevel = IStatus.ERROR;
    String message;

    // deployment directory is optional, it can be empty
    if (pDeploymentDir.equals(""))
    {
      return createStatus(IStatus.OK, ""); //$NON-NLS-1$
    }

    IPath path = new Path(""); //$NON-NLS-1$
    if (!path.isValidPath(pDeploymentDir))
    {
      return createStatus(IStatus.ERROR, "The deployment directory name contains characters that are not allowed");
    }

    /*
     * Validate if the location exists
     */
    path = new Path(pDeploymentDir);
    if (!(path.toFile().exists()))
    {
      return createStatus(IStatus.ERROR, "The deployment directory does not exist");
    }

    return createStatus(IStatus.OK, ""); //$NON-NLS-1$
  }

  /**
   * Given a proposed installation directory, this method validates it syntax. Notice that
   * installation directory that does not exist is considered invalid.
   * 
   * @param pInstallationDir installation directory to validate its syntax
   * 
   * @return a status object with serverity and message the reflect validation result
   */
  public static IStatus validateInstallationDirectory(String pInstallationDir)
  {
    int statusLevel = IStatus.ERROR;
    String message;

    // installation directory is not optional, it must not be empty
    if (pInstallationDir.equals(""))
    {
      return createStatus(IStatus.INFO, "Installation directory must be specified"); //$NON-NLS-1$
    }
    
    IPath path = new Path(""); //$NON-NLS-1$
    if (!path.isValidPath(pInstallationDir))
    {
      return createStatus(IStatus.ERROR, "The installation directory name contains characters that are not allowed");
    }

    /*
     * Validate if the location exists
     */
    path = new Path(pInstallationDir);
    if (!(path.toFile().exists()))
    {
      return createStatus(IStatus.ERROR, "The installation directory does not exist");
    }

    return createStatus(IStatus.OK, ""); //$NON-NLS-1$
  }
  
  private static IStatus createStatus(int pStatusLevel, String pMessage)
  {
    return new Status(pStatusLevel, ExoUIPlugin.PLUGIN_ID, 0, pMessage, null);
  }
}