/**
 * Copyright 2001-2004 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/
package org.exoplatform.services.css.scanner;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.exoplatform.services.css.scanner.CssScanner;
import org.exoplatform.services.css.scanner.event.*;
import org.exoplatform.services.css.scanner.event.DefaultScannerListener;
import org.exoplatform.services.css.scanner.event.ScannerEventImpl;
import org.exoplatform.services.css.scanner.model.*;


/**
 * @author Ove Ranheim (oranheim@yahoo.no)
 * @since 03.jul.2004 15:21:37
 */
public class CssScannerImpl implements CssScanner {

    // @.+ Always test for not-{
    static final private String RE_RULE = "@.+";

    //  .+(\{), group0=text
    // group1=cpara for group0 always test for not-@, then skip
    static final private String RE_STATEMENT = ".+(\\{)";
    //\:)*([\;])
    //  .[^\{]*\}
    static final private String RE_BLOCK = ".[^\\{]*\\}";

    // Testdata: foo:exo-content-portlet[test=fr] *#UIContentHtmlEditorForm
    // .htmlarea td, th , select {
    // [^ \f]+\p{Punct}*\p{Alpha}\]*\ *\,*
    // [^ \f]+[\p{L}&&[^\p{Lu}]]*\p{Alpha}\]*[\ *\,*]
    // .[a-zA-Z\-\.\:\[\=\]]+[ \,]*
    // .[a-zA-Z\*\#\-\.\:\[\=\]]*[ \,]* 
    static final private String RE_SELECTOR = ".[a-zA-Z\\*\\#\\-\\.\\:\\[\\=\\]]*[ \\,]*";
    
    // .(.+):(.*)\;
    // (.+):(.*)[\ ;]
    static final private String RE_PROPERTY = "(.+)\\:(.*)\\;";

    static final private int EVENT_START_RULE = 0;
    static final private int EVENT_END_RULE = 1;
    static final private int EVENT_START_SELECTOR = 2;
    static final private int EVENT_END_SELECTOR = 3;
    static final private int EVENT_START_PROPERTY = 4;
    static final private int EVENT_END_PROPERTY = 5;
    static final private int EVENT_START_COMMENT = 6;
    static final private int EVENT_END_COMMENT = 7;

    // -----------------------------------------------------------------------------------------------------------------------

    private InputStream _source;
    private List _eventListener;

    public CssScannerImpl() {
        _eventListener = new ArrayList();
        _eventListener.add(new DefaultScannerListener());
    }

    // -----------------------------------------------------------------------------------------------------------------------

    static final protected Pattern rulePattern() {
        return Pattern.compile(RE_RULE);
    }

    static final protected Pattern statementPattern() {
        return Pattern.compile(RE_STATEMENT);
    }

    static final protected Pattern blockPattern() {
        return Pattern.compile(RE_BLOCK);
    }

    static final protected Pattern selectorPattern() {
        return Pattern.compile(RE_SELECTOR);
    }

    static final protected Pattern propertyPattern() {
        return Pattern.compile(RE_PROPERTY);
    }

    // -----------------------------------------------------------------------------------------------------------------------

    public void addListener(ScannerListener listener) {
        _eventListener.add(listener);
    }

    public InputStream getSource() {
        return _source;
    }

    public void setSource(InputStream source) {
        _source = source;
    }

    protected StringBuffer getStyleSheet() throws IOException {
        StringBuffer result = new StringBuffer();
        byte[] b = new byte[8192];
        int read = 0;
        while ((read = _source.read(b, 0, 8192)) > 0) {
            result.append(new String(b, 0, read));
        }
        return result;
    }

    // -----------------------------------------------------------------------------------------------------------------------

    protected void fireEvent(ScannerEvent event, int eventType) {
        for (Iterator i = _eventListener.iterator(); i.hasNext();) {
            ScannerListener listener = (ScannerListener) i.next();
            switch (eventType) {
	            case EVENT_START_RULE:
	                listener.startRule(event);
	                break;
	            case EVENT_END_RULE:
	                listener.endRule(event);
	                break;
	            case EVENT_START_SELECTOR:
	                listener.startSelector(event);
	                break;
	            case EVENT_END_SELECTOR:
	                listener.endSelector(event);
	                break;
	            case EVENT_START_PROPERTY:
	                listener.startProperty(event);
	                break;
	            case EVENT_END_PROPERTY:
	                listener.endProperty(event);
	                break;
	            case EVENT_START_COMMENT:
	                listener.startComment(event);
	                break;
	            case EVENT_END_COMMENT:
	                listener.endComment(event);
	                break;
            }
        }
    }

    // -----------------------------------------------------------------------------------------------------------------------

    public Document parse() throws IOException {
        Document document = new Document();
        DefaultScannerListener listener = (DefaultScannerListener) _eventListener.get(0);
        listener.setDocument(document);
        StringBuffer css = getStyleSheet();
        int n = 0;
        String[] stmtSplit = statementPattern().split(css);
        Matcher stmtMatcher = statementPattern().matcher(css);
        while (stmtMatcher.find()) {
            n++;
            String block = stmtSplit[n];
            String stmt = stmtMatcher.group(0);
            
            List properties = new ArrayList();
            Matcher propertyMatcher = propertyPattern().matcher(block.replaceAll(";", ";\n"));
            while (propertyMatcher.find()) {
                String property = propertyMatcher.group(0).trim();
                int start = propertyMatcher.start(0);
                int end = propertyMatcher.end(0);
                String attribute = propertyMatcher.group(1).trim();
                String value = propertyMatcher.group(2).trim();
                
                Property p = new Property();
                p.setSequence(property);
                p.setName(attribute);
                p.setValue(value);
                p.setStart(start);
                p.setEnd(end);
                properties.add(p);
            }
            
            
            Matcher selectorMatcher = selectorPattern().matcher(stmt);
            Selector parent = null;
            int depth = 0;
            boolean hasNext = selectorMatcher.lookingAt();
            while (hasNext) {
                String selector = selectorMatcher.group(0).replaceAll(" ", "");
                int start = selectorMatcher.start(0);
                int end = selectorMatcher.end(0);
                hasNext = selectorMatcher.find();
                String nextSelector = selectorMatcher.group(0).replaceAll(" ", "");
                if ("{".equals(nextSelector)) hasNext = false;
                
                ScannerEventImpl event = new ScannerEventImpl();
                Selector current = new Selector(parent);
                if (parent != null) parent.setNext(current);
                current.setName(selector);
                current.setSequence(stmt);
                current.setStart(start);
                current.setEnd(end);
                event.setEventObject(current);
                event.setDepth(depth);
                event.setHasNext(hasNext);
                
                current.setProperties(properties);
                document.addSelector(current);
                
                fireEvent(event, EVENT_START_SELECTOR);
                
                for(Iterator i = properties.iterator(); i.hasNext(); ) {
                    Property p = (Property) i.next();
                    ScannerEventImpl pevent = new ScannerEventImpl();
                    pevent.setEventObject(p);
                    pevent.setHasNext(hasNext);
                    pevent.setDepth(depth);
                    fireEvent(pevent, EVENT_START_PROPERTY);
                    if (!pevent.hasNext()) fireEvent(pevent, EVENT_END_PROPERTY);
                }
                
                if (!hasNext) fireEvent(event, EVENT_END_SELECTOR);
                
                parent = current;
                depth++;
            }
        }
        return document;
    }

}