/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.core;

import javax.jcr.AccessDeniedException;
import javax.jcr.InvalidItemStateException;
import javax.jcr.Item;
import javax.jcr.ItemNotFoundException;
import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.ReferentialIntegrityException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.lock.LockException;
import javax.jcr.nodetype.ConstraintViolationException;
import javax.jcr.nodetype.ItemDefinition;
import javax.jcr.version.VersionException;

import org.exoplatform.services.jcr.datamodel.ItemData;
import org.exoplatform.services.jcr.impl.core.value.ValueFactoryImpl;
import org.exoplatform.services.jcr.impl.storage.TransientItemData;
import org.exoplatform.services.log.LogService;
import org.exoplatform.services.log.impl.ExoLog;

/**
 * Created by The eXo Platform SARL .
 * 
 * 
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov
 *         </a>
 * @version $Id: ItemImpl.java 3755 2006-01-12 10:42:52Z peterit $
 */

public abstract class ItemImpl implements Item {

  protected final SessionImpl session;

  /////////
  protected JCRPath location;

  protected String internalUUID;

  protected Node parent;

  protected int state;

  protected int persistedVersion;

  /////////

  protected LogService logService;

  protected ExoLog log;

  // used for lazy initialization nodes/properties/parent
  protected ItemData data;

  protected SessionDataManager dataManager;

  protected LocationFactory locationFactory;

  protected ValueFactoryImpl valueFactory;

  ItemImpl(ItemData data, NodeImpl parentNode, int state, SessionImpl session,
      LogService logService) throws RepositoryException {

    this.session = session;
    this.state = state;
    this.internalUUID = data.getUUID();
    this.persistedVersion = data.getPersistedVersion();
    
    this.location = session.getLocationFactory().createJCRPath(data.getQPath());
    this.parent = parentNode;

    this.logService = logService;
    this.log = (ExoLog) logService.getLog(this.getClass());
    log.setLevel(LogService.DEBUG);

    this.data = data;

    this.dataManager = session.getTransientNodesManager();
    this.locationFactory = session.getLocationFactory();
    this.valueFactory = (ValueFactoryImpl) session.getValueFactory();

  }
  
  void loadLocation(JCRPath location) {
    this.location = location;
  }

  /**
   * @see javax.jcr.Item#getPath
   */
  public String getPath() {
    return getLocation().getAsString(false);
  }

  /**
   * @see javax.jcr.Item#getName
   */
  public String getName() {
    return getLocation().getName().getAsString();
  }

  /**
   * @see javax.jcr.Item#getAncestor
   */
  public Item getAncestor(int degree) throws ItemNotFoundException,
      AccessDeniedException, RepositoryException {
    try {
      //6.2.8 If depth > n is specified then an ItemNotFoundException is
      // thrown.
      if (degree < 0)
        throw new ItemNotFoundException(
            "Workspace.getAncestor() ancestor's degree < 0.");
      int n = getDepth() - degree;
      //      log.debug("Item.getAncestor(" + degree + ") depth " +
      // getDepth()+" "+n);
      if (n == 0)
        return this;
      else if (n < 0)
        throw new ItemNotFoundException(
            "Workspace.getAncestor() ancestor's degree > depth of this item.");
      else {

        JCRPath ancestorLoc = getLocation().makeAncestorPath(n);
        
        return findItemByPath(ancestorLoc);
      }
    } catch (PathNotFoundException e) {
      throw new ItemNotFoundException(e.getMessage(), e);
    }
  }

  /**
   * Parent is lazy initialized
   * 
   * @see javax.jcr.Item#getParent
   */
  public Node getParent() throws ItemNotFoundException, AccessDeniedException,
      RepositoryException {
    if (isRoot())
      throw new ItemNotFoundException("Root node does not have a parent");

    if (this.parent == null) {
      //System.out.println("Parent is null for >>> "+this.getPath()+" "+dataManager.dump());
      parent = (Node) findItemByPath(getLocation().makeParentPath());
    }
    
    // it does not work for newly added, and not registered with dataManager
    // (for CopyVisitor for ex)
    // it is just hack - TODO remove it in new model impl !!
    if(parent == null) {
      parent = (Node)dataManager.getItemFromCache(this.getActualItemData().getParentUUID());
    }
    ////////////////////////////////////////////////////////
    
    if (parent == null) {
      throw new RepositoryException("Parent not found for " + this.getPath()
          + " state: " + getState()+" at workspace: "+session.getWorkspace().getName());
    }
    return parent;
  }

  /**
   * @see javax.jcr.Item#getSession
   */
  public Session getSession() {
    return session;
  }

  /**
   * @see javax.jcr.Item#getDepth
   */
  public int getDepth() {
    return getLocation().getDepth();
  }

  /**
   * @see javax.jcr.Item#isSame
   */
  public boolean isSame(Item otherItem) {
    if (otherItem == null)
      return false;

    if (!this.getClass().getName().equals(otherItem.getClass().getName()))
      return false;

    try {
      return this.getActualItemData().getUUID().equals(
          ((ItemImpl) otherItem).getActualItemData().getUUID());
    } catch (RepositoryException e) {
      log.debug("Item.isSame() failed " + e);
      return false;
    }

  }

  /**
   * @see javax.jcr.Item#isNew
   */
  public boolean isNew() {
    return (getState() == ItemChangeState.ADDED);
  }

  /**
   * @see javax.jcr.Item#isModified
   */
//  public boolean isModified() {
//    return (getState() == ItemChangeState.UPDATED);
//  }

  //    In the subclassess:
  //    public boolean isNode();
  //    public void accept(ItemVisitor visitor) throws RepositoryException;

  /**
   * @see javax.jcr.Item#remove
   */
  public void remove() throws RepositoryException,
      ConstraintViolationException, VersionException, LockException {

    doRemove(true);
  }

  void doRemove(boolean externalValidation) throws RepositoryException,
      ConstraintViolationException, VersionException, LockException {

    if (isRoot())
      throw new RepositoryException("Can't remove ROOT node.");

    // Check constraints
    if (externalValidation) {
      ItemDefinition def;
      if (isNode())
        def = ((NodeImpl) this).getDefinition();
      else
        def = ((PropertyImpl) this).getDefinition();

      if (def.isMandatory() || def.isProtected())
        throw new ConstraintViolationException(
            "Can't remove mandatory or protected item " + getPath());

    }
    // Check if versionable ancestor is not checked-in
    parent = (NodeImpl) getParent();
    if (!parent.isCheckedOut())
      throw new VersionException("Node " + parent.getPath()
          + " or its nearest ancestor is checked-in");

    // Check locking
    if (!((NodeImpl) parent).checkLocking())
      throw new LockException("Node " + parent.getPath() + " is locked ");

    // remove from datamanager
    dataManager.delete(this);       

    // remove from parent
    if (isNode()) {
      ((NodeImpl) parent).removeChildNode(this);
    } else {
      ((NodeImpl) parent).removeChildProperty(this);
    }
    
    log.debug("REMOVED "+this.getState());
  }

  /**
   * @see javax.jcr.Item#save
   */
  public void save() throws ReferentialIntegrityException,
      AccessDeniedException, LockException, ConstraintViolationException,
      InvalidItemStateException, ReferentialIntegrityException,
      RepositoryException {

    if (isNew())
      throw new RepositoryException(
          "It is impossible to call save() on the newly added item "
              + getPath());
    session.getTransientNodesManager().commit(this);
  }

  /**
   * @see javax.jcr.Item#refresh
   */
  public void refresh(boolean keepChanges) throws InvalidItemStateException,
      RepositoryException {
    if (!keepChanges)
      session.getTransientNodesManager().rollback(this);
    else
      session.getTransientNodesManager().refresh(this);
  }

  // -------------------- Implementation ----------------------

  public abstract TransientItemData getActualItemData()
      throws RepositoryException;

  public String getInternalUUID() {
    return internalUUID;
  }

  public JCRPath getLocation() {
    return this.location;
/*    
    try {
      //return this.location;
      return locationFactory.createJCRPath(data.getQPath());
    } catch (RepositoryException e) {
      // never happen
      e.printStackTrace();
      return null;
    }
*/    
  }

  public boolean isRoot() {
    return (getDepth() == 0);
  }

  public int getState() {
    return state;
  }

  /**
   * @param newState
   *          The state to set.
   */
  protected void setState(int newState) {
    // old new result
    //  A U A
    //  D U D
    //  D A A

    if ((this.state == ItemChangeState.ADDED || this.state == ItemChangeState.AUTO_ADDED)
        && newState == ItemChangeState.UPDATED)
      newState = state; // not change
    else if (this.state == ItemChangeState.DELETED || this.state == ItemChangeState.AUTO_DELETED) {
      newState = state;
    }

    this.state = newState;
  }

  abstract void loadData(ItemData data) throws RepositoryException;

  public void setPersistedVersion(int persistedVersion) {
    this.persistedVersion = persistedVersion;
  }

  public boolean hasPermission(String action) throws RepositoryException {
    NodeImpl parentNode = (NodeImpl) ((isRoot()) ? (Node) this : getParent());
    boolean hasPerm = session.getAccessManager().hasPermission(
        parentNode.getChildrenACL(), action, session.getUserID());
    //log.debug("User "+session.getUserID()+" item "+getPath()+" has permission
    // "+hasPerm);
    return hasPerm;
  }

  public LogService getLogService() {
    return logService;
  }

  int getPersistedVersion() {
    return persistedVersion;
  }

  protected Item findItemByPath(JCRPath path) throws RepositoryException {
    return dataManager.getItem(path);
  }

}