/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.core;

import java.security.AccessControlException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.jcr.AccessDeniedException;
import javax.jcr.InvalidItemStateException;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Property;
import javax.jcr.PropertyIterator;
import javax.jcr.ReferentialIntegrityException;
import javax.jcr.RepositoryException;
import javax.jcr.util.TraversingItemVisitor;

import org.apache.commons.collections.MapIterator;
import org.apache.commons.collections.map.ReferenceMap;
import org.exoplatform.services.jcr.access.PermissionType;
import org.exoplatform.services.jcr.datamodel.ItemData;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.datamodel.ValueData;
import org.exoplatform.services.jcr.impl.core.version.VersionHistoryImpl;
import org.exoplatform.services.jcr.impl.core.version.VersionImpl;
import org.exoplatform.services.log.LogService;
import org.exoplatform.services.log.impl.ExoLog;

/**
 * Created by The eXo Platform SARL .<br>
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: SessionDataManager.java 3755 2006-01-12 10:42:52Z peterit $
 */
public class SessionDataManager {

  protected ExoLog log;

  private LogService logService;

  private SessionImpl session;

  private ChangedItemsStorage changedStorage;

  // uuid -> ItemImpl
  private Map sessionCache;

  private BatcheableWorkspaceDataManager workspaceDataManager;

  public SessionDataManager(SessionImpl session,
      BatcheableWorkspaceDataManager workspaceDataManager, LogService logService)
      throws RepositoryException {
    this.session = session;
    this.logService = logService;
    // this.log = logService.getLog("org.exoplatform.services.jcr");
    this.log = (ExoLog) logService.getLog(this.getClass());
    log.setLevel(LogService.DEBUG);
// hard, soft
    this.sessionCache = new ReferenceMap(ReferenceMap.HARD, ReferenceMap.SOFT);
    this.workspaceDataManager = workspaceDataManager;
    this.changedStorage = new ChangedItemsStorage(logService);
  }

  /**
   * Finds item by absolute path in this tnsient storage then in workspace
   * container.
   * 
   * @param path
   * @return item or null if not found
   * @throws RepositoryException
   */
  public ItemImpl getItem(JCRPath path) throws RepositoryException {

    // 1. Try in transient changes
    ItemImpl item = changedStorage.get(path);

    if (item == null) {

      // 2. Try from workspacedatamanager
      ItemData data = workspaceDataManager.getItemData(path);
      if (data == null)
        return null;
      else {
        // parent is lazy loaded
        if (data instanceof NodeData)
          item = createUnchangedNode((NodeData) data);
        else
          item = new PropertyImpl((PropertyData) data, null,
              ItemChangeState.UNCHANGED, session, logService);
      }
    }

    if (item == null || item.getState() == ItemChangeState.DELETED) {
      return null;
    } else if (!item.hasPermission(PermissionType.READ)) {

      throw new AccessDeniedException("Access denied "
          + path.getAsString(false) + " for " + session.getUserID());
    } else {

      return cacheItem(item, false);

    }
  }

  /**
   * @param uuid
   * @return item by UUID or null
   * @throws RepositoryException
   */
  public ItemImpl getItemByUUID(String uuid) throws RepositoryException {

    // 1. Try in transient changes
    ItemImpl item = changedStorage.get(uuid);

    if (item == null) {

      // 2. Try from workspacedatamanager
      ItemData data = workspaceDataManager.getItemDataByUUID(uuid);
      if (data == null)
        return null;
      else {
        // parent is lazy loaded
        if (data instanceof NodeData)
          item = createUnchangedNode((NodeData) data);
        else
          item = new PropertyImpl((PropertyData) data, null,
              ItemChangeState.UNCHANGED, session, logService);
      }

    }

    
    if (item == null || item.getState() == ItemChangeState.DELETED) {
      return null;
    } else if (!item.hasPermission(PermissionType.READ)) {
      throw new AccessDeniedException("Access denied, item: " + item.getPath());
    } else {
      log.debug("SessionDataManager found item by UUID " + uuid
          + " item path: " + item.getPath());

      return cacheItem(item, false);
    }

  }

  private NodeImpl createUnchangedNode(NodeData data)
      throws RepositoryException {
    NodeImpl node = new NodeImpl(data, null, ItemChangeState.UNCHANGED,
        session, logService);
    if (node.isNodeType("nt:version"))
      return new VersionImpl(data, null, ItemChangeState.UNCHANGED, session,
          logService);
    else if (node.isNodeType("nt:versionHistory"))
      return new VersionHistoryImpl(data, null, ItemChangeState.UNCHANGED,
          session, logService);
    else
      return node;

  }

  List getReferences(String uuid) throws RepositoryException {
    return workspaceDataManager.getReferences(uuid);
  }

  /**
   * Mark the changed item as DELETED
   * 
   * @param item
   * @throws RepositoryException
   */
  synchronized void delete(ItemImpl item) throws RepositoryException {
    log.debug("NodesModificationManager is marking to DELETE " + item.getPath()
        + " " + item.getInternalUUID());

    // Child changes as well
    Iterator childChanges = changedStorage
        .getChanges(item.getLocation(), false).iterator();
    while (childChanges.hasNext()) {
      ItemImpl child = (ItemImpl) childChanges.next();

      // remove all children changes
      // if (!child.getPath().equals(item.getPath())) {
      changedStorage.remove(child);
      log.debug("NodesModificationManager removed child " + child.getPath());
      // }

    }
    item.setState(ItemChangeState.DELETED);
    changedStorage.put(item);
  }

  /**
   * Adds if not exist or update item in transient storage
   * 
   * @param item
   * @throws RepositoryException
   */
  synchronized void update(ItemImpl item) throws RepositoryException {

    if (item.getState() != ItemChangeState.ADDED
        && item.getState() != ItemChangeState.DELETED
        && item.getState() != ItemChangeState.UPDATED) {

      throw new RepositoryException("The state "
          + ItemChangeState.getStateName(item.getState()) + " is not accepted");
    }

    ItemImpl item2 = changedStorage.get(item.getInternalUUID());

    if (item2 != null) {
      if (item.isNode() != item2.isNode())
        throw new RepositoryException("There is transient item in "
            + item.getPath()
            + " with not the same type as incoming. Try to save current first ");
    }

    changedStorage.put(item);

  }

  /**
   * Stored changes (if any) for item and descendants
   * 
   * @param item
   * @throws RepositoryException
   * @throws AccessDeniedException
   * @throws ReferentialIntegrityException
   */
  synchronized void commit(ItemImpl item) throws RepositoryException,
      AccessDeniedException, ReferentialIntegrityException,
      InvalidItemStateException {
    
    // 1. validate all, throw an exception if validation failed
    validate(item);

    Iterator changes = changedStorage.getChanges(item.getLocation(), false)
        .iterator();
    
    // 2. create changes list
    List resultChanges = new ArrayList();
    
    try {
      
      workspaceDataManager.beginSessionBatch(this.session);
      
      while (changes.hasNext()) {
        ItemImpl childItem = (ItemImpl) changes.next();
        
        resultChanges.add(childItem);

        //workspaceDataManager.beginTransaction();
        
        
        if (childItem.getState() == ItemChangeState.DELETED) {

          workspaceDataManager.delete(childItem);
          //workspaceDataManager.doDelete(childItem);

          removeFromCache(childItem);
        } else if (childItem.getState() == ItemChangeState.ADDED) {

          // no need to process AUTO_ADDED as this item should containe
          // the whole tree
          workspaceDataManager.add(childItem);
          //workspaceDataManager.doAdd(childItem);
          
          // reset state of children to not to process them one more
          // in a case if they added separately i.e. has ADDED state
          childItem.accept(new ItemStateResetter());
          
          //cacheItem(childItem, true);
          
          //System.out.println("COMMIT: "+childItem.getPath()+" "+childItem.getState());

        } else if (childItem.getState() == ItemChangeState.UPDATED) {

          workspaceDataManager.update(childItem);
          //workspaceDataManager.doUpdate(childItem);
          //cacheItem(childItem, true);

        }
        
      }
    } finally {
      workspaceDataManager.endSessionBatch(this.session);
    } 
    
    // 3. post notify
    //workspaceDataManager.endSessionBatch(this.session);
    
    // 4. clean
    for(int i=0; i<resultChanges.size(); i++) {
      
      ItemImpl childItem = (ItemImpl) resultChanges.get(i);
      // remove from changes table
      changedStorage.remove(childItem);

      // reset state as unchanged
      childItem.setState(ItemChangeState.UNCHANGED);
      
    }
  }
  

  private synchronized void validate(ItemImpl item) throws RepositoryException,
      AccessDeniedException, ReferentialIntegrityException {

    Iterator changes = changedStorage.getChanges(item.getLocation(), false)
        .iterator();
    while (changes.hasNext()) {
      ItemImpl changedItem = (ItemImpl) changes.next();
      log.debug("SessionDataManager is VALIDATING " + changedItem.getPath()
          + " " + changedItem.getState() + " " + changedItem.getInternalUUID());

      // Check referencial integrity
      if ((changedItem.getState() == ItemChangeState.DELETED || changedItem
          .getState() == ItemChangeState.AUTO_DELETED)
          && changedItem.isNode()
          && ((NodeImpl) changedItem).isNodeType("mix:referenceable")) {

        log.debug("SessionDataManager is VALIDATING deleted refereceable "
            + changedItem.getPath());

        NodeImpl node = (NodeImpl) changedItem;
        List nodeRefs = getReferences(node.getUUID());
        Iterator refs = nodeRefs.iterator();

        while (refs.hasNext()) {
          PropertyData refProp = (PropertyData) refs.next();
          // String path = ((SessionImpl)item.getSession()).
          JCRPath path = session.getLocationFactory().createJCRPath(
              refProp.getQPath());

          // if ref property is deleted in this session
          int state = getItemState(path);
          if (state == ItemChangeState.DELETED
              || state == ItemChangeState.AUTO_DELETED)
            continue;

          try {
            session.checkPermission(path.getAsString(false),
                PermissionType.READ);
          } catch (AccessControlException e) {
            throw new AccessDeniedException("Can not delete node "
                + node.getPath()
                + " ("+ node.getUUID()+ ")"
                + ". It is currently the target of a REFERENCE property "
                + path+ " ("
                + new String(((ValueData) refProp.getValues().get(0))
                    .getAsByteArray()) + ")");
          }
          throw new ReferentialIntegrityException("Can not delete node "
              + node.getPath()
              + " ("
              + node.getUUID()
              + ")"
              + ". It is currently the target of a REFERENCE property "
              + path
              + " ("
              + new String(((ValueData) refProp.getValues().get(0))
                  .getAsByteArray()) + ")");
        }
      }

      // validate in workspace data manager
      workspaceDataManager.validate(changedItem);
    }

  }

  /**
   * Removes all pending changes of this item
   * 
   * @param item
   * @throws RepositoryException
   */
  synchronized void rollback(ItemImpl item) throws InvalidItemStateException, 
  RepositoryException {
    Iterator changes = changedStorage.getChanges(item.getLocation(), false)
        .iterator();

    while (changes.hasNext()) {
      ItemImpl change = (ItemImpl) changes.next();

      if (change.isNew()) { // remove from parent
        NodeImpl parent = (NodeImpl) change.getParent();
        if (change.isNode())
          parent.removeChildNode(change);
        else
          parent.removeChildProperty(change);
      } else if (change.isModified()) {               
        change.loadData(workspaceDataManager.getItemData(change.getLocation()));
      }
      
      changedStorage.remove(change);
    }

  }

  /**
   * Reloads data from persistence storage
   * 
   * @param item
   * @throws RepositoryException
   */
  synchronized void refresh(ItemImpl item) throws InvalidItemStateException, 
  RepositoryException {

    Iterator changes = changedStorage.getChanges(item.getLocation(), false)
        .iterator();

    while (changes.hasNext()) {
      ItemImpl change = (ItemImpl) changes.next();
      if (change.getState() == ItemChangeState.UNCHANGED) {
        change.loadData(workspaceDataManager.getItemData(change.getLocation()));
      }
    }
  }

  /**
   * Returns if node placed in loc or its descendants have pending changes
   * 
   * @param loc
   * @return
   */
  public boolean hasPendingChanges(JCRPath loc, boolean childrenOnly) {
    try {
      Collection items = changedStorage.getChanges(loc, childrenOnly);
      return (items.size() > 0);
    } catch (RepositoryException e) {
      return false;
    }
  }

  public boolean hasAddedOrRemovedDescendant(JCRPath loc, boolean childrenOnly) {
    try {
      Collection items = changedStorage.getChanges(loc, childrenOnly);
      for (Iterator it = items.iterator(); it.hasNext();) {
        ItemImpl item = (ItemImpl) it.next();
        if (item.getState() == ItemChangeState.ADDED
            || item.getState() == ItemChangeState.DELETED)
          return true;
      }
    } catch (RepositoryException e) {
    }
    return false;

  }

  /**
   * @param node
   * @param isNodes
   *          nodes or properties
   * @return changed children of the node
   * @throws RepositoryException
   */
  synchronized List getChildren(NodeImpl node, boolean isNode)
      throws RepositoryException, AccessDeniedException {

    log.debug("SessionDataManager Get children of node " + node.getPath()
        + " child nodes? " + isNode + " " + node.storedNodeData());
    // get items from container
    List items = new ArrayList();
    NodeData data = node.storedNodeData();
    if (isNode) {
      List nodes = data.getChildNodes();
      if (nodes == null) {
        nodes = new ArrayList();
        log.debug("SessionDataManager Get children of node " + node.getPath()
            + " nodes == null");
      }
      for (int i = 0; i < nodes.size(); i++) {
        NodeData nodeData = (NodeData) nodes.get(i);

        NodeImpl n = null;
        n = (NodeImpl) getItemByUUID(nodeData.getUUID());
        if (n != null) {
          log.debug("SessionDataManager Get child node " + n.getPath());
          items.add(n);
        } else {
          log.warn("CHILD NODE is null " + nodeData.getQPath().getAsString()
              + " " + nodeData.getUUID());
        }

      }
    } else {
      List props = data.getChildProperties();
      for (int i = 0; i < props.size(); i++) {
        PropertyData propData = (PropertyData) props.get(i);
        PropertyImpl prop = null;
        prop = (PropertyImpl) getItemByUUID(propData.getUUID());

        if (prop != null)
          items.add(prop);
        else
          log.debug("CHILD PROPERTY is null "
              + propData.getQPath().getAsString());

      }
    }

    // add ADDED from transient storage ????

    Iterator entries = changedStorage.getChanges().iterator();
    while (entries.hasNext()) {
      ItemImpl item = (ItemImpl) entries.next();
      JCRPath testLocation = item.getLocation();
      if (testLocation.isDescendantOf(node.getLocation(), true)) {
        if (item != null && item.isNode() == isNode && item.isNew()
            && !items.contains(item))
          items.add(item);

      }
    }

    // TODO
    for (int i = 0; i < items.size(); i++) {
      if (((ItemImpl) items.get(i)).getPath().equals(node.getPath()))
        throw new RepositoryException(
            "FATAL: internal error child node path equals parent "
                + node.getPath() + " " + node.getState());

    }

    return items;
  }
  
  // TEMPORARY for ItemImpl.findParent hack!!!
  ItemImpl getItemFromCache(String id) {
    return (ItemImpl)sessionCache.get(id);
  }

  /**
   * Returns state of item in path in session. It includes finding item itself
   * and its ancestor in transient storage In a case if closest ancestor has ADD
   * or DELETE state - this item has it as well.
   * 
   * @param path
   * @return
   */
  int getItemState(JCRPath path) throws RepositoryException {
    // JCRPath path = session.getLocationFactory().createJCRPath(qpath);
    // System.out.println(">>>> getItemState >>>> "+path.getAsString(false)+"
    // "+path.getDepth());

    for (int i = path.getDepth(); i >= 0; i--) {
      ItemImpl item = changedStorage.get(path.makeAncestorPath(i));
      if (item != null) {
        int state = item.getState();
        if (i == 0 || state == ItemChangeState.ADDED
            || state == ItemChangeState.AUTO_ADDED
            || state == ItemChangeState.DELETED
            || state == ItemChangeState.AUTO_DELETED) {
          // System.out.println(">>>> getItemState >>>>
          // "+path.getAsString(false)+" "+item.getPath()+" "+state);
          return state;
        }
      }
    }
    // System.out.println(">>>> getItemState >>>> "+path.getAsString(false)+"
    // unchanged");

    return ItemChangeState.UNCHANGED;
  }

  /**
   * insert item into session cache (if not found)
   * 
   * @param item
   * @param reloadState
   *          if true item will be reloaded with new state
   * @return inserted item
   * @throws RepositoryException
   */
  ItemImpl cacheItem(ItemImpl item, boolean reloadState)
      throws RepositoryException {

    String uuid = new String(item.getInternalUUID());

    if (sessionCache.keySet().contains(uuid)) {
      // get value stored in session cache

      ItemImpl storedItem = (ItemImpl) sessionCache.get(uuid);

      if (!reloadState) {
        return storedItem;
      } else {

        storedItem.loadData(item.getActualItemData());
        storedItem.setState(item.getState());

        // location have to reloaded as well
        //if (!storedItem.getLocation().equals(item.getLocation())) {
        //  System.out.println("SESSION CACHE same item "+uuid+" "+item.getPath()+" "+storedItem.getPath());
        //  storedItem.loadLocation(item.getLocation());
        //}
        item = storedItem;
      }

    }

    // put item to session cache
    sessionCache.put(uuid, item);
    //System.out.println("SESSION CACHE put "+uuid+" "+item.getPath()+"
    // "+item);

    if (item.isNode()) {
      NodeImpl node = (NodeImpl) item;
      if (node.arePropertiesInitialized()) {
        PropertyIterator props = node.getProperties();
        while (props.hasNext())
          cacheItem((PropertyImpl) props.nextProperty(), reloadState);
      }

      if (node.areNodesInitialized()) {
        NodeIterator nodes = node.getNodes();
        while (nodes.hasNext())
          cacheItem((NodeImpl) nodes.nextNode(), reloadState);
      }
    }

    return item;
  }

  private void removeFromCache(ItemImpl item) {
    // Remove from ReferenceMap, doing it instead GC  
    if (sessionCache instanceof ReferenceMap) {
      ReferenceMap cache = (ReferenceMap) sessionCache;
      
      MapIterator casheIterator = cache.mapIterator();
      while (casheIterator.hasNext()) {
        String cUuid = (String) casheIterator.next();
        if (item.getInternalUUID().equals(cUuid)) {
          // ItemImpl cItem = (ItemImpl) casheIterator.getValue();
          casheIterator.remove();
          return;
        }
      }
    }
  }


  /**
   * @return Returns the workspDataManager.
   */
  public BatcheableWorkspaceDataManager getWorkspaceDataManager() {
    return workspaceDataManager;
  }

  public String dump() {
    String d = "\nChanges:";
    for (Iterator changes = changedStorage.getChanges().iterator(); changes
        .hasNext();) {
      ItemImpl item = (ItemImpl) changes.next();
      d += "\nItemImpl: isNode:" + item.isNode() + " uuid: "
          + item.getInternalUUID() + " path:" + item.getPath() + " state:"
          + ItemChangeState.getStateName(item.getState());
    }

    d += "\nCache:";
    for (Iterator cache = sessionCache.values().iterator(); cache.hasNext();) {
      ItemImpl item = (ItemImpl) cache.next();
      d += "\nItemImpl: isNode:" + item.isNode() + " uuid: "
          + item.getInternalUUID() + " path:" + item.getPath() + " state:"
          + ItemChangeState.getStateName(item.getState());
    }

    return d;
  }

  private class ItemStateResetter extends TraversingItemVisitor.Default {

    protected void entering(Node node, int level) throws RepositoryException {
      NodeImpl nodeImpl = (NodeImpl)node; 
      nodeImpl.setState(ItemChangeState.UNCHANGED);
      //System.out.println("RESET>>> "+sessionCache.get(nodeImpl.getInternalUUID())+" "+nodeImpl);
    }

    protected void entering(Property property, int level)
        throws RepositoryException {
      ((ItemImpl) property).setState(ItemChangeState.UNCHANGED);
    }

  }
  
  public boolean isCacheEnabled() {
    return workspaceDataManager.isCacheEnabled();
  }

  public void setCacheEnabled(boolean enabled) {
    workspaceDataManager.setCacheEnabled(enabled);
  }  

  public void clearCache() throws RepositoryException {
    workspaceDataManager.clearCache();
  }
  
}
