/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.core;

import java.util.List;

import javax.jcr.InvalidItemStateException;
import javax.jcr.Node;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.ValueFactory;

import org.exoplatform.services.jcr.datamodel.IllegalPathException;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.ItemData;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.impl.Constants;
import org.exoplatform.services.jcr.impl.core.version.VersionHistoryImpl;
import org.exoplatform.services.jcr.impl.storage.TransientNodeData;
import org.exoplatform.services.jcr.impl.storage.cache.WorkspaceCache;
import org.exoplatform.services.jcr.storage.WorkspaceDataContainer;
import org.exoplatform.services.log.LogService;

/**
 * Created by The eXo Platform SARL        .
 * 
 * Responsible for: 
 * *redirecting repository operations if item is descendant of
 * /jcr:system/jcr:versionStorage
 * *adding version history for newly added/assigned mix:versionable 
 * 
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: VersionableWorkspaceDataManager.java 3547 2006-01-03 08:42:56Z geaz $
 */

public class VersionableWorkspaceDataManager extends WorkspaceDataManager {

  private WorkspaceDataManager versionDataManager;
  
  private InternalQPath verStoragePath;
  private InternalQPath systemPath;


  public VersionableWorkspaceDataManager(WorkspaceDataContainer dataContainer,
      WorkspaceCache cache, LogService logService) {
    super(dataContainer, cache, logService);
    try {
      this.verStoragePath = InternalQPath.parse(Constants.JCR_VERSION_STORAGE_URI);
      this.systemPath = InternalQPath.parse(Constants.JCR_SYSTEM_URI);
      
    } catch (IllegalPathException e) {
      // newer thrown
      e.printStackTrace();
    }
    
    //this.addItemPersistenceListener(new VersionStorageManager(logService));
  }
  
  // called by WorkspaceContainer after repository initialization
  public void setSystemSession(SessionImpl systemSession) {

    this.versionDataManager = systemSession.getTransientNodesManager().
    getWorkspaceDataManager();
  }
  
  /*
   * @see org.exoplatform.services.jcr.impl.core.WorkspaceDataManager#getItemData(org.exoplatform.services.jcr.impl.core.JCRPath)
   */
  public ItemData getItemData(JCRPath path) throws RepositoryException {
    if(isVSDescendant(path)&& !this.equals(versionDataManager))
      return versionDataManager.getItemData(path);
    else
      return super.getItemData(path);
  }
  /*
   * @see org.exoplatform.services.jcr.impl.core.WorkspaceDataManager#getItemDataByUUID(java.lang.String)
   */
  public ItemData getItemDataByUUID(String uuid) throws RepositoryException {
    ItemData data = super.getItemDataByUUID(uuid);
    if(data != null)
      return data;
    else if(!this.equals(versionDataManager)) { 
      // try from version storage if not the same
      data = versionDataManager.getItemDataByUUID(uuid);
      if(data != null && isVSDescendant(data.getQPath()))
        return data;
    } 
    return null;
  }
  /*
   * @see org.exoplatform.services.jcr.impl.core.WorkspaceDataManager#getReferences(java.lang.String)
   */
  public List getReferences(String uuid) throws RepositoryException {
    // TODO
    return super.getReferences(uuid);
  }
  
  /* 
   * @see org.exoplatform.services.jcr.impl.core.WorkspaceDataManager#update(org.exoplatform.services.jcr.impl.core.ItemImpl)
   */
  public synchronized void update(ItemImpl item) throws RepositoryException,
      InvalidItemStateException {
    if(isVSDescendant(item.getLocation()) && !this.equals(versionDataManager))
      versionDataManager.update(item);
    else      
      super.update(item);
  }

  /* 
   * @see org.exoplatform.services.jcr.impl.core.WorkspaceDataManager#add(org.exoplatform.services.jcr.impl.core.ItemImpl)
   */
  public synchronized void add(ItemImpl item) throws RepositoryException,
      InvalidItemStateException {
    
    // add item to corresponding storage (this or version)
    if(isVSDescendant(item.getLocation()) && !this.equals(versionDataManager))
      versionDataManager.add(item);
    else      
      super.add(item);
    
    //System.out.println(">>> Versionable "+this.getDataContainer().getName()+" "+item.getPath()+" "+((NodeImpl)item).isNodeType("mix:versionable"));
    if (item.isNode() && ((NodeImpl)item).isNodeType("mix:versionable")) {
      
      addVersionHistory((NodeImpl)item);
      
    } else if (!item.isNode() && item.getName().equals("jcr:versionHistory")) {

      // mix:versionable seems to be assigned
      String parentUuid = item.getActualItemData().getParentUUID(); 
      addVersionHistory((NodeImpl)item.getSession().getNodeByUUID(parentUuid));
    }

  }
  
  /* 
   * @see org.exoplatform.services.jcr.impl.core.WorkspaceDataManager#delete(org.exoplatform.services.jcr.impl.core.ItemImpl)
   */
  public synchronized void delete(ItemImpl item) throws RepositoryException,
      InvalidItemStateException {
    if(isVSDescendant(item.getLocation()) && !this.equals(versionDataManager))
      versionDataManager.delete(item);
    else      
      super.delete(item);
  }

  //private WorkspaceDataManager dataManager(JCRPath path) throws RepositoryException {
  //  if(isVSDescendant(path) && !this.equals(versionDataManager))
  //    return versionDataManager;
  //  else      
  //    return this;
  //}
  
  private boolean isVSDescendant(JCRPath path) throws RepositoryException {
    return path.getAsString(false).equals("/jcr:system") || 
    path.getInternalPath().isDescendantOf(systemPath, false);

//    return path.getAsString(false).equals("/jcr:system/jcr:versionStorage") || 
//    path.getInternalPath().isDescendantOf(verStoragePath, false) 
//    ;
  }
  
  private boolean isVSDescendant(InternalQPath path) throws RepositoryException {
    return path.getAsString().equals(Constants.JCR_SYSTEM_URI) || 
    path.isDescendantOf(systemPath, false);
  }

  
  /**
   * Initializes version history for mix:versionable node
   * 
   * @param versionableNode
   * @throws RepositoryException
   */
  private void addVersionHistory(NodeImpl versionableNode) throws RepositoryException {
    
    
    if(versionDataManager.getItemDataByUUID(versionableNode.
        getProperty("jcr:versionHistory").getString()) != null)
        return;
    
    VersionHistoryImpl versionHistory = createVersionHistory(versionableNode);

    SessionImpl sess = (SessionImpl)versionableNode.getSession();
    
    ValueFactory valueFactory = sess.getValueFactory();

    versionHistory.createChildProperty("jcr:primaryType",
        new Value[] { valueFactory.createValue("nt:versionHistory",
            PropertyType.NAME) }, PropertyType.NAME);
    versionHistory.createChildProperty("jcr:uuid", new Value[] { valueFactory
        .createValue(versionableNode.getProperty("jcr:versionHistory").getString()) },
        PropertyType.STRING);
    versionHistory.createChildProperty("jcr:versionableUuid",
        new Value[] { valueFactory.createValue(versionableNode) }, PropertyType.REFERENCE);

    // Version labels
    NodeImpl versionLabels = versionHistory.createChildNode(
        "jcr:versionLabels", "nt:versionLabels", true);
    
    versionHistory.addRootVersion(versionableNode);

    // Save
    versionDataManager.add(versionHistory);

  }
  
  private VersionHistoryImpl createVersionHistory(Node versionableNode)
      throws RepositoryException {
    
    SessionImpl sess = (SessionImpl)versionableNode.getSession();
    NodeImpl versionStorage = versionStorageNode(sess);

    // Create new NodeData inmemory
    JCRPath path = sess.getLocationFactory().createJCRPath(versionStorage.getLocation(),
        versionableNode.getUUID());
    String uuid = versionableNode.getProperty("jcr:versionHistory").getString();
    int orderNum = 0;
    NodeData nodeData = new TransientNodeData(path.getInternalPath(), uuid, -1,
        Constants.NT_VERSION_HISTORY, new InternalQName[0], orderNum,
        versionStorage.getInternalUUID());
    
    VersionHistoryImpl versionHistory = new VersionHistoryImpl(nodeData,
        versionStorage, versionStorage.getState(),
        (SessionImpl) versionableNode.getSession(), logService);
    
    return versionHistory;
  }
  
  
  private NodeImpl versionStorageNode(SessionImpl session) throws RepositoryException {
    
    JCRPath path = session.getLocationFactory().parseAbsPath("/jcr:system/jcr:versionStorage");
    NodeData data = (NodeData) getItemData(path);
    return new NodeImpl(data, null, ItemChangeState.UNCHANGED, session, logService);
  }
}
