/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.core;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.jcr.AccessDeniedException;
import javax.jcr.InvalidSerializedDataException;
import javax.jcr.ItemExistsException;
import javax.jcr.ItemNotFoundException;
import javax.jcr.NamespaceRegistry;
import javax.jcr.NoSuchWorkspaceException;
import javax.jcr.PathNotFoundException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.UnsupportedRepositoryOperationException;
import javax.jcr.Workspace;
import javax.jcr.lock.LockException;
import javax.jcr.nodetype.ConstraintViolationException;
import javax.jcr.nodetype.NodeTypeManager;
import javax.jcr.observation.ObservationManager;
import javax.jcr.query.QueryManager;
import javax.jcr.version.Version;
import javax.jcr.version.VersionException;

import org.apache.commons.logging.Log;
import org.exoplatform.container.ExoContainer;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.impl.core.itemvisitors.NodeCloneVisitor;
import org.exoplatform.services.jcr.impl.core.itemvisitors.NodeCopyVisitor;
import org.exoplatform.services.jcr.impl.core.query.QueryManagerFactory;
import org.exoplatform.services.jcr.impl.core.version.VersionImpl;
import org.exoplatform.services.jcr.impl.core.version.VersionStorageManager;
import org.exoplatform.services.log.LogService;
import org.xml.sax.ContentHandler;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: WorkspaceImpl.java 3698 2006-01-10 18:00:22Z geaz $
 */

public class WorkspaceImpl implements Workspace {

  protected Log log;

  private SessionImpl session;

  private ExoContainer container;

  private BatcheableWorkspaceDataManager dataManager;
  
  private ObservationManager observationManager;
  
  private String name;

  
  public WorkspaceImpl(String name, ExoContainer container, SessionImpl session, 
      ObservationManager observationManager)
   throws RepositoryException {

    LogService logService = (LogService) container
        .getComponentInstanceOfType(LogService.class);

    log = logService.getLog("org.exoplatform.services.jcr");
    this.session = session;
    this.container = container;
    this.dataManager = (BatcheableWorkspaceDataManager) container
        .getComponentInstanceOfType(BatcheableWorkspaceDataManager.class);
    this.name = name;
    this.observationManager = observationManager;
  }
  
  /**
   * @see javax.jcr.Workspace#getName
   */
  public String getName() {
    return name;
  }

  /**
   * @see javax.jcr.Workspace#importXML TODO (the same as Session.importXML - ?)
   */
  public ContentHandler getImportContentHandler(String parentAbsPath,
      int uuidBehavior) throws PathNotFoundException,
      ConstraintViolationException, VersionException, RepositoryException {
    return session.getImportContentHandler(parentAbsPath, uuidBehavior);
  }

  /**
   * @see javax.jcr.Workspace#importXML TODO (the same as Session.importXML - ?)
   */
  public void importXML(String parentAbsPath, InputStream in, int uuidBehavior)
      throws IOException, PathNotFoundException, ItemExistsException,
      ConstraintViolationException, InvalidSerializedDataException,
      RepositoryException {
    
    session.importXML(parentAbsPath, in, uuidBehavior);
    session.save();
  }

  /**
   * @see javax.jcr.Workspace#getSession
   */
  public Session getSession() {
    return session;
  }

  /**
   * @see javax.jcr.Workspace#getAccessibleWorkspaceNames
   */
  public String[] getAccessibleWorkspaceNames() throws RepositoryException {
    RepositoryImpl rep = (RepositoryImpl) session.getRepository();
    return rep.getWorkspaceNames();
  }

  /**
   * @see javax.jcr.Workspace#copy
   */
  public void copy(String srcWorkspace, String srcPath, String destPath)
      throws NoSuchWorkspaceException, ConstraintViolationException,
      VersionException, AccessDeniedException, PathNotFoundException,
      ItemExistsException, RepositoryException {

    dataManager.beginSessionBatch(this.session);
    copyNode(srcWorkspace, srcPath, destPath, false);
    dataManager.endSessionBatch(this.session);
  }

  /**
   * @see javax.jcr.Workspace#copy
   */
  public void copy(String srcPath, String destPath) throws ItemExistsException,
      VersionException, PathNotFoundException, ItemExistsException,
      ConstraintViolationException, RepositoryException {

    dataManager.beginSessionBatch(this.session);
    copyNode(this.name, srcPath, destPath, false);
    dataManager.endSessionBatch(this.session);
  }

  /**
   * @see javax.jcr.Workspace#move
   */
  public void move(String srcPath, String destPath)
      throws ConstraintViolationException, VersionException,
      AccessDeniedException, PathNotFoundException, ItemExistsException,
      RepositoryException {

    NodeImpl srcNode = (NodeImpl) getSession().getItem(srcPath);

    dataManager.beginSessionBatch(this.session);
    copyNode(this.name, srcPath, destPath, false);
    dataManager.delete(srcNode);
    dataManager.endSessionBatch(this.session);

  }
  
  // TODO: keepUUIDs does not work correctly. do not set "true" 
  private void copyNode(String srcWorkspace, String srcPath, String destPath, boolean keepUUIDs)
  throws NoSuchWorkspaceException, ConstraintViolationException,
  VersionException, AccessDeniedException, PathNotFoundException,
  ItemExistsException, RepositoryException {
    
    SessionImpl srcSession = (SessionImpl) ((RepositoryImpl) session
        .getRepository()).login(session.getCredentials(), srcWorkspace);
    

    NodeImpl srcNode = (NodeImpl) srcSession.getItem(srcPath);

    // make dest node
    JCRPath destNodePath = session.getLocationFactory().parseAbsPath(destPath); 
    if(destNodePath.isIndexSetExplicitly())
      throw new RepositoryException("DestPath should not contain an index "+destPath);

    String parentPath = destNodePath.makeParentPath().getAsString(false);
    NodeImpl parentNode = (NodeImpl) getSession().getItem(parentPath);
    
    // Check if versionable ancestor is not checked-in
    if (!parentNode.isCheckedOut())
      throw new VersionException("Node " + parentNode.getPath()
          + " or its nearest ancestor is checked-in");

    
    // Check locking
    if(!parentNode.checkLocking())
      throw new LockException("Node "+ parentNode.getPath()+" is locked ");

    
    NodeImpl destNode = parentNode.createChildNode(
        destNodePath.getName().getAsString(), 
        srcNode.getPrimaryNodeType().getName(), 
        srcNode.getMixinTypeNames(), true);

    NodeCopyVisitor initializer = new NodeCopyVisitor(destNode, keepUUIDs);
    srcNode.accept(initializer);
    
    dataManager.validate(destNode);
    
    dataManager.add(destNode);
    
  }

  /**
   * @see javax.jcr.Workspace#clone
   */
  public void clone(String srcWorkspace, String srcPath, String destPath,
      boolean removeExisting) throws NoSuchWorkspaceException,
      ConstraintViolationException, VersionException, AccessDeniedException,
      PathNotFoundException, ItemExistsException, RepositoryException {

    // find src node
    //SessionImpl srcSession = (SessionImpl) ((RepositoryImpl) session
    //    .getRepository()).getSystemSession(srcWorkspace);
    
    SessionImpl srcSession = (SessionImpl) ((RepositoryImpl) session
        .getRepository()).login(session.getCredentials(), srcWorkspace);

    NodeImpl srcNode = (NodeImpl) srcSession.getItem(srcPath);
    
    // make dest node
    JCRPath destNodePath = session.getLocationFactory().parseAbsPath(destPath);
    if(destNodePath.isIndexSetExplicitly())
      throw new RepositoryException("DestPath should not contain an index "+destPath);
    String parentPath = destNodePath.makeParentPath().getAsString(false);
    NodeImpl parentNode = (NodeImpl) getSession().getItem(parentPath);
    
    // Check if versionable ancestor is not checked-in
    if (!parentNode.isCheckedOut())
      throw new VersionException("Node " + parentNode.getPath()
          + " or its nearest ancestor is checked-in");
    
    // Check locking
    if(!parentNode.checkLocking())
      throw new LockException("Node "+ parentNode.getPath()+" is locked ");

    //System.out.println("Root node to clone: "+session.getWorkspace().getName()+" "+this.session.getTransientNodesManager().dump());

    
    //if(parentNode.isNew())
    //  throw new PathNotFoundException("Destination parent node "+parentPath+" exists but is new in "+name);
    String uuid = null;
    List nodesToRemove = new ArrayList();
    if(srcNode.isNodeType("mix:referenceable")) {
      uuid = srcNode.getUUID();
      // check root
      try {
        NodeImpl relNode = (NodeImpl)session.getNodeByUUID(uuid);
        if(!removeExisting) {
          throw new ItemExistsException(
              "Item exists uuid = " + uuid + " at "
                  + session.getWorkspace().getName()+" path: "+relNode.getPath());
        } else {
          nodesToRemove.add(relNode);
        }
      } catch (ItemNotFoundException e) { // save UUID (i.e. clone)
      }

    } //else
      //uuid = session.getUuidGenerator().generateStringUUID(srcPath);


    NodeImpl destNode = parentNode.copyChildNodeInmemory(
        destNodePath.getInternalPath().getName(),
        (NodeData)srcNode.getActualItemData(), 
        srcNode.isNodeType("mix:referenceable"),
        false, false, false, false);
     
//    System.out.println("Root node to clone: "+destNode.getPath());

    
    // initialize dest node's children 
    NodeCloneVisitor initializer = new NodeCloneVisitor(destNode, removeExisting, nodesToRemove);
    srcNode.accept(initializer);

    dataManager.beginSessionBatch(session);
    for(Iterator it = nodesToRemove.iterator(); it.hasNext();) {
      NodeImpl node = (NodeImpl)it.next();
      // TODO it is possible the node is already removed along with removed parent
      node.remove();
      node.save();
    }
    
    dataManager.validate(destNode);
    dataManager.add(destNode);
    dataManager.endSessionBatch(session);
  }

  /**
   * @see javax.jcr.Workspace#getQueryManager
   */
  public QueryManager getQueryManager() throws RepositoryException {
    QueryManagerFactory qf = (QueryManagerFactory) container
    .getComponentInstanceOfType(QueryManagerFactory.class);

    if(qf == null)
      throw new RepositoryException("Query Manager Factory not found. Check configuration.");
    else
      return qf.getQueryManager(session);
  }

  /**
   * @see javax.jcr.Workspace#getNodeTypeManager
   */
  public NodeTypeManager getNodeTypeManager() throws RepositoryException {
    return (NodeTypeManager) container
        .getComponentInstanceOfType(NodeTypeManager.class);
  }

  /**
   * @see javax.jcr.Workspace#getNamespaceRegistry
   */
  public NamespaceRegistry getNamespaceRegistry() throws RepositoryException {
    return (NamespaceRegistry) container
        .getComponentInstanceOfType(NamespaceRegistry.class);
  }

  /**
   * @see javax.jcr.Workspace#getObservationManager
   */
  public ObservationManager getObservationManager()
      throws UnsupportedRepositoryOperationException, RepositoryException {
    return observationManager;
  }

  /**
   * @see javax.jcr.Workspace#restore
   */
  public void restore(Version[] versions, boolean removeExisting)
      throws UnsupportedRepositoryOperationException, VersionException,
      RepositoryException {
    
    // check if there are existed node
    List notExisted = new ArrayList();
    List existed = new ArrayList();
    for(int i=0; i<versions.length; i++) {
      String versionableUuid = versions[i].getContainingHistory().getVersionableUUID();
      NodeImpl node = (NodeImpl)session.getNodeByUUID(versionableUuid);
      if(node.isNew())
        notExisted.add(node.getLocation());
      else {
        existed.add(node.getLocation());
      }
    }
    recognizeExistense(notExisted, existed);
    if(notExisted.size() >0)
      throw new VersionException("There are not existed nodes with not existed ancestors in set "+notExisted.get(0));
    
    
    for(int i=0; i<versions.length; i++) {
      String versionableUuid = versions[i].getContainingHistory().getVersionableUUID();
      NodeImpl node = (NodeImpl)session.getNodeByUUID(versionableUuid);
      ((VersionImpl)versions[i]).restore(node, removeExisting);
    }
  }
  
  private void recognizeExistense(List notExisted, List existed) {
    int cnt = notExisted.size(); // should be enough to determine 
    for (int i = 0; i < cnt; i++) {
      for (int n = 0; n < notExisted.size(); n++) {
        JCRPath neLoc = (JCRPath) notExisted.get(n);
        for (int e = 0; e < existed.size(); e++) {
          JCRPath eLoc = (JCRPath) existed.get(e);
          if (neLoc.isDescendantOf(eLoc, true)) {
            notExisted.remove(n);
            existed.add(neLoc);
          }
        }
      }
    }
  }
  
  public VersionStorageManager getVersionStorageManager() {
    return (VersionStorageManager) container
        .getComponentInstanceOfType(VersionStorageManager.class);
  }


}