/*
 * Copyright 2004-2005 The Apache Software Foundation or its licensors,
 *                     as applicable.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.exoplatform.services.jcr.impl.core.query;

import java.io.IOException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.jcr.NamespaceException;
import javax.jcr.NamespaceRegistry;
import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.query.InvalidQueryException;
import javax.jcr.query.Query;

import org.apache.log4j.Logger;
import org.exoplatform.services.jcr.config.WorkspaceEntry;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.impl.core.ItemChangeState;
import org.exoplatform.services.jcr.impl.core.ItemState;
import org.exoplatform.services.jcr.impl.core.ItemsPersistenceListener;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.core.WorkspaceDataManager;
import org.picocontainer.Startable;

import sun.management.FileSystem;

/**
 * Acts as a global entry point to execute queries and index nodes.
 */
public class SearchManager implements ItemsPersistenceListener, Startable {

  /**
   * Logger instance for this class
   */
  private static final Logger log = Logger.getLogger(SearchManager.class);

  /**
   * Namespace URI for xpath functions
   */
  // @todo this is not final! What should we use?
  private static final String NS_FN_PREFIX = "fn";

  public static final String NS_FN_URI = "http://www.w3.org/2004/10/xpath-functions";

  /**
   * Namespace URI for XML schema
   */
  private static final String NS_XS_PREFIX = "xs";

  public static final String NS_XS_URI = "http://www.w3.org/2001/XMLSchema";

  /**
   * QueryHandler where query execution is delegated to
   */
  private final QueryHandler handler;

  private final WorkspaceDataManager wsDataManager;

  /**
   * Creates a new <code>SearchManager</code>.
   * @param session the system session.
   * @param config the search configuration.
   * @param ntReg the node type registry.
   * @param itemMgr the shared item state manager.
   * @throws RepositoryException
   */
  public SearchManager(SessionImpl session, QueryHandler handler,
      WorkspaceEntry wsInfo) throws RepositoryException {

    this.wsDataManager = session.getTransientNodesManager()
        .getWorkspaceDataManager();

    // register namespaces
    NamespaceRegistry nsReg = session.getWorkspace().getNamespaceRegistry();
    try {
      nsReg.getPrefix(NS_XS_URI);
    } catch (NamespaceException e) {
      // not yet known
      nsReg.registerNamespace(NS_XS_PREFIX, NS_XS_URI);
    }
    try {
      nsReg.getPrefix(NS_FN_URI);
    } catch (RepositoryException e) {
      // not yet known
      nsReg.registerNamespace(NS_FN_PREFIX, NS_FN_URI);
    }

    this.handler = handler;

    wsDataManager.addItemPersistenceListener(this);
    //System.out.println("Listener ADDED ");

    // set properties

  }

  /**
   * Closes this <code>SearchManager</code> and also closes the
   * {@link FileSystem} configured in {@link SearchConfig}.
   */
  public void close() {
    try {
      handler.close();
      //            if (fs != null) {
      //                fs.close();
      //           }
    } catch (IOException e) {
      log.error("Exception closing QueryHandler.", e);
    }
  }

  /**
   * Creates a query object that can be executed on the workspace.
   *
   * @param session   the session of the user executing the query.
   * @param itemMgr   the item manager of the user executing the query. Needed
   *                  to return <code>Node</code> instances in the result set.
   * @param statement the actual query statement.
   * @param language  the syntax of the query statement.
   * @return a <code>Query</code> instance to execute.
   * @throws InvalidQueryException if the query is malformed or the
   *                               <code>language</code> is unknown.
   * @throws RepositoryException   if any other error occurs.
   */
  public Query createQuery(SessionImpl session, String statement,
      String language) throws InvalidQueryException, RepositoryException {
    return new QueryImpl(session, handler, statement, language);
  }

  /**
   * Creates a query object from a node that can be executed on the workspace.
   *
   * @param session the session of the user executing the query.
   * @param itemMgr the item manager of the user executing the query. Needed
   *                to return <code>Node</code> instances in the result set.
   * @param node a node of type nt:query.
   * @return a <code>Query</code> instance to execute.
   * @throws InvalidQueryException if <code>absPath</code> is not a valid
   *                               persisted query (that is, a node of type nt:query)
   * @throws RepositoryException   if any other error occurs.
   */
  public Query createQuery(SessionImpl session, Node node)
      throws InvalidQueryException, RepositoryException {
    return new QueryImpl(session, handler, node);
  }

  ////// ------------------------- Startable -------------------------

  public void start() {
    //    wsDataManager.addItemPersistenceListener(this);
  }

  public void stop() {
    close();
  }

  //--------------- ItemPersistenceListener ----------------------------
  public void onBatchSaveItems(List itemStates, SessionImpl session) {
  }
  
  public void onSaveItems(List itemStates, SessionImpl session) {

    // nodes that need to be removed from the index.
    Set removedNodes = new HashSet();
    // nodes that need to be added to the index.
    Set addedNodes = new HashSet();

    Iterator items = itemStates.iterator();
    while (items.hasNext()) {
      ItemState itemState = (ItemState) items.next();

      //if(itemImpl.getPath().startsWith("/jcr:system"))
      //  continue;

      if (itemState.isNode()) {
        //System.out.println(">>>> Add node "+itemState.getData().getQPath().getAsString());
        if (itemState.getState() == ItemChangeState.ADDED
            || itemState.getState() == ItemChangeState.AUTO_ADDED) {
          addedNodes.add(itemState.getData());
          //traverseAddedNode((NodeData)itemState.getData(), addedNodes);
        } else if (itemState.getState() == ItemChangeState.DELETED
            || itemState.getState() == ItemChangeState.AUTO_DELETED) {
          removedNodes.add(itemState.getData().getUUID());
        }
      } else {
        //System.out.println(">>>> Add Property "+itemState.getData().getQPath().getAsString()+" parent "+itemState.getData().getParentUUID());
        String parentUUID = itemState.getData().getParentUUID();
        
        if (itemState.getState() == ItemChangeState.ADDED) {
          removedNodes.add(parentUUID); // ????????
        } else if(itemState.getState() == ItemChangeState.DELETED
            || itemState.getState() == ItemChangeState.UPDATED){
          try {
            addedNodes.add(wsDataManager.getItemDataByUUID(parentUUID));
          } catch (RepositoryException e) {
            log.error("Error indexing node.", e);
          }
          removedNodes.add(parentUUID);
        }
        
      }
    }
    try {
      handler.updateNodes(removedNodes.iterator(), addedNodes.iterator());
    } catch (RepositoryException e) {
      log.error("Error indexing node.", e);
    } catch (IOException e) {
      log.error("Error indexing node.", e);
    } catch (Throwable e) {
      log.error("Error indexing node.", e);
    }
  }
/*    
   private void traverseAddedNode(NodeData node, Set addedNodes) {
   addedNodes.add(node);
   try {
   List nodes = node.getChildNodes();
   for (int i = 0; i < nodes.size(); i++) {
   traverseAddedNode((NodeData)nodes.get(i), addedNodes);
   }
   } catch (RepositoryException e) {
   log.error("Error adding index for children of "+node.getQPath().getAsString()+" "+e);
   }
   }
*/   
}